/*****************************************************************************
*
* Motorola Inc.
* (c) Copyright 2002 Motorola, Inc.
* ALL RIGHTS RESERVED.
*
******************************************************************************
*
* File Name: adc.h
*
* Description: ADC driver macros and definitions
*
* Modules Included: adctarget.h
*
*****************************************************************************/

#ifndef __ADC_H
#define __ADC_H

#if !defined(__ARCH_H) || !defined(__PERIPH_H)
#error arch.h and periph.h must be included before adc.h
#endif

#ifdef __cplusplus
extern "C" {
#endif

/************************************************************
* ADC module identifiers (see also adctarget.h)
*************************************************************/

#define ADC_A  (&ArchIO.AdcA)
#define ADC_B  (&ArchIO.AdcB)

/******************************************************************************
*
*                      General Interface Description
*
* The DSP56F801, DSP56F803, and DSP56F805 each have a dual, 4-input,
* analog-to-digital converter (ADC) named ADCA totaling eight analog inputs. The
* DSP56F807 has two dual, 4-inputs, 12-bit ACDs, named ADCA and ADCB, totaling 16
* analog inputs. While the registers are named identically, they have different
* addresses.
* 
* The analog-to-digital converter (ADC) consists of two separate and complete ADCs,
* each with their own sample and hold circuits. A common digital control module
* configures and controls the functioning of these ADCs.
* 
* ADC features:
*  - 12-bit resolution
*  - Sampling rate up to 1.66 million samples per second1
*  - Maximum ADC clock frequency is 5MHz with 200ns period
*  - Single conversion time of 8.5 ADC clock cycles (8.5 x 200 ns = 1.7ms)
*  - Additional conversion time of 6 ADC clock cycles (6 x 200 ns = 1.2ms)
*  - Eight conversions in 26.5 ADC clock cycles (26.5 x 200 ns = 5.3ms) using
*    simultaneous mode
*  - ADC can be synchronized to the PWM via the sync signal
*  - Simultaneous or sequential sampling
*  - Internal multiplexer to select two of eight inputs
*  - Ability to sequentially scan and store up to eight measurements
*  - Ability to simultaneously sample and hold two inputs
*  - Optional interrupts at end of scan, if an out-of-range limit is exceeded
*    or at zero crossing
*  - Optional sample correction by subtracting a pre-programmed offset value
*  - Signed or unsigned result
*  - Single ended or differential inputs
* 
* For more information, please refer to the DSP56F80x User's Manual,
* for a description of the ADC device.
*
* The ADC device driver interface uses "ioctl" call to control specific
* ADC functions during operation. For details see "ioctl" call.
*      
*
******************************************************************************/

/**************************************************************
* Configuration items for appconfig.h (init values, ...),
**************************************************************/

/* Configuration items for appconfig.h  *

  void adcxISR( void );              prototype for user ISR 
  #define INT_VECTOR_ADDR_yy          adcxISR
  #define INT_PRIORITY_LEVEL_yy        value 0-7 (0 = interrupt disabled)

  where:
	 x   is ADC module (A or B)
         yy  is interrupt vector number
                57 - ADC A Zero Crossing or Limit Error
                56 - ADC B Zero Crossing or Limit Error
                55 - ADC A Conversion Complete
                54 - ADC B Conversion Complete


	ADC_A_ADCR1_INIT   	// control reg 1 
	ADC_A_ADCR2_INIT   	// control reg 2    
	ADC_A_ADZCC_INIT   	// zero cross control reg
	ADC_A_ADLST1_INIT   // channel list reg 1
	ADC_A_ADLST2_INIT   // channel list reg 2
	ADC_A_ADSDIS_INIT   // disable reg

	ADC_A_ADLLMT0_INIT 	// low limit regs
	ADC_A_ADLLMT1_INIT 
	ADC_A_ADLLMT2_INIT 
	ADC_A_ADLLMT3_INIT 
	ADC_A_ADLLMT4_INIT 
	ADC_A_ADLLMT5_INIT 
	ADC_A_ADLLMT6_INIT 
	ADC_A_ADLLMT7_INIT 

	ADC_A_ADHLMT0_INIT  // high limit regs
	ADC_A_ADHLMT1_INIT  
	ADC_A_ADHLMT2_INIT  
	ADC_A_ADHLMT3_INIT  
	ADC_A_ADHLMT4_INIT  
	ADC_A_ADHLMT5_INIT  
	ADC_A_ADHLMT6_INIT  
	ADC_A_ADHLMT7_INIT  

	ADC_A_ADOFS0_INIT   // offset regs
	ADC_A_ADOFS1_INIT   
	ADC_A_ADOFS2_INIT   
	ADC_A_ADOFS3_INIT   
	ADC_A_ADOFS4_INIT   
	ADC_A_ADOFS5_INIT   
	ADC_A_ADOFS6_INIT   
	ADC_A_ADOFS7_INIT   


	// dtto for ADC_B
*/

/***************************************************
* ADC Ioctl commands
****************************************************/
/* command                      |        Param     */

/* ADC control register 1 commands */
#define ADC_INIT                    /* NULL */
#define ADC_START					/* NULL */
#define ADC_STOP					/* ADC_ON(1)/ADC_OFF(0) */
#define ADC_TEST_MODE 				/* ADC_ENABLE / ADC_DISABLE */
#define ADC_SYNC					/* ADC_ON(1)/ADC_OFF(0) */
#define ADC_SET_CHANNEL_CONFIG		/* ADC_ANx_ANy_SE , ADC_ANx_ANy_DIFF */
#define ADC_SET_SCAN_MODE   		/* ADC_SCAN_xxx */
//#define ADC_END_OF_SCAN_INT			/* ADC_ENABLE / ADC_DISABLE */
//#define ADC_ZERO_CROSS_INT			/* ADC_ENABLE / ADC_DISABLE */
//#define ADC_LOW_LIMIT_INT			/* ADC_ENABLE / ADC_DISABLE */
//#define ADC_HIGH_LIMIT_INT			/* ADC_ENABLE / ADC_DISABLE */
#define ADC_INT_ENABLE				/* ADC_END_OF_SCAN | ADC_ZERO_CROSS | ADC_LOW_LIMIT | ADC_HIGH_LIMIT */
#define ADC_INT_DISABLE				/* ADC_END_OF_SCAN | ADC_ZERO_CROSS | ADC_LOW_LIMIT | ADC_HIGH_LIMIT */
#define ADC_INT_SELECT				/* ADC_END_OF_SCAN | ADC_ZERO_CROSS | ADC_LOW_LIMIT | ADC_HIGH_LIMIT */

/* ADC control register 2 set */
#define ADC_SET_DIVISOR 			/* UWord16:5 */

/* write ADC zero cross register */
//#define ADC_WRITE_ZERO_CROSS_CNTRL	/* ADC_Sx_ZC_DISABLE | ADC_Sx_ZC_POSITIVE_NEGATIVE | ADC_Sx_ZC_NEGATIVE_POSITIVE | ADC_Sx_ZC_ANY_CROSS */
#define ADC_ZERO_CROSS_CH0			/* ADC_ZC_DISABLE / ADC_ZC_POS2NEG / ADC_ZC_NEG2POS / ADC_ZC_BOTH */
#define ADC_ZERO_CROSS_CH1			/* ADC_ZC_DISABLE / ADC_ZC_POS2NEG / ADC_ZC_NEG2POS / ADC_ZC_BOTH */
#define ADC_ZERO_CROSS_CH2			/* ADC_ZC_DISABLE / ADC_ZC_POS2NEG / ADC_ZC_NEG2POS / ADC_ZC_BOTH */
#define ADC_ZERO_CROSS_CH3			/* ADC_ZC_DISABLE / ADC_ZC_POS2NEG / ADC_ZC_NEG2POS / ADC_ZC_BOTH */
#define ADC_ZERO_CROSS_CH4			/* ADC_ZC_DISABLE / ADC_ZC_POS2NEG / ADC_ZC_NEG2POS / ADC_ZC_BOTH */
#define ADC_ZERO_CROSS_CH5			/* ADC_ZC_DISABLE / ADC_ZC_POS2NEG / ADC_ZC_NEG2POS / ADC_ZC_BOTH */
#define ADC_ZERO_CROSS_CH6			/* ADC_ZC_DISABLE / ADC_ZC_POS2NEG / ADC_ZC_NEG2POS / ADC_ZC_BOTH */
#define ADC_ZERO_CROSS_CH7			/* ADC_ZC_DISABLE / ADC_ZC_POS2NEG / ADC_ZC_NEG2POS / ADC_ZC_BOTH */

/* write ADC channel list registers */
//#define ADC_WRITE_CHANNEL_LIST1		/* ADC_ANx_Sy | ADC_ANx_Sy ... */
//#define ADC_WRITE_CHANNEL_LIST2		/* ADC_ANx_Sy | ADC_ANx_Sy ... */
#define ADC_SET_LIST_SAMPLE0		/* one of ADC_CH0, ..., ADC_CH7 (number 0-7) */
#define ADC_SET_LIST_SAMPLE1		/* one of ADC_CH0, ..., ADC_CH7 (number 0-7) */
#define ADC_SET_LIST_SAMPLE2		/* one of ADC_CH0, ..., ADC_CH7 (number 0-7) */
#define ADC_SET_LIST_SAMPLE3		/* one of ADC_CH0, ..., ADC_CH7 (number 0-7) */
#define ADC_SET_LIST_SAMPLE4		/* one of ADC_CH0, ..., ADC_CH7 (number 0-7) */
#define ADC_SET_LIST_SAMPLE5		/* one of ADC_CH0, ..., ADC_CH7 (number 0-7) */
#define ADC_SET_LIST_SAMPLE6		/* one of ADC_CH0, ..., ADC_CH7 (number 0-7) */
#define ADC_SET_LIST_SAMPLE7		/* one of ADC_CH0, ..., ADC_CH7 (number 0-7) */

/* write ADC sample disable register */
#define ADC_WRITE_SAMPLE_DISABLE	/* UWord16 : number 0-7 */

/* read ADC result register */
#define ADC_READ_SAMPLE			    /* UWord16 : sample number 0-7 */

/* read all ADC result registers */
#define ADC_READ_ALL_SAMPLES		/* adc_tBuff */

/* read ADC status register */
#define ADC_READ_STATUS 			/* NULL */

/* write ADC low limit registers */
#define ADC_WRITE_LOW_LIMIT0  		/* UWord16 : low limit (sample 0) */
#define ADC_WRITE_LOW_LIMIT1  		/* UWord16 : low limit (sample 1) */
#define ADC_WRITE_LOW_LIMIT2  		/* UWord16 : low limit (sample 2) */
#define ADC_WRITE_LOW_LIMIT3  		/* UWord16 : low limit (sample 3) */
#define ADC_WRITE_LOW_LIMIT4  		/* UWord16 : low limit (sample 4) */
#define ADC_WRITE_LOW_LIMIT5  		/* UWord16 : low limit (sample 5) */
#define ADC_WRITE_LOW_LIMIT6  		/* UWord16 : low limit (sample 6) */
#define ADC_WRITE_LOW_LIMIT7  		/* UWord16 : low limit (sample 7) */

/* read ADC low limit register x */
#define ADC_READ_LOW_LIMIT 			/* UWord16: sample number (0-7) */

/* write ADC high limit registers */
#define ADC_WRITE_HIGH_LIMIT0 		/* UWord16 : high limit (sample 0) */
#define ADC_WRITE_HIGH_LIMIT1 		/* UWord16 : high limit (sample 1) */
#define ADC_WRITE_HIGH_LIMIT2 		/* UWord16 : high limit (sample 2) */
#define ADC_WRITE_HIGH_LIMIT3 		/* UWord16 : high limit (sample 3) */
#define ADC_WRITE_HIGH_LIMIT4 		/* UWord16 : high limit (sample 4) */
#define ADC_WRITE_HIGH_LIMIT5 		/* UWord16 : high limit (sample 5) */
#define ADC_WRITE_HIGH_LIMIT6 		/* UWord16 : high limit (sample 6) */
#define ADC_WRITE_HIGH_LIMIT7 		/* UWord16 : high limit (sample 7) */

/* read ADC high limit register x */
#define ADC_READ_HIGH_LIMIT 		/* UWord16: sample number (0-7) */

/* write ADC offset registers */
#define ADC_WRITE_OFFSET0	 		/* UWord16 : offset (sample 0) */
#define ADC_WRITE_OFFSET1	 		/* UWord16 : offset (sample 1) */
#define ADC_WRITE_OFFSET2	 		/* UWord16 : offset (sample 2) */
#define ADC_WRITE_OFFSET3	 		/* UWord16 : offset (sample 3) */
#define ADC_WRITE_OFFSET4	 		/* UWord16 : offset (sample 4) */
#define ADC_WRITE_OFFSET5	 		/* UWord16 : offset (sample 5) */
#define ADC_WRITE_OFFSET6	 		/* UWord16 : offset (sample 6) */
#define ADC_WRITE_OFFSET7	 		/* UWord16 : offset (sample 7) */

/* read ADC offset register x */
#define ADC_READ_OFFSET 			/* UWord16: sample number (0-7) */

/* read ADC limit status register */
#define ADC_READ_LIMIT_STATUS 		/* NULL */
/* read ADC zero crossing status register */
#define ADC_READ_ZERO_CROSS_STATUS 	/* NULL */

/* test ADC status register bits - can be used directly in codition */
/* if it is used in assignement then it returns bit value in original position */
#define ADC_GET_STATUS_CIP			/* NULL */
#define ADC_GET_STATUS_EOSI			/* NULL */
#define ADC_GET_STATUS_ZCI			/* NULL */
#define ADC_GET_STATUS_LLMTI		/* NULL */
#define ADC_GET_STATUS_HLMTI		/* NULL */
#define ADC_GET_STATUS_RDY			/* UWord16 : sample number (0-7) */

/* test ADC limit status register bits */
#define ADC_GET_LIMIT_STATUS_LLS	/* UWord16 : sample number (0-7) */
#define ADC_GET_LIMIT_STATUS_HLS	/* UWord16 : sample number (0-7) */

#define ADC_GET_ZERO_CROSS_STATUS_ZCS	/* UWord16 : sample number (0-7) */

/* clear ADC status registers flags */
//#define ADC_CLEAR_STATUS_CIP		/* NULL */
#define ADC_CLEAR_STATUS_EOSI		/* NULL */
#define ADC_CLEAR_STATUS_LLMTI		/* NULL */
#define ADC_CLEAR_STATUS_HLMTI	    /* NULL */	
#define ADC_CLEAR_STATUS_ZCI        /* NULL */		

/* clearing bits in Limit Status, Zero Crossing Status reg individaully */
#define ADC_CLEAR_LIMIT_STATUS_HLS		/* UWord16 : sample number (0-7) */
#define ADC_CLEAR_LIMIT_STATUS_LLS		/* UWord16 : sample number (0-7) */
#define ADC_CLEAR_ZERO_CROSS_STATUS_ZCS	/* UWord16 : sample number (0-7) */ 


/****************************************************************************
* ADC command parameters
*****************************************************************************/

#define ADC_ENABLE	1
#define ADC_DISABLE	0

#define ADC_ON	1
#define ADC_OFF	0

/* interrupts */
#define ADC_END_OF_SCAN	ADC_ADCR1_EOSIE 
#define ADC_ZERO_CROSS	ADC_ADCR1_ZCIE  
#define ADC_LOW_LIMIT	ADC_ADCR1_LLMTIE
#define ADC_HIGH_LIMIT	ADC_ADCR1_HLMTIE
				
/* zero crossing modes */
#define ADC_ZC_DISABLE 0	
#define ADC_ZC_POS2NEG 1
#define ADC_ZC_NEG2POS 2
#define ADC_ZC_BOTH    3

/* channel numbers */
#define ADC_CH0 0
#define ADC_CH1 1
#define ADC_CH2 2
#define ADC_CH3 3
#define ADC_CH4 4
#define ADC_CH5 5
#define ADC_CH6 6
#define ADC_CH7 7

/* channel configurations */
#define ADC_AN0_AN1_SE		0
#define ADC_AN0_AN1_DIFF 	1
#define ADC_AN2_AN3_SE		0
#define ADC_AN2_AN3_DIFF 	2
#define ADC_AN4_AN5_SE		0
#define ADC_AN4_AN5_DIFF 	4
#define ADC_AN6_AN7_SE		0
#define ADC_AN6_AN7_DIFF 	8

/* adc modes */
#define ADC_SCAN_ONCE_SEQUENTIAL	ADC_ADCR1_SMODE_ONCESEQ
#define ADC_SCAN_ONCE_SIMULTANEOUS	ADC_ADCR1_SMODE_ONCESIM
#define ADC_SCAN_LOOP_SEQUENTIAL	ADC_ADCR1_SMODE_LOOPSEQ
#define ADC_SCAN_LOOP_SIMULTANEOUS	ADC_ADCR1_SMODE_LOOPSIM
#define ADC_SCAN_TRIG_SEQUENTIAL	ADC_ADCR1_SMODE_TRIGSEQ
#define ADC_SCAN_TRIG_SIMULTANEOUS	ADC_ADCR1_SMODE_TRIGSIM

/* bit mask for testing of the status registers ADC_READ_STATUS (RDYx bits) */
#define ADC_S0	ADC_ADSTAT_RDY0
#define ADC_S1	ADC_ADSTAT_RDY1
#define ADC_S2	ADC_ADSTAT_RDY2
#define ADC_S3	ADC_ADSTAT_RDY3
#define ADC_S4	ADC_ADSTAT_RDY4
#define ADC_S5	ADC_ADSTAT_RDY5
#define ADC_S6	ADC_ADSTAT_RDY6
#define ADC_S7	ADC_ADSTAT_RDY7

/* bit mask for testing of the status registers ADC_READ_LIMIT_STATUS */
#define ADC_LLS0	ADC_ADLSTAT_LLS0
#define ADC_LLS1	ADC_ADLSTAT_LLS1
#define ADC_LLS2	ADC_ADLSTAT_LLS2
#define ADC_LLS3	ADC_ADLSTAT_LLS3
#define ADC_LLS4	ADC_ADLSTAT_LLS4
#define ADC_LLS5	ADC_ADLSTAT_LLS5
#define ADC_LLS6	ADC_ADLSTAT_LLS6
#define ADC_LLS7	ADC_ADLSTAT_LLS7

#define ADC_HLS0	ADC_ADLSTAT_HLS0
#define ADC_HLS1	ADC_ADLSTAT_HLS1
#define ADC_HLS2	ADC_ADLSTAT_HLS2
#define ADC_HLS3	ADC_ADLSTAT_HLS3
#define ADC_HLS4	ADC_ADLSTAT_HLS4
#define ADC_HLS5	ADC_ADLSTAT_HLS5
#define ADC_HLS6	ADC_ADLSTAT_HLS6
#define ADC_HLS7	ADC_ADLSTAT_HLS7

/* channel list registers - masks for samples & inputs */
/* sample 0 */
#define ADC_AN0_S0	0x0000
#define ADC_AN1_S0	0x0001
#define ADC_AN2_S0	0x0002
#define ADC_AN3_S0	0x0003
#define ADC_AN4_S0	0x0004
#define ADC_AN5_S0	0x0005
#define ADC_AN6_S0	0x0006
#define ADC_AN7_S0	0x0007
/* sample 1 */
#define ADC_AN0_S1	0x0000
#define ADC_AN1_S1	0x0010
#define ADC_AN2_S1	0x0020
#define ADC_AN3_S1	0x0030
#define ADC_AN4_S1	0x0040
#define ADC_AN5_S1	0x0050
#define ADC_AN6_S1	0x0060
#define ADC_AN7_S1	0x0070
/* sample 2 */
#define ADC_AN0_S2	0x0000
#define ADC_AN1_S2	0x0100
#define ADC_AN2_S2	0x0200
#define ADC_AN3_S2	0x0300
#define ADC_AN4_S2	0x0400
#define ADC_AN5_S2	0x0500
#define ADC_AN6_S2	0x0600
#define ADC_AN7_S2	0x0700
/* sample 3 */
#define ADC_AN0_S3	0x0000
#define ADC_AN1_S3	0x1000
#define ADC_AN2_S3	0x2000
#define ADC_AN3_S3	0x3000
#define ADC_AN4_S3	0x4000
#define ADC_AN5_S3	0x5000
#define ADC_AN6_S3	0x6000
#define ADC_AN7_S3	0x7000

/* sample 4 */
#define ADC_AN0_S4	0x0000
#define ADC_AN1_S4	0x0001
#define ADC_AN2_S4	0x0002
#define ADC_AN3_S4	0x0003
#define ADC_AN4_S4	0x0004
#define ADC_AN5_S4	0x0005
#define ADC_AN6_S4	0x0006
#define ADC_AN7_S4	0x0007
/* sample 5 */
#define ADC_AN0_S5	0x0000
#define ADC_AN1_S5	0x0010
#define ADC_AN2_S5	0x0020
#define ADC_AN3_S5	0x0030
#define ADC_AN4_S5	0x0040
#define ADC_AN5_S5	0x0050
#define ADC_AN6_S5	0x0060
#define ADC_AN7_S5	0x0070
/* sample 6 */
#define ADC_AN0_S6	0x0000
#define ADC_AN1_S6	0x0100
#define ADC_AN2_S6	0x0200
#define ADC_AN3_S6	0x0300
#define ADC_AN4_S6	0x0400
#define ADC_AN5_S6	0x0500
#define ADC_AN6_S6	0x0600
#define ADC_AN7_S6	0x0700
/* sample 7 */
#define ADC_AN0_S7	0x0000
#define ADC_AN1_S7	0x1000
#define ADC_AN2_S7	0x2000
#define ADC_AN3_S7	0x3000
#define ADC_AN4_S7	0x4000
#define ADC_AN5_S7	0x5000
#define ADC_AN6_S7	0x6000
#define ADC_AN7_S7	0x7000

/* zero crossing control register - masks for samples and crossing types */
/* sample 0 */
#define ADC_S0_ZC_DISABLE			0x0000
#define ADC_S0_ZC_POSITIVE_NEGATIVE	0x0001
#define ADC_S0_ZC_NEGATIVE_POSITIVE	0x0002
#define ADC_S0_ZC_ANY_CROSS			0x0003
/* sample 1 */
#define ADC_S1_ZC_DISABLE			0x0000
#define ADC_S1_ZC_POSITIVE_NEGATIVE	0x0004
#define ADC_S1_ZC_NEGATIVE_POSITIVE	0x0008
#define ADC_S1_ZC_ANY_CROSS			0x000C
/* sample 2 */
#define ADC_S2_ZC_DISABLE			0x0000
#define ADC_S2_ZC_POSITIVE_NEGATIVE	0x0010
#define ADC_S2_ZC_NEGATIVE_POSITIVE	0x0020
#define ADC_S2_ZC_ANY_CROSS			0x0030
/* sample 3 */
#define ADC_S3_ZC_DISABLE			0x0000
#define ADC_S3_ZC_POSITIVE_NEGATIVE	0x0040
#define ADC_S3_ZC_NEGATIVE_POSITIVE	0x0080
#define ADC_S3_ZC_ANY_CROSS			0x00C0
/* sample 4 */
#define ADC_S4_ZC_DISABLE			0x0000
#define ADC_S4_ZC_POSITIVE_NEGATIVE	0x0100
#define ADC_S4_ZC_NEGATIVE_POSITIVE	0x0200
#define ADC_S4_ZC_ANY_CROSS			0x0300
/* sample 5 */
#define ADC_S5_ZC_DISABLE			0x0000
#define ADC_S5_ZC_POSITIVE_NEGATIVE	0x0400
#define ADC_S5_ZC_NEGATIVE_POSITIVE	0x0800
#define ADC_S5_ZC_ANY_CROSS			0x0C00
/* sample 6 */
#define ADC_S6_ZC_DISABLE			0x0000
#define ADC_S6_ZC_POSITIVE_NEGATIVE	0x1000
#define ADC_S6_ZC_NEGATIVE_POSITIVE	0x2000
#define ADC_S6_ZC_ANY_CROSS			0x3000
/* sample 7 */
#define ADC_S7_ZC_DISABLE			0x0000
#define ADC_S7_ZC_POSITIVE_NEGATIVE	0x4000
#define ADC_S7_ZC_NEGATIVE_POSITIVE	0x8000
#define ADC_S7_ZC_ANY_CROSS			0xC000


/****************************************************************************
* register and bit names used in the ioctl functions below
*****************************************************************************/

/* control register (ADCR1) */

#define ADC_ADCR1_STOP        0x4000
#define ADC_ADCR1_START       0x2000
#define ADC_ADCR1_SYNC        0x1000
#define ADC_ADCR1_EOSIE       0x0800    
#define ADC_ADCR1_ZCIE        0x0400    
#define ADC_ADCR1_LLMTIE      0x0200
#define ADC_ADCR1_HLMTIE      0x0100
#define ADC_ADCR1_CHNCFG_MASK 0x00f0
#define ADC_ADCR1_CHNCFG_AN01_DIFF 0x0010
#define ADC_ADCR1_CHNCFG_AN23_DIFF 0x0020
#define ADC_ADCR1_CHNCFG_AN45_DIFF 0x0040
#define ADC_ADCR1_CHNCFG_AN67_DIFF 0x0080
#define ADC_ADCR1_SMODE_MASK     0x0007
#define ADC_ADCR1_SMODE_ONCESEQ  0x0000
#define ADC_ADCR1_SMODE_ONCESIM  0x0001
#define ADC_ADCR1_SMODE_LOOPSEQ  0x0002
#define ADC_ADCR1_SMODE_LOOPSIM  0x0003
#define ADC_ADCR1_SMODE_TRIGSEQ  0x0004
#define ADC_ADCR1_SMODE_TRIGSIM  0x0005

/* control register (ADCR2) */

#define ADC_ADCR2_DIV_MASK  0x000f

/* sample disable register (ADSDIS) */

#define ADC_ADSDIS_TEST_MASK 	0xc000
#define ADC_ADSDIS_TEST_TEST   	0x4000

/* status register ADSTAT */

#define ADC_ADSTAT_CIP  	0x8000    
#define ADC_ADSTAT_EOSI 	0x0800    
#define ADC_ADSTAT_ZCI  	0x0400    
#define ADC_ADSTAT_LLMTI 	0x0200
#define ADC_ADSTAT_HLMTI 	0x0100
#define ADC_ADSTAT_RDY7		0x0080
#define ADC_ADSTAT_RDY6		0x0040
#define ADC_ADSTAT_RDY5		0x0020
#define ADC_ADSTAT_RDY4		0x0010
#define ADC_ADSTAT_RDY3		0x0008
#define ADC_ADSTAT_RDY2		0x0004
#define ADC_ADSTAT_RDY1		0x0002
#define ADC_ADSTAT_RDY0		0x0001

/* limit status register ADLSTAT */

#define ADC_ADLSTAT_HLS7	0x8000
#define ADC_ADLSTAT_HLS6	0x4000
#define ADC_ADLSTAT_HLS5	0x2000
#define ADC_ADLSTAT_HLS4	0x1000
#define ADC_ADLSTAT_HLS3	0x0800
#define ADC_ADLSTAT_HLS2	0x0400
#define ADC_ADLSTAT_HLS1	0x0200
#define ADC_ADLSTAT_HLS0	0x0100
#define ADC_ADLSTAT_LLS7	0x0080
#define ADC_ADLSTAT_LLS6	0x0040
#define ADC_ADLSTAT_LLS5	0x0020
#define ADC_ADLSTAT_LLS4	0x0010
#define ADC_ADLSTAT_LLS3	0x0008
#define ADC_ADLSTAT_LLS2	0x0004
#define ADC_ADLSTAT_LLS1	0x0002
#define ADC_ADLSTAT_LLS0	0x0001

/* old, deprecated names of registers bits */

/* control register 1 */
#define ADC_BSTOP   0x4000
#define ADC_BSTART  0x2000
#define ADC_BSYNC   0x1000
#define ADC_EOSIE   0x0800    
#define ADC_ZCIE    0x0400    
#define ADC_LLMTIE  0x0200
#define ADC_HLMTIE  0x0100
    
/* status register */
/* - usable also for status register testing with ADC_READ_STATUS */
#define ADC_CIP  	0x8000    
#define ADC_EOSI 	0x0800    
#define ADC_ZCI  	0x0400    
#define ADC_LLMTI   0x0200
#define ADC_HLMTI   0x0100


/* results buffer */
typedef UWord16 adc_tBuff[8];

void adcInit(arch_sADC *pAdcBase);


/****************************************************************************
* ADC ioctl macro implementation
*****************************************************************************/

/* module names for internal use in ioctl implementation */

#define __ADC_A ADC_A
#define __ADC_B ADC_B

/***********************************************************************
* ADC init
***********************************************************************/

#define ioctlADC_INIT(reg,bParam) adcInit(reg)

/***********************************************************************
* ADC Control Register
***********************************************************************/

/* start ADC conversion */
#define ioctlADC_START(pAdcBase,param) \
  periphBitSet(ADC_ADCR1_START,&pAdcBase->adcr1)
                      
/* stop ADC conversion */
#define ioctlADC_STOP(pAdcBase,param) \
 if (param) periphBitSet(ADC_ADCR1_STOP,&pAdcBase->adcr1); \
 else   periphBitClear(ADC_ADCR1_STOP,&pAdcBase->adcr1)
 
/* switch on/off ADC conversion starting by SYNC (spec. timer output) */
#define ioctlADC_SYNC(pAdcBase,param) \
 if (param) periphBitSet(ADC_ADCR1_SYNC,&pAdcBase->adcr1); \
 else periphBitClear(ADC_ADCR1_SYNC,&pAdcBase->adcr1)

/* enable/disable interrupt on the end of the ADC conversion */
#define ioctlADC_END_OF_SCAN_INT(pAdcBase,param) \
  if (param) periphBitSet(ADC_ADCR1_EOSIE,&pAdcBase->adcr1); \
  else periphBitClear(ADC_ADCR1_EOSIE,&pAdcBase->adcr1)
  
/* enable/disable interrupt on ADC zero crossing */
#define ioctlADC_ZERO_CROSS_INT(pAdcBase,param) \
  if (param) periphBitSet(ADC_ADCR1_ZCIE,&pAdcBase->adcr1); \
  else periphBitClear(ADC_ADCR1_ZCIE,&pAdcBase->adcr1)
  
/* enable/disable interrupt on ADC low limit compare */
#define ioctlADC_LOW_LIMIT_INT(pAdcBase,param) \
 if (param) periphBitSet(ADC_ADCR1_LLMTIE,&pAdcBase->adcr1); \
 else periphBitClear(ADC_ADCR1_LLMTIE,&pAdcBase->adcr1)
 
/* enable/disable interrupt on ADC high limit compare */
#define ioctlADC_HIGH_LIMIT_INT(pAdcBase,param) \
 if (param) periphBitSet(ADC_ADCR1_HLMTIE,&pAdcBase->adcr1); \
 else periphBitClear(ADC_ADCR1_HLMTIE,&pAdcBase->adcr1)

/* enable interrupts */
#define ioctlADC_INT_ENABLE(pAdcBase, param)		\
  periphBitSet (param,&pAdcBase->adcr1)

/* enable interrupts */
#define ioctlADC_INT_DISABLE(pAdcBase, param)		\
  periphBitClear (param,&pAdcBase->adcr1)

/* enable interrupts */
#define ioctlADC_INT_SELECT(pAdcBase, param)		\
  periphBitGrpRS (ADC_ADCR1_EOSIE | ADC_ADCR1_ZCIE | ADC_ADCR1_LLMTIE | ADC_ADCR1_HLMTIE, \
  	param,&pAdcBase->adcr1)

/* write to CHNCFG[3:0] bits in ADCR1 */
#define ioctlADC_SET_CHANNEL_CONFIG(pAdcBase,param) \
  periphBitGrpZS(ADC_ADCR1_CHNCFG_MASK, (param)<<4,&pAdcBase->adcr1)

/* write to SMODE[2:0] bits in ADCR1 */
#define ioctlADC_SET_SCAN_MODE(pAdcBase,param) \
  periphBitGrpZS(ADC_ADCR1_SMODE_MASK, param,&pAdcBase->adcr1)

/* set ADC clock divisor select */
#define ioctlADC_SET_DIVISOR(pAdcBase,param) \
  periphMemWrite((UWord16) (param),&pAdcBase->adcr2)

/***********************************************************************
* ADC zero crossing register
***********************************************************************/

/* set ADC zero crossing control register */
#define ioctlADC_WRITE_ZERO_CROSS_CNTRL(pAdcBase,param) \
  periphMemWrite((UWord16) (param),&pAdcBase->adzcc)
  
/* individual zero-cross settings for each channel (added) */
#define ioctlADC_ZERO_CROSS_CH0(pAdcBase,param) \
  periphBitGrpZS(3 << 0, (UWord16) ((param)<<0),&pAdcBase->adzcc)
#define ioctlADC_ZERO_CROSS_CH1(pAdcBase,param) \
  periphBitGrpZS(3 << 2, (UWord16) ((param)<<2),&pAdcBase->adzcc)
#define ioctlADC_ZERO_CROSS_CH2(pAdcBase,param) \
  periphBitGrpZS(3 << 4, (UWord16) ((param)<<4),&pAdcBase->adzcc)
#define ioctlADC_ZERO_CROSS_CH3(pAdcBase,param) \
  periphBitGrpZS(3 << 6, (UWord16) ((param)<<6),&pAdcBase->adzcc)
#define ioctlADC_ZERO_CROSS_CH4(pAdcBase,param) \
  periphBitGrpZS(3 << 8, (UWord16) ((param)<<8),&pAdcBase->adzcc)
#define ioctlADC_ZERO_CROSS_CH5(pAdcBase,param) \
  periphBitGrpZS(3 << 10, (UWord16) ((param)<<10),&pAdcBase->adzcc)
#define ioctlADC_ZERO_CROSS_CH6(pAdcBase,param) \
  periphBitGrpZS(3 << 12, (UWord16) ((param)<<12),&pAdcBase->adzcc)
#define ioctlADC_ZERO_CROSS_CH7(pAdcBase,param) \
  periphBitGrpZS(3 << 14, (UWord16) ((param)<<14),&pAdcBase->adzcc)
  

/* set ADC channel list register */
#define ioctlADC_WRITE_CHANNEL_LIST1(pAdcBase,param) \
  periphMemWrite((UWord16) (param),&pAdcBase->adlst1)

#define ioctlADC_WRITE_CHANNEL_LIST2(pAdcBase,param) \
  periphMemWrite((UWord16) (param),&pAdcBase->adlst2)
  
/* individual channel list manipulation for each channel (added) */
#define ioctlADC_SET_LIST_SAMPLE0(pAdcBase,param) \
  periphBitGrpZS(0xf << 0, (UWord16) ((param)<<0),&pAdcBase->adlst1)
#define ioctlADC_SET_LIST_SAMPLE1(pAdcBase,param) \
  periphBitGrpZS(0xf << 4, (UWord16) ((param)<<4),&pAdcBase->adlst1)
#define ioctlADC_SET_LIST_SAMPLE2(pAdcBase,param) \
  periphBitGrpZS(0xf << 8, (UWord16) ((param)<<8),&pAdcBase->adlst1)
#define ioctlADC_SET_LIST_SAMPLE3(pAdcBase,param) \
  periphBitGrpZS(0xf << 12, (UWord16) ((param)<<12),&pAdcBase->adlst1)
#define ioctlADC_SET_LIST_SAMPLE4(pAdcBase,param) \
  periphBitGrpZS(0xf << 0, (UWord16) ((param)<<0),&pAdcBase->adlst2)
#define ioctlADC_SET_LIST_SAMPLE5(pAdcBase,param) \
  periphBitGrpZS(0xf << 4, (UWord16) ((param)<<4),&pAdcBase->adlst2)
#define ioctlADC_SET_LIST_SAMPLE6(pAdcBase,param) \
  periphBitGrpZS(0xf << 8, (UWord16) ((param)<<8),&pAdcBase->adlst2)
#define ioctlADC_SET_LIST_SAMPLE7(pAdcBase,param) \
  periphBitGrpZS(0xf << 12, (UWord16) ((param)<<12),&pAdcBase->adlst2)
  
/* ADC sample disable register */
#define ioctlADC_WRITE_SAMPLE_DISABLE(pAdcBase,param) \
  periphBitGrpSR(0x00FF, (1 << ((UWord16)(param))),&pAdcBase->adsdis)
  

#define ioctlADC_TEST_MODE(pAdcBase,param) \
  if(param) periphBitSet(ADC_ADSDIS_TEST_TEST,&pAdcBase->adsdis); \
  else periphBitClear(ADC_ADSDIS_TEST_TEST,&pAdcBase->adsdis)

/* write ADC low limit register x */
#define ioctlADC_WRITE_LOW_LIMIT0(pAdcBase,param) \
  periphMemWrite((UWord16) (param),&pAdcBase->adllmt[0])
#define ioctlADC_WRITE_LOW_LIMIT1(pAdcBase,param) \
  periphMemWrite((UWord16) (param),&pAdcBase->adllmt[1])
#define ioctlADC_WRITE_LOW_LIMIT2(pAdcBase,param) \
  periphMemWrite((UWord16) (param),&pAdcBase->adllmt[2])
#define ioctlADC_WRITE_LOW_LIMIT3(pAdcBase,param) \
  periphMemWrite((UWord16) (param),&pAdcBase->adllmt[3])
#define ioctlADC_WRITE_LOW_LIMIT4(pAdcBase,param) \
  periphMemWrite((UWord16) (param),&pAdcBase->adllmt[4])
#define ioctlADC_WRITE_LOW_LIMIT5(pAdcBase,param) \
  periphMemWrite((UWord16) (param),&pAdcBase->adllmt[5])
#define ioctlADC_WRITE_LOW_LIMIT6(pAdcBase,param) \
  periphMemWrite((UWord16) (param),&pAdcBase->adllmt[6])
#define ioctlADC_WRITE_LOW_LIMIT7(pAdcBase,param) \
  periphMemWrite((UWord16) (param),&pAdcBase->adllmt[7])

/* read ADC low limit register x */
#define ioctlADC_READ_LOW_LIMIT(pAdcBase,param)  \
  periphMemRead(&pAdcBase->adllmt[param])
  
/* set ADC high limit register x */
#define ioctlADC_WRITE_HIGH_LIMIT0(pAdcBase,param) \
  periphMemWrite((UWord16) (param),&pAdcBase->adhlmt[0])
#define ioctlADC_WRITE_HIGH_LIMIT1(pAdcBase,param) \
  periphMemWrite((UWord16) (param),&pAdcBase->adhlmt[1])
#define ioctlADC_WRITE_HIGH_LIMIT2(pAdcBase,param) \
  periphMemWrite((UWord16) (param),&pAdcBase->adhlmt[2])
#define ioctlADC_WRITE_HIGH_LIMIT3(pAdcBase,param) \
  periphMemWrite((UWord16) (param),&pAdcBase->adhlmt[3])
#define ioctlADC_WRITE_HIGH_LIMIT4(pAdcBase,param) \
  periphMemWrite((UWord16) (param),&pAdcBase->adhlmt[4])
#define ioctlADC_WRITE_HIGH_LIMIT5(pAdcBase,param) \
  periphMemWrite((UWord16) (param),&pAdcBase->adhlmt[5])
#define ioctlADC_WRITE_HIGH_LIMIT6(pAdcBase,param) \
  periphMemWrite((UWord16) (param),&pAdcBase->adhlmt[6])
#define ioctlADC_WRITE_HIGH_LIMIT7(pAdcBase,param) \
  periphMemWrite((UWord16) (param),&pAdcBase->adhlmt[7])
                                          
/* read ADC high limit register x */
#define ioctlADC_READ_HIGH_LIMIT(pAdcBase,param)  \
  periphMemRead(&pAdcBase->adhlmt[param])
                                          
/* write ADC offset register x */
#define ioctlADC_WRITE_OFFSET0(pAdcBase,param) \
  periphMemWrite((UWord16) (param),&pAdcBase->adofs[0])
#define ioctlADC_WRITE_OFFSET1(pAdcBase,param) \
  periphMemWrite((UWord16) (param),&pAdcBase->adofs[1])
#define ioctlADC_WRITE_OFFSET2(pAdcBase,param) \
  periphMemWrite((UWord16) (param),&pAdcBase->adofs[2])
#define ioctlADC_WRITE_OFFSET3(pAdcBase,param) \
  periphMemWrite((UWord16) (param),&pAdcBase->adofs[3])
#define ioctlADC_WRITE_OFFSET4(pAdcBase,param) \
  periphMemWrite((UWord16) (param),&pAdcBase->adofs[4])
#define ioctlADC_WRITE_OFFSET5(pAdcBase,param) \
  periphMemWrite((UWord16) (param),&pAdcBase->adofs[5])
#define ioctlADC_WRITE_OFFSET6(pAdcBase,param) \
  periphMemWrite((UWord16) (param),&pAdcBase->adofs[6])
#define ioctlADC_WRITE_OFFSET7(pAdcBase,param) \
  periphMemWrite((UWord16) (param),&pAdcBase->adofs[7])

/* read ADC offset register x */
#define ioctlADC_READ_OFFSET(pAdcBase,param) \
 periphMemRead(&pAdcBase->adofs[param])
                                                                                                                                                                                                                                                                 
/* reads samples from result registers */
/* one at a time, param is the number of the Result reg (0-7) */
#define ioctlADC_READ_SAMPLE(pAdcBase,param) \
  periphMemRead(&pAdcBase->ResultReg[param])

/* reads samples from result registers - all 8 to the buffer pointed by pParam */
inline void AdcRdResults(adc_tBuff buff, UWord16 *reg) {
	int i; for(i=0; i<8; i++) *buff++ = periphMemRead(reg++); 
}
#define ioctlADC_READ_ALL_SAMPLES(pAdcBase,param) \
  AdcRdResults(param,(UWord16 *)(&pAdcBase->adrslt[0]))

/* read ADC status register */
#define ioctlADC_READ_STATUS(pAdcBase,param) \
  periphMemRead(&pAdcBase->adstat)

/* test some ADC status bits */
#define ioctlADC_GET_STATUS_CIP(pAdcBase,param) \
  periphBitTest(ADC_ADSTAT_CIP,&pAdcBase->adstat)
#define ioctlADC_GET_STATUS_EOSI(pAdcBase,param) \
  periphBitTest(ADC_ADSTAT_EOSI,&pAdcBase->adstat)
#define ioctlADC_GET_STATUS_ZCI(pAdcBase,param) \
  periphBitTest(ADC_ADSTAT_ZCI,&pAdcBase->adstat)
#define ioctlADC_GET_STATUS_LLMTI(pAdcBase,param) \
  periphBitTest(ADC_ADSTAT_LLMTI,&pAdcBase->adstat)
#define ioctlADC_GET_STATUS_HLMTI(pAdcBase,param) \
  periphBitTest(ADC_ADSTAT_HLMTI,&pAdcBase->adstat)

/* test ADC status - ready channel x flag (RDYx), x should be immediate */
#define ioctlADC_GET_STATUS_RDY(pAdcBase,param) \
  periphBitTest(1 << ((UWord16)(param)),&pAdcBase->adstat)
//#define ioctlADC_GET_STATUS_RDY(pAdcBase,param) \
//  periphBitTest(param,&pAdcBase->StatusReg)

/* write ADC status register */
#define ioctlADC_WRITE_STATUS(pAdcBase,param) \
  periphMemWrite((UWord16) (param),&pAdcBase->adstat)

/* clear ADC status bit - end of scan interrupt flag (EOSI) */
#define ioctlADC_CLEAR_STATUS_EOSI(pAdcBase,param) \
  periphMemWrite(ADC_ADSTAT_EOSI,&pAdcBase->StatusReg)

/* reads ADC limit status register - contains all sample limit flag bits */
#define ioctlADC_READ_LIMIT_STATUS(pAdcBase,param) \
  periphMemRead(&pAdcBase->adlstat)

/* test ADC limit status - low limit sample flag x (LLSx) - each sample has distinctive flag bit */
#define ioctlADC_GET_LIMIT_STATUS_LLS(pAdcBase,param) \
  periphBitTest(1 << ((UWord16)(param)),&pAdcBase->adlstat)

/* test ADC limit status - high limit sample flag x (HLSx) - each sample has distinctive flag bit */
#define ioctlADC_GET_LIMIT_STATUS_HLS(pAdcBase,param) \
  periphBitTest(1 << ((UWord16)(param)+8),&pAdcBase->adlstat)

/* reads ADC zero cross status register */
#define ioctlADC_READ_ZERO_CROSS_STATUS(pAdcBase,param) \
  periphMemRead(&pAdcBase->adzcstat)

/* test ADC zero cross status - zero cross sample flag x (ZCSx) */
#define ioctlADC_GET_ZERO_CROSS_STATUS_ZCS(pAdcBase,param) \
  periphBitTest(1 << ((UWord16)(param)),&pAdcBase->adzcstat)

/* clear ADC status bit - low limit interrupt flag (LLMTI) */
/* cleared indirectly by clearing all LLSx flags in limit status registers */
#define ioctlADC_CLEAR_STATUS_LLMTI(pAdcBase,param) \
  periphMemWrite(0x00FF,&pAdcBase->adlstat)

/* clear ADC status bit - high limit interrupt flag (HLMTI) */
/* cleared indirectly by clearing all HLSx flags in limit status registers */
#define ioctlADC_CLEAR_STATUS_HLMTI(pAdcBase,param) \
  periphMemWrite(0xFF00,&pAdcBase->adlstat)

/* clear ADC status bit - zero cross interrupt flag (ZCI) */
#define ioctlADC_CLEAR_STATUS_ZCI(pAdcBase,param) \
  periphMemWrite(0x00FF,&pAdcBase->adzcstat)

/* clear bits in ADC limit status - low limit sample flag x (LLSx) */
#define ioctlADC_CLEAR_LIMIT_STATUS_LLS(pAdcBase,param) \
  periphMemWrite(1 << ((UWord16)(param)),&pAdcBase->adlstat)

/* clear bits in ADC limit status - high limit sample flag x (HLSx) */
#define ioctlADC_CLEAR_LIMIT_STATUS_HLS(pAdcBase,param) \
  periphMemWrite(1 << ((UWord16)(param)+8),&pAdcBase->adlstat)

/* clear bits in ADC zero cross status - zero cross sample flag x (ZCSx) */
#define ioctlADC_CLEAR_ZERO_CROSS_STATUS_ZCS(pAdcBase,param) \
  periphMemWrite(1 << ((UWord16)(param)),&pAdcBase->adzcstat)


/**********************************************************************
* Include device specific ADC commands and defines
***********************************************************************/
#include "adctarget.h"

#ifdef __cplusplus
}
#endif

#endif
