/*****************************************************************************
*
* Motorola Inc.
* (c) Copyright 2002 Motorola, Inc.
* ALL RIGHTS RESERVED.
*
******************************************************************************
*
* File Name: cop.h
*
* Description: COP (Computer Operating Properly - Watchdog) driver macros
*              and definitions
*
* Modules Included: none
*
*****************************************************************************/


#ifndef __COP_H
#define __COP_H

#include "appconfig.h"

#if !defined(__ARCH_H) || !defined(__PERIPH_H)
#error arch.h and periph.h must be included before cop.h
#endif

#ifdef __cplusplus
extern "C" {
#endif

/************************************************************
* COP module identifier, corresponds to module base address
*************************************************************/

#define COP (&ArchIO.Cop)

/******************************************************************************
*
*                      General Interface Description
*
* The COP is a free-running counter. Once enabled, it is designed to generate
* a reset on overflow. Software must periodically service the COP in order to
* clear the counter and prevent a reset. The COP uses a 12-bit counter with
* a prescaler to provide 4096 different time-out periods.
* 
* For more information, please refer to the DSP56F80x User's Manual,
* for a description of the COP device.
*
* The COP device driver interface uses "ioctl" call to control specific
* COP functions during operation. For details see "ioctl" call.
*      
*
******************************************************************************/


/***********************************************************************
* COP static configuration items for appconfig.h
***********************************************************************/
/*
	#define COP_COPCTL_INIT 0x0002
	#define COP_COPTO_INIT  0xFFFF
*/

/***************************************************
* COP ioctl commands
****************************************************/
/* command                    |        Param       */

/* COP peripheral initialization according static config in AppConfig.h */
#define COP_INIT				/* null */

/* COP peripheral enable/disable - functional only if COP Write protect Bit (CWP) is not set */
/* - functional only if COP Write protect Bit (CWP) is not set */
#define COP_DEVICE				/* enable(1)/disable(0) */

/* COP peripheral write protect  - functional only if COP Write protect Bit (CWP) is not set */
/* if this command is issued any subsequent write to COPCTL is blocked untill the chip RESET */
#define COP_WRITE_PROTECT

/* write COP timeout interval */
/* - functional only if COP Write protect Bit (CWP) is not set */
/* - recommended is also to disable COP before writing this value */
#define COP_SET_TIMEOUT 		/* ToutValue */

/* control the state of COP in WAIT and STOP mode */
/* - functional only if COP Write protect Bit (CWP) is not set */
#define COP_RUN_IN_STOP			/* run enable(1)/disable(0) in STOP mode */	
#define COP_RUN_IN_WAIT			/* run enable(1)/disable(0) in WAIT mode */	

/* clear COP counter - service to be used in program */
/* loop before COP timeout expiration to prevent COP reset */
#define COP_CLEAR_COUNTER		/* NULL */
#define COP_CLEAR_COUNTER_PART1		/* NULL */
#define COP_CLEAR_COUNTER_PART2		/* NULL */

/* recovery of stack pointer and occs settings after COP reset */
#define COP_RESET_RECOVERY		/* NULL */

/****************************************************************************
* COP constants and type declarations used in the ioctl functions
*****************************************************************************/

#define COP_ENABLE 	1
#define COP_DISABLE 0

/* register bit names */
#define COP_COPCTL_CSEN   0x0008
#define COP_COPCTL_CWEN   0x0004
#define COP_COPCTL_CEN    0x0002
#define COP_COPCTL_CWP    0x0001

#define COP_CSEN   0x0008
#define COP_CWEN   0x0004
#define COP_CEN    0x0002
#define COP_CWP    0x0001


/****************************************************************************
* COP function prototypes
*****************************************************************************/
extern void cop_Init(arch_sCOP *pCopBase);

/***********************************************************************
* COP control command macros
***********************************************************************/
/* COP initialization */
#define ioctlCOP_INIT(pCopBase,param) cop_Init(pCopBase)

/* COP peripheral enable/disable - functional only if COP Write protect Bit (CWP) is not set */
#define ioctlCOP_DEVICE(pCopBase,param) \
  if (param) periphBitSet(COP_CEN,&pCopBase->copctl); \
  else periphBitClear(COP_CEN,&pCopBase->copctl)


/* COP peripheral write protect  - functional only if COP Write protect Bit (CWP) is not set */
/* if this command issued any subsequent write to COPCTL (COP Control Reg.) is blocked */
#define ioctlCOP_WRITE_PROTECT(pCopBase,param) \
  periphBitSet(COP_CWP,&pCopBase->copctl)

/* COP Run in Stop mode enable/disable */
#define ioctlCOP_RUN_IN_STOP(pCopBase,param) \
  if (param) periphBitSet(COP_CSEN,&pCopBase->copctl); \
  else periphBitClear(COP_CSEN,&pCopBase->copctl)

/* COP Run in Stop mode enable/disable */
#define ioctlCOP_RUN_IN_WAIT(pCopBase,param) \
  if (param) periphBitSet(COP_CWEN,&pCopBase->copctl); \
  else periphBitClear(COP_CWEN,&pCopBase->copctl)

/* set COP timeout interval, param value is decremented before write to timeout reg */
#define ioctlCOP_SET_TIMEOUT(pCopBase,param) \
  periphMemWrite((param)-1,&pCopBase->copto)

/* clear(service) COP timer */
#define ioctlCOP_CLEAR_COUNTER(pCopBase,param) \
  { periphMemWrite(0x5555,&pCopBase->copsrv);\
    periphMemWrite(0xAAAA,&pCopBase->copsrv); }

/* clear(service) COP timer, 1st part*/
#define ioctlCOP_CLEAR_COUNTER_PART1(pCopBase,param) \
    periphMemWrite(0x5555,&pCopBase->copsrv)

/* clear(service) COP timer, 2nd part*/
#define ioctlCOP_CLEAR_COUNTER_PART2(pCopBase,param) \
    periphMemWrite(0xAAAA,&pCopBase->copsrv)

/* recovery of stack pointer and occs settings after COP reset */
#define ioctlCOP_RESET_RECOVERY(pCopBase,param)  copResetRecovery()

#define STS    SYS_BASE+1

#define PLLCR  CLKGEN_BASE+0
#define PLLDB  CLKGEN_BASE+1
#define PLLSR  CLKGEN_BASE+2
#define IOSCTL CLKGEN_BASE+5

#ifndef OCCS_REQUIRED_LOCK_MODE
#define OCCS_REQUIRED_LOCK_MODE 0x40 /* coarse (LCK1) by default */
#endif
#if (OCCS_REQUIRED_LOCK_MODE != 0x40) && (OCCS_REQUIRED_LOCK_MODE != 0x20)
#error OCCS_REQUIRED_LOCK_MODE must be one of 0x20 (LCK0-fine) or 0x40 (LCK1-coarse)
#endif

extern _stack_addr;

inline void copResetRecovery(void){
asm{
/* clear COP reset flag */
    bfchg  #$000f,x:STS

/*
#ifdef COP_COPTO_INIT
	move	#COP_COPTO_INIT,X:(COP_BASE+1)
#endif

#ifdef COP_COPCTL_INIT
	move	#COP_COPCTL_INIT,X:COP_BASE
#endif

/*

	Uncomment the two instructions above if you want the COP to be
	initialized right after clearing the COP reset flag.

*/

#ifdef BCR_INIT
    move    #BCR_INIT,x:<$FFF9          /* set waitstates in BCR*/
#endif

#ifdef SYS_CNTL_INIT
    move    #SYS_CNTL_INIT,x:SYS_BASE	/* set SYS_CNTL register */
#endif

#ifdef TARGET_MODE_0B
	bfset	#$10,x:SYS_BASE
	bfclr	#$03,OMR
#endif

	move    #_stack_addr,x0      /*init stack (definition in linker cmd file)*/
	move    x0,x:<$3F           /*set frame pointer*/
	move    x0,sp				/*set stack pointer*/

/* setup the PLL according to appconfig.h values */

#ifdef DSP56F801
#ifdef OCCS_IOSCTL_INIT
    move  #OCCS_IOSCTL_INIT,x:IOSCTL /* DSP56F801 internal relaxation oscillator control */
#endif
#endif    

#ifdef OCCS_PLLDB_INIT
	move  #OCCS_PLLDB_INIT,x:PLLDB	/* PLLDB first, even if PLL is not used */
	nop
	nop
#endif

#ifdef OCCS_PLLCR_INIT

#ifdef DSP56F801
#if ((OCCS_PLLCR_INIT & 4) == 4) /* 801 Crystal oscillator ? */
    bfclr #$000C,x:GPIO_B_BASE   /* disable pull-ups for the EXTAL and XTAL pins */
    nop
    nop
#endif    
#endif    

#if ((OCCS_PLLCR_INIT & 3) == 2) /* PLL active ? */

#define PLLCR_TEMP (OCCS_PLLCR_INIT & 0xfc | 0x01) /* interrupts off and PLL bypassed */

	move  #$11,x0              	/* this is reset value of PLL Control Register */
	cmp   x:PLLCR,x0           	/* test PLL Control Register */
	bne   skip_pll_lock        	/* we are in simulator mode ! (skip PLL setup) */
   
	move  #PLLCR_TEMP,x:PLLCR	/* PLL lock detector ON, core still on prescaler */
	
pll_lock:
	brclr #OCCS_REQUIRED_LOCK_MODE,x:PLLSR,pll_lock	/* test lock (LCK1 or LCK0) */
	
	move  #OCCS_PLLCR_INIT,x:PLLCR	/* PLL is locked: final PLL setup */

skip_pll_lock:
	move  x:PLLSR,x0			/* clear pending clkgen interrupts */
	move  x0,x:PLLSR

#else /* ((OCCS_PLLCR_INIT & 3) == 2) PLL not active */

	move  #OCCS_PLLCR_INIT,x:PLLCR	/* just write the PLLCR init value */

#endif /* ((OCCS_PLLCR_INIT & 3) == 2) */
#endif /* OCCS_PLLCR_INIT */

}
}

#ifdef __cplusplus
}
#endif

#endif
