/*****************************************************************************
*
* Motorola Inc.
* (c) Copyright 2002 Motorola, Inc.
* ALL RIGHTS RESERVED.
*
******************************************************************************
*
* File Name: mscan.h
*
* Description: msCAN (Motorola Scallable CAN) driver macros and definitions
*
* Modules Included: none
*
*****************************************************************************/

#ifndef __MSCAN_H
#define __MSCAN_H

#if !defined(__ARCH_H) || !defined(__PERIPH_H)
#error arch.h and periph.h must be included before mscan.h
#endif

#ifdef __cplusplus
extern "C" {
#endif

/************************************************************
* MSCAN module identifier, corresponds to module base address
*************************************************************/

#define MSCAN (&ArchIO.MSCan)

#define MSCAN_RB  (&ArchIO.MSCan.canrb)
#define MSCAN_TB0 (&ArchIO.MSCan.cantb[0])
#define MSCAN_TB1 (&ArchIO.MSCan.cantb[1])
#define MSCAN_TB2 (&ArchIO.MSCan.cantb[2])

#define MSCAN_MB0 MSCAN_RB 
#define MSCAN_MB1 MSCAN_TB0
#define MSCAN_MB2 MSCAN_TB1
#define MSCAN_MB3 MSCAN_TB2

typedef arch_sMSCAN_MB MSCAN_MB;

/******************************************************************************
*
*                      General Interface Description
*
* The Motorola scalable controller area network (MSCAN) is available
* on three of the four DSP core-based family chips: DSP56F803/805/807.
* Only the DSP56F801 has no MSCAN. The MSCAN definition is based on
* the MSCAN12 definition, the specific implementation of the Motorola
* scalable CAN concept, originally targeted for the Motorola MC68HC12
* micro controller family.
* 
* The module is a communication controller implementing the CAN 2.0 A/B
* protocol, as defined in the Bosch Specification dated September 1991.
* To fully understand the MSCAN specification, we recommend first reading
* the Bosch Specification adapting to terms and concepts contained within
* this document.
* 
* Basic features of the MSCAN:
* - Modular architecture
* - Implementation of the CAN protocolVersion 2.0A/B
* 	 Standard and extended data frames
* 	 Zero to eight bytes data length
* 	 Programmable bit rate up to 1Mbps
* 	 Support for remote frames
* - Double-buffered receive storage scheme
* - Triple-buffered transmit storage scheme with internal prioritization
*   using a local priority concept
* - Flexible identifier filter capable of masking supports two full-size
*   extended identifier filters: two 32-bit, four 16-bit, or eight 8-bit filters
* - Programmable wake-up functionality with integrated low-pass filter
* - Programmable loop back mode supports self-test operation
* - Separate signalling and interrupt capabilities for all CAN receiver
*   and transmitter error states: warning, error passive, bus-off
* - Programmable MSCAN clock source, either IPbus clock or crystal
*   oscillator output
* - Three low power modes: sleep, soft reset and power down
* 
* For more information, please refer to the DSP56F80x User's Manual,
* for a description of the MSCAN device.
*
* The MSCAN device driver interface uses "ioctl" call to control specific
* MSCAN functions during operation. For details see "ioctl" call.
*      
*
******************************************************************************/

/***********************************************************************
* MSCAN static configuration items for appconfig.h
***********************************************************************/
/*
	MSCAN_CANCTL0_INIT 
	MSCAN_CANCTL1_INIT 
	MSCAN_CANBTR0_INIT 
	MSCAN_CANBTR1_INIT 
	MSCAN_CANIDAC_INIT 
	MSCAN_CANIDARn_INIT n=0..7
	MSCAN_CANIDMRn_INIT n=0..7
*/

/***************************************************
* MSCAN ioctl commands
****************************************************/
/* command                    |        Param       */

#define MSCAN_INIT				/* MSCAN peripheral initialization, NULL */
#define MSCAN_SOFT_RESET		/* Enter/leave soft-reset state, MSCAN_ENABLE/MSCAN_DISABLE */
#define MSCAN_SLEEP				/* Enter/leave sleeping state, MSCAN_ENABLE/MSCAN_DISABLE */
#define MSCAN_WAIT_SLEEP		/* Wait for sleep mode, NULL */
#define MSCAN_GET_SLEEP_MODE	/* Returns the sleeping state MSCAN_SLEEPING, ..., NULL */
#define MSCAN_TEST_SYNCH		/* Test bus-off/on condition, NULL */
#define MSCAN_STOP_IN_WAIT		/* Enable low-power in wait mode; MSCAN_ENABLE/MSCAN_DISABLE */

#define MSCAN_DEVICE			/* Enable/disable the MSCAN peripheral; MSCAN_ENABLE/MSCAN_DISABLE */
#define MSCAN_LOOPBACK_MODE		/* Enable/disable test loopback mode; MSCAN_ENABLE/MSCAN_DISABLE */
#define MSCAN_WAKEUP_FILTER		/* Enable/disable low-pass filter on wake-up pin; MSCAN_ENABLE/MSCAN_DISABLE */

#define MSCAN_SET_CLOCK_SOURCE	/* select clock source; MSCAN_IPBUS / MSCAN_XTAL */
#define MSCAN_SET_PRESCALER		/* set BRP; number 1, .. 64 */
#define MSCAN_SET_SJW			/* set SJW; number MSCAN_SJW_1, .. MSCAN_SJW_4 or number 1..4 */
#define MSCAN_SET_TSEG1			/* set TSEG1; number MSCAN_TSEG_1, .. MSCAN_TSEG_16 or number 1..16 */
#define MSCAN_SET_TSEG2			/* set TSEG1; number MSCAN_TSEG_1, .. MSCAN_TSEG_8  or number 1..8 */
#define MSCAN_SET_SAMPLING		/* set samples per bit; MSCAN_1SAMP_PER_BIT, MSCAN_3SAMPS_PER_BIT,  */

#define MSCAN_ERINT_ENABLE		/* Enable Error & Receiver Interrupt; Interrupts switches (see bellow) */
#define MSCAN_ERINT_DISABLE		/* Disable Error & Receiver Interrupt; Interrupts switches (see bellow) */
#define MSCAN_TINT_ENABLE		/* Enable Tx Interrupt; Interrupts switches (see bellow) */
#define MSCAN_TINT_DISABLE		/* Disable Tx Interrupt; Interrupts switches (see bellow) */

#define MSCAN_READ_EINT_FLAGS	/* Get Error Interrupt status (RFLG except RXF & WUP); NULL */
#define MSCAN_READ_TINT_FLAGS	/* Get Transmitter Interrupt status (all TXEn bits); NULL */
#define MSCAN_CLEAR_RINT_FLAG	/* Clear RXF interrupt flag, NULL */
#define MSCAN_CLEAR_WINT_FLAG	/* Clear WUP interrupt flag, NULL */
#define MSCAN_CLEAR_EINT_FLAGS	/* Clear Error interrupt flags, Bits to clear (obtained from MSCAN_READ_EINT_FLAGS) */
#define MSCAN_TRANSMIT			/* Send selected Tx buffers (Clear Transmitter interrupt flag); MSCAN_TXBUFFERn or Bits obtained from MSCAN_READ_TINT_FLAGS */

#define MSCAN_SET_ACC_MODE		/* set acceptance mode; MSCAN_ACC_MODE_2X32, ... */

#define MSCAN_SET_ACC_MASKR_32_0	/* set 32bit acceptance mask 0; value for mask registers */
#define MSCAN_SET_ACC_MASKR_32_1	/* set 32bit acceptance mask 1; value for mask registers */
#define MSCAN_SET_ACC_MASKR_16_0	/* set 16bit acceptance mask 0; value for mask registers */
#define MSCAN_SET_ACC_MASKR_16_1	/* set 16bit acceptance mask 1; value for mask registers */
#define MSCAN_SET_ACC_MASKR_16_2	/* set 16bit acceptance mask 2; value for mask registers */
#define MSCAN_SET_ACC_MASKR_16_3	/* set 16bit acceptance mask 3; value for mask registers */
#define MSCAN_SET_ACC_MASKR_8_0 	/* set 8bit acceptance mask 0; value for mask registers */
#define MSCAN_SET_ACC_MASKR_8_1 	/* set 8bit acceptance mask 1; value for mask registers */
#define MSCAN_SET_ACC_MASKR_8_2 	/* set 8bit acceptance mask 2; value for mask registers */
#define MSCAN_SET_ACC_MASKR_8_3 	/* set 8bit acceptance mask 3; value for mask registers */
#define MSCAN_SET_ACC_MASKR_8_4 	/* set 8bit acceptance mask 4; value for mask registers */
#define MSCAN_SET_ACC_MASKR_8_5 	/* set 8bit acceptance mask 5; value for mask registers */
#define MSCAN_SET_ACC_MASKR_8_6 	/* set 8bit acceptance mask 6; value for mask registers */
#define MSCAN_SET_ACC_MASKR_8_7 	/* set 8bit acceptance mask 7; value for mask registers */

#define MSCAN_SET_ACC_IDR_32_0	/* set 32bit acceptance idr 0; value for IDR registers */
#define MSCAN_SET_ACC_IDR_32_1	/* set 32bit acceptance idr 1; value for IDR registers */
#define MSCAN_SET_ACC_IDR_16_0	/* set 16bit acceptance idr 0; value for IDR registers */
#define MSCAN_SET_ACC_IDR_16_1	/* set 16bit acceptance idr 1; value for IDR registers */
#define MSCAN_SET_ACC_IDR_16_2	/* set 16bit acceptance idr 2; value for IDR registers */
#define MSCAN_SET_ACC_IDR_16_3	/* set 16bit acceptance idr 3; value for IDR registers */
#define MSCAN_SET_ACC_IDR_8_0	/* set 8bit acceptance idr 0; value for IDR registers */
#define MSCAN_SET_ACC_IDR_8_1	/* set 8bit acceptance idr 1; value for IDR registers */
#define MSCAN_SET_ACC_IDR_8_2	/* set 8bit acceptance idr 2; value for IDR registers */
#define MSCAN_SET_ACC_IDR_8_3	/* set 8bit acceptance idr 3; value for IDR registers */
#define MSCAN_SET_ACC_IDR_8_4	/* set 8bit acceptance idr 4; value for IDR registers */
#define MSCAN_SET_ACC_IDR_8_5	/* set 8bit acceptance idr 5; value for IDR registers */
#define MSCAN_SET_ACC_IDR_8_6	/* set 8bit acceptance idr 6; value for IDR registers */
#define MSCAN_SET_ACC_IDR_8_7	/* set 8bit acceptance idr 7; value for IDR registers */

#define MSCAN_GET_RX_ERR_COUNT		/* read RX error counter; NULL */
#define MSCAN_GET_TX_ERR_COUNT		/* read TX error counter; NULL */

#define MSCAN_GET_RB_MODULE			/* Get pointer to Receiver's MB structure; NULL */
#define MSCAN_GET_TB_MODULE			/* Get pointer to Transmitter's MB structure; TB index (0,1,2) */

/****************************************************
 * ioctl commands for MB modules
 ***************************************************/

#define MSCANMB_GET_ID		/* Get translated ID; NULL */
#define MSCANMB_GET_ID_RAW	/* Get four IDR registers of MB as a single 32 bit word; NULL */
#define MSCANMB_GET_LEN		/* Get Data Length Register of given MB; NULL */
#define MSCANMB_GET_RTR 	/* Return non-zero if RTR is set in given MB; NULL */
#define MSCANMB_GET_DATAPTR	/* Get Pointer to 8 data registers for MB; NULL */

#define MSCANMB_SET_ID		/* Set ID; 32bit value, use MSCAN_ID_EXT and MSCAN_ID_RTR */
#define MSCANMB_SET_ID_V	/* Same as above, implemented as function */
#define MSCANMB_SET_ID_RAW	/* Set four IDR registers from a single 32 bit word; 32bit value */
#define MSCANMB_SET_RTR 	/* Set/Clear RTR bits in given MB; MSCAN_ON/MSCAN_OFF */
#define MSCANMB_SET_LEN		/* Set length field of the MB; length */
#define MSCANMB_SET_TBP		/* Set Transmit Buffer Priority of given MB; priority */

/****************************************************************************
* MSCAN constants and type declarations used in the ioctl functions
*****************************************************************************/

#define MSCAN_ENABLE 	1
#define MSCAN_DISABLE 	0

#define MSCAN_ON  1
#define MSCAN_OFF 0

#define MSCAN_NOT_SLEEPING 	   (0)
#define MSCAN_SLEEPING 		   (MSCAN_CANCTL0_SLPRQ | MSCAN_CANCTL0_SLPAK)
#define MSCAN_SLEEP_REQUESTED  (MSCAN_CANCTL0_SLPRQ)

// Clock selection
#define MSCAN_IPBUS 1
#define MSCAN_XTAL  0

// Acceptance modes
#define MSCAN_ACC_MODE_2X32   MSCAN_CANIDAC_IDAM_2X32
#define MSCAN_ACC_MODE_4X16   MSCAN_CANIDAC_IDAM_4X16
#define MSCAN_ACC_MODE_8X8    MSCAN_CANIDAC_IDAM_8X8
#define MSCAN_ACC_MODE_CLOSED MSCAN_CANIDAC_IDAM_CLOSED

// Sampling mode
#define MSCAN_1SAMP_PER_BIT   0
#define MSCAN_2SAMPS_PER_BIT  1

// SJW vlaues
#define MSCAN_SJW_1 1
#define MSCAN_SJW_2 2
#define MSCAN_SJW_3 3
#define MSCAN_SJW_4 4

// Time Segment values
#define MSCAN_TSEG_1	1
#define MSCAN_TSEG_2	2
#define MSCAN_TSEG_3	3
#define MSCAN_TSEG_4	4
#define MSCAN_TSEG_5	5
#define MSCAN_TSEG_6	6
#define MSCAN_TSEG_7	7
#define MSCAN_TSEG_8	8
#define MSCAN_TSEG_9	9
#define MSCAN_TSEG_10	10
#define MSCAN_TSEG_11	11
#define MSCAN_TSEG_12	12
#define MSCAN_TSEG_13	13
#define MSCAN_TSEG_14	14
#define MSCAN_TSEG_15	15
#define MSCAN_TSEG_16	16

// Receiver and Error interrupts & status bits
#define MSCAN_WAKEUP	MSCAN_CANRIER_WUPIE
#define MSCAN_RXWARNING	MSCAN_CANRIER_RWRNIE
#define MSCAN_TXWARNING	MSCAN_CANRIER_TWRNIE
#define MSCAN_RXERROR	MSCAN_CANRIER_RERRIE
#define MSCAN_TXERROR	MSCAN_CANRIER_TERRIE
#define MSCAN_BUSOFF	MSCAN_CANRIER_BOFFIE
#define MSCAN_OVERRUN	MSCAN_CANRIER_OVRIE
#define MSCAN_RXFULL	MSCAN_CANRIER_RXFIE

// Transmitter interrupts
#define MSCAN_TXEMPTY0	MSCAN_CANTCR_TXEIE0
#define MSCAN_TXEMPTY1	MSCAN_CANTCR_TXEIE1
#define MSCAN_TXEMPTY2	MSCAN_CANTCR_TXEIE2
#define MSCAN_TXEMPTY_ALL (MSCAN_TXEMPTY0 | MSCAN_TXEMPTY1 | MSCAN_TXEMPTY2)

// Transmit buffers
#define MSCAN_TXBUFFER0  MSCAN_TXEMPTY0
#define MSCAN_TXBUFFER1  MSCAN_TXEMPTY1
#define MSCAN_TXBUFFER2  MSCAN_TXEMPTY2

// special flags for passing the ID value
#define MSCAN_ID_EXT 0x80000000	// specifies extended ID anywhere the ID is required
#define MSCAN_ID_RTR 0x40000000	// turns on the RTR bit when calling MSCANMB_SET_ID (only)

/****************************************************************************
* MSCAN register bit names
*****************************************************************************/

/* CANCTL0 */
#define MSCAN_CANCTL0_SFTRES 0x0001
#define MSCAN_CANCTL0_SLPRQ  0x0002
#define MSCAN_CANCTL0_SLPAK  0x0004
#define MSCAN_CANCTL0_SYNCH  0x0010
#define MSCAN_CANCTL0_CSWAI  0x0020
#define MSCAN_CANCTL0_RXACT  0x0040
#define MSCAN_CANCTL0_RXFRM  0x0080

/* CANCTL1 */
#define MSCAN_CANCTL1_CANE   0x0080
#define MSCAN_CANCTL1_LOOPB  0x0004
#define MSCAN_CANCTL1_WUPM   0x0002
#define MSCAN_CANCTL1_CLKSRC 0x0001

/* CANBTR0 */
#define MSCAN_CANBTR0_SJW_MASK 0x00c0
#define MSCAN_CANBTR0_BRP_MASK 0x003f

/* CANBTR1 */
#define MSCAN_CANBTR1_SAMP       0x0080
#define MSCAN_CANBTR1_TSEG2_MASK 0x0070
#define MSCAN_CANBTR1_TSEG1_MASK 0x000f

/* CANRFLG */
#define MSCAN_CANRFLG_RXF    0x0001
#define MSCAN_CANRFLG_OVRIF  0x0002
#define MSCAN_CANRFLG_BOFFIF 0x0004
#define MSCAN_CANRFLG_TERRIF 0x0008
#define MSCAN_CANRFLG_RERRIF 0x0010
#define MSCAN_CANRFLG_TWRNIF 0x0020
#define MSCAN_CANRFLG_RWRNIF 0x0040
#define MSCAN_CANRFLG_WUPIF  0x0080
#define MSCAN_CANRFLG_ERRF_MASK 0x007e

/* CANRIER */
#define MSCAN_CANRIER_RXFIE  0x0001
#define MSCAN_CANRIER_OVRIE  0x0002
#define MSCAN_CANRIER_BOFFIE 0x0004
#define MSCAN_CANRIER_TERRIE 0x0008
#define MSCAN_CANRIER_RERRIE 0x0010
#define MSCAN_CANRIER_TWRNIE 0x0020
#define MSCAN_CANRIER_RWRNIE 0x0040
#define MSCAN_CANRIER_WUPIE  0x0080
#define MSCAN_CANRIER_IE_MASK 0x00ff

/* CANTFLG */
#define MSCAN_CANTFLG_TXE0   0x0001
#define MSCAN_CANTFLG_TXE1   0x0002
#define MSCAN_CANTFLG_TXE2   0x0004
#define MSCAN_CANTFLG_ABTAK0 0x0010
#define MSCAN_CANTFLG_ABTAK1 0x0020
#define MSCAN_CANTFLG_ABTAK2 0x0040
#define MSCAN_CANTFLG_TXEF_MASK 0x0007

/* CANTCR */
#define MSCAN_CANTCR_TXEIE0  0x0001
#define MSCAN_CANTCR_TXEIE1  0x0002
#define MSCAN_CANTCR_TXEIE2  0x0004
#define MSCAN_CANTCR_ABTRQ0  0x0010
#define MSCAN_CANTCR_ABTRQ1  0x0020
#define MSCAN_CANTCR_ABTRQ2  0x0040
#define MSCAN_CANTCR_IE_MASK 0x0007

/* CANIDAC */
#define MSCAN_CANIDAC_IDAM_MASK   0x0030
#define MSCAN_CANIDAC_IDAM_2X32   0x0000
#define MSCAN_CANIDAC_IDAM_4X16   0x0010
#define MSCAN_CANIDAC_IDAM_8X8    0x0020
#define MSCAN_CANIDAC_IDAM_CLOSED 0x0030
#define MSCAN_CANIDAC_IDAM_IDHIT2 0x0004
#define MSCAN_CANIDAC_IDAM_IDHIT1 0x0002
#define MSCAN_CANIDAC_IDAM_IDHIT0 0x0001

/* MB */

#define MSCAN_MB_IDR1_SRTR 0x0010
#define MSCAN_MB_IDR1_IDE  0x0008
#define MSCAN_MB_IDR3_ERTR 0x0001

/****************************************************************************
* MSCAN function prototypes
*****************************************************************************/

extern void mscan_Init(arch_sMSCAN *pMSCanBase);

/***********************************************************************
* MSCAN inline functions
***********************************************************************/

// ID to IDR
inline UWord32 MSCAN_Id2Idr(UWord32 id)
{
	// Extended ID ?
	if(id & 0x80000000)
		return (((id & 0x1FFC0000L) << 3) | 0x00180000L | ((id & 0x0003FFFFL) << 1));
	// Standard ID !
	else
		return id << 21;
}

// IDR to ID
inline UWord32 MSCAN_Idr2Id(UWord32 idr)
{
	// Extended ID ?
	if(idr & 0x80000L)
		return ((idr >> 1) & 0x3FFFFL) | ((idr >> 3) & 0x1FFC0000) | 0x80000000;
	// Standard ID !
	else
		return idr >> 21;
}

// same as above but implemented as a function call (use when passing a variable)
UWord32 MSCAN_Id2Idr_V(register UWord32 id);
UWord32 MSCAN_Idr2Id_V(register UWord32 idr);

/***********************************************************************
* MSCAN control command macros
***********************************************************************/

/* MSCAN initialization */
#define ioctlMSCAN_INIT(pMSCanBase, param) mscan_Init((arch_sMSCAN*)pMSCanBase)

#define ioctlMSCAN_SOFT_RESET(pMSCanBase, param)		\
  if (param) periphBitSet(MSCAN_CANCTL0_SFTRES,&pMSCanBase->canctl0); \
  else periphBitClear(MSCAN_CANCTL0_SFTRES,&pMSCanBase->canctl0)

#define ioctlMSCAN_SLEEP(pMSCanBase, param)		\
  if (param) periphBitSet(MSCAN_CANCTL0_SLPRQ,&pMSCanBase->canctl0); \
  else periphBitClear(MSCAN_CANCTL0_SLPRQ,&pMSCanBase->canctl0)

#define ioctlMSCAN_WAIT_SLEEP(pMSCanBase, param)		\
	while(!periphBitTest(MSCAN_CANCTL0_SLPAK,&pMSCanBase->canctl0));

#define ioctlMSCAN_GET_SLEEP_MODE(pMSCanBase, param)		\
  periphBitTest(MSCAN_CANCTL0_SLPRQ | MSCAN_CANCTL0_SLPAK,&pMSCanBase->canctl0)

#define ioctlMSCAN_TEST_SYNCH(pMSCanBase, param)		\
  periphBitTest(MSCAN_CANCTL0_SYNCH,&pMSCanBase->canctl0)

/* Enable low-power in wait mode; MSCAN_ON/MSCAN_OFF */
#define ioctlMSCAN_STOP_IN_WAIT(pMSCanBase, param)		\
  if (param) periphBitSet(MSCAN_CANCTL0_CSWAI,&pMSCanBase->canctl0); \
  else periphBitClear(MSCAN_CANCTL0_CSWAI,&pMSCanBase->canctl0)
	
/* Enable/disable the MSCAN peripheral; MSCAN_ENABLE/MSCAN_DISABLE */
#define ioctlMSCAN_DEVICE(pMSCanBase, param)		\
  if (param) periphBitSet(MSCAN_CANCTL1_CANE,&pMSCanBase->canctl1); \
  else periphBitClear(MSCAN_CANCTL1_CANE,&pMSCanBase->canctl1)

/* Enable/disable test loopback mode; MSCAN_ENABLE/MSCAN_DISABLE */
#define ioctlMSCAN_LOOPBACK_MODE(pMSCanBase, param)		\
  if (param) periphBitSet(MSCAN_CANCTL1_LOOPB,&pMSCanBase->canctl1); \
  else periphBitClear(MSCAN_CANCTL1_LOOPB,&pMSCanBase->canctl1)
  
/* Enable/disable low-pass filter on wake-up pin; MSCAN_ENABLE/MSCAN_DISABLE */
#define ioctlMSCAN_WAKEUP_FILTER(pMSCanBase, param)		\
  if (param) periphBitSet(MSCAN_CANCTL1_WUPM,&pMSCanBase->canctl1); \
  else periphBitClear(MSCAN_CANCTL1_WUPM,&pMSCanBase->canctl1)

/* select clock source; MSCAN_IPBUS / MSCAN_XTAL */
#define ioctlMSCAN_SET_CLOCK_SOURCE(pMSCanBase, param)		\
  if (param) periphBitSet(MSCAN_CANCTL1_CLKSRC,&pMSCanBase->canctl1); \
  else periphBitClear(MSCAN_CANCTL1_CLKSRC,&pMSCanBase->canctl1)
  
/* set BRP; number 1, .. 64 */
#define ioctlMSCAN_SET_PRESCALER(pMSCanBase, param) 	\
  periphBitGrpRS (MSCAN_CANBTR0_BRP_MASK, param - 1,&pMSCanBase->canbtr0)

/* set SJW; number 1, .. 4 */
#define ioctlMSCAN_SET_SJW(pMSCanBase, param) 	\
  periphBitGrpRS (MSCAN_CANBTR0_SJW_MASK, (param - 1) << 6,&pMSCanBase->canbtr0)

/* set TSEG1; number 1, .. 16 */
#define ioctlMSCAN_SET_TSEG1(pMSCanBase, param) 	\
  periphBitGrpRS (MSCAN_CANBTR1_TSEG1_MASK, param - 1,&pMSCanBase->canbtr1)

/* set TSEG2; number 1, .. 8 */
#define ioctlMSCAN_SET_TSEG2(pMSCanBase, param) 	\
  periphBitGrpRS (MSCAN_CANBTR1_TSEG2_MASK, (param - 1) << 4,&pMSCanBase->canbtr1)

/* set samples per bit; MSCAN_1SAMP_PER_BIT, MSCAN_3SAMPS_PER_BIT,  */
#define ioctlMSCAN_SET_SAMPLING(pMSCanBase, param)		\
  if (param) periphBitSet(MSCAN_CANBTR1_SAMP,&pMSCanBase->canbtr1); \
  else periphBitClear(MSCAN_CANBTR1_SAMP,&pMSCanBase->canbtr1)

#define ioctlMSCAN_ERINT_ENABLE(pMSCanBase, param) 	\
  periphBitSet ((param) & MSCAN_CANRIER_IE_MASK,&pMSCanBase->canrier)
  
#define ioctlMSCAN_ERINT_DISABLE(pMSCanBase, param) \
  periphBitClear ((param) & MSCAN_CANRIER_IE_MASK,&pMSCanBase->canrier)
    
#define ioctlMSCAN_TINT_ENABLE(pMSCanBase, param) 	\
  periphBitSet ((param) & MSCAN_CANTCR_IE_MASK,&pMSCanBase->cantcr)

#define ioctlMSCAN_TINT_DISABLE(pMSCanBase, param) 	\
  periphBitClear ((param) & MSCAN_CANTCR_IE_MASK,&pMSCanBase->cantcr)

/* Get Error Interrupt status flags (RFLG except RXF & WUP); NULL */
#define ioctlMSCAN_READ_EINT_FLAGS(pMSCanBase, param) 	\
  periphBitTest (MSCAN_CANRFLG_ERRF_MASK,&pMSCanBase->canrflg)

/* Get Transmitter Interrupt status flags (all TXEn bits); NULL */
#define ioctlMSCAN_READ_TINT_FLAGS(pMSCanBase, param) 	\
  periphBitTest (MSCAN_CANTFLG_TXEF_MASK,&pMSCanBase->cantflg)

/* Send TX buffers (Clear selected Transmitter Interrupt flags) */
#define ioctlMSCAN_TRANSMIT(pMSCanBase, param) 	\
  periphMemWrite ((param) & MSCAN_CANTFLG_TXEF_MASK,&pMSCanBase->cantflg)

/* Clear selected Error Interrupt flags */
#define ioctlMSCAN_CLEAR_EINT_FLAGS(pMSCanBase, param) 	\
  periphMemWrite ((param) & MSCAN_CANRFLG_ERRF_MASK,&pMSCanBase->canrflg)

/* Clear selected RXF Interrupt flag */
#define ioctlMSCAN_CLEAR_RINT_FLAG(pMSCanBase, param) 	\
  periphMemWrite (MSCAN_CANRFLG_RXF,&pMSCanBase->canrflg)

/* Clear selected WakeUp Interrupt flag */
#define ioctlMSCAN_CLEAR_WINT_FLAG(pMSCanBase, param) 	\
  periphMemWrite (MSCAN_CANRFLG_WUPIF,&pMSCanBase->canrflg)

/* set acceptance mode; MSCAN_ACC_MODE_2X32, ... */
#define ioctlMSCAN_SET_ACC_MODE(pMSCanBase, param) 		\
  periphBitGrpRS (MSCAN_CANIDAC_IDAM_MASK, param,&pMSCanBase->canidac)

/* set 32bit acceptance mask 0 */
#define ioctlMSCAN_SET_ACC_MASKR_32_0(pMSCanBase, param)  { \
	periphMemWrite((UWord16)(param),&pMSCanBase->canidmr3);			\
	periphMemWrite((UWord16)((param) >> 16),&pMSCanBase->canidmr1);	\
	periphMemWrite((UWord16)((param) >>  8),&pMSCanBase->canidmr2);	\
	periphMemWrite((UWord16)((param) >> 24),&pMSCanBase->canidmr0);	\
  }

/* set 32bit acceptance mask 1 */
#define ioctlMSCAN_SET_ACC_MASKR_32_1(pMSCanBase, param)  { \
	periphMemWrite((UWord16)(param),&pMSCanBase->canidmr7);			\
	periphMemWrite((UWord16)((param) >> 16),&pMSCanBase->canidmr5);	\
	periphMemWrite((UWord16)((param) >>  8),&pMSCanBase->canidmr6);	\
	periphMemWrite((UWord16)((param) >> 24),&pMSCanBase->canidmr4);	\
  }

/* set 16bit acceptance mask 0 */
#define ioctlMSCAN_SET_ACC_MASKR_16_0(pMSCanBase, param)  { \
	periphMemWrite((UWord16)(param),&pMSCanBase->canidmr1);		\
	periphMemWrite((UWord16)((param) >> 8),&pMSCanBase->canidmr0);\
  }

/* set 16bit acceptance mask 1 */
#define ioctlMSCAN_SET_ACC_MASKR_16_1(pMSCanBase, param)  { \
	periphMemWrite((UWord16)(param),&pMSCanBase->canidmr3);		\
	periphMemWrite((UWord16)((param) >> 8),&pMSCanBase->canidmr2);\
  }

/* set 16bit acceptance mask 2 */
#define ioctlMSCAN_SET_ACC_MASKR_16_2(pMSCanBase, param)  { \
	periphMemWrite((UWord16)(param),&pMSCanBase->canidmr5);		\
	periphMemWrite((UWord16)((param) >> 8),&pMSCanBase->canidmr4);\
  }

/* set 16bit acceptance mask 3 */
#define ioctlMSCAN_SET_ACC_MASKR_16_3(pMSCanBase, param)  { \
	periphMemWrite((UWord16)(param),&pMSCanBase->canidmr7);		\
	periphMemWrite((UWord16)((param) >> 8),&pMSCanBase->canidmr6);\
  }

/* set 8bit acceptance mask 0 */
#define ioctlMSCAN_SET_ACC_MASKR_8_0(pMSCanBase, param)   \
	periphMemWrite((UWord16)(param),&pMSCanBase->canidmr0)

/* set 8bit acceptance mask 1 */
#define ioctlMSCAN_SET_ACC_MASKR_8_1(pMSCanBase, param)   \
	periphMemWrite((UWord16)(param),&pMSCanBase->canidmr1)

/* set 8bit acceptance mask 2 */
#define ioctlMSCAN_SET_ACC_MASKR_8_2(pMSCanBase, param)   \
	periphMemWrite((UWord16)(param),&pMSCanBase->canidmr2)

/* set 8bit acceptance mask 3 */
#define ioctlMSCAN_SET_ACC_MASKR_8_3(pMSCanBase, param)   \
	periphMemWrite((UWord16)(param),&pMSCanBase->canidmr3)

/* set 8bit acceptance mask 4 */
#define ioctlMSCAN_SET_ACC_MASKR_8_4(pMSCanBase, param)   \
	periphMemWrite((UWord16)(param),&pMSCanBase->canidmr4)

/* set 8bit acceptance mask 5 */
#define ioctlMSCAN_SET_ACC_MASKR_8_5(pMSCanBase, param)   \
	periphMemWrite((UWord16)(param),&pMSCanBase->canidmr5)

/* set 8bit acceptance mask 6 */
#define ioctlMSCAN_SET_ACC_MASKR_8_6(pMSCanBase, param)   \
	periphMemWrite((UWord16)(param),&pMSCanBase->canidmr6)

/* set 8bit acceptance mask 7 */
#define ioctlMSCAN_SET_ACC_MASKR_8_7(pMSCanBase, param)   \
	periphMemWrite((UWord16)(param),&pMSCanBase->canidmr7)

//////////////////

/* set 32bit acceptance mask 0 */
#define ioctlMSCAN_SET_ACC_IDR_32_0(pMSCanBase, param)  { \
	periphMemWrite((UWord16)(param),&pMSCanBase->canidar3);			\
	periphMemWrite((UWord16)((param) >> 16),&pMSCanBase->canidar1);	\
	periphMemWrite((UWord16)((param) >>  8),&pMSCanBase->canidar2);	\
	periphMemWrite((UWord16)((param) >> 24),&pMSCanBase->canidar0);	\
  }

/* set 32bit acceptance mask 1 */
#define ioctlMSCAN_SET_ACC_IDR_32_1(pMSCanBase, param)  { \
	periphMemWrite((UWord16)(param),&pMSCanBase->canidar7);			\
	periphMemWrite((UWord16)((param) >> 16),&pMSCanBase->canidar5);	\
	periphMemWrite((UWord16)((param) >>  8),&pMSCanBase->canidar6);	\
	periphMemWrite((UWord16)((param) >> 24),&pMSCanBase->canidar4);	\
  }

/* set 16bit acceptance mask 0 */
#define ioctlMSCAN_SET_ACC_IDR_16_0(pMSCanBase, param)  { \
	periphMemWrite((UWord16)(param),&pMSCanBase->canidar1);		\
	periphMemWrite((UWord16)((param) >> 8),&pMSCanBase->canidar0);\
  }

/* set 16bit acceptance mask 1 */
#define ioctlMSCAN_SET_ACC_IDR_16_1(pMSCanBase, param)  { \
	periphMemWrite((UWord16)(param),&pMSCanBase->canidar3);		\
	periphMemWrite((UWord16)((param) >> 8),&pMSCanBase->canidar2);\
  }

/* set 16bit acceptance mask 2 */
#define ioctlMSCAN_SET_ACC_IDR_16_2(pMSCanBase, param)  { \
	periphMemWrite((UWord16)(param),&pMSCanBase->canidar5);		\
	periphMemWrite((UWord16)((param) >> 8),&pMSCanBase->canidar4);\
  }

/* set 16bit acceptance mask 3 */
#define ioctlMSCAN_SET_ACC_IDR_16_3(pMSCanBase, param)  { \
	periphMemWrite((UWord16)(param),&pMSCanBase->canidar7);		\
	periphMemWrite((UWord16)((param) >> 8),&pMSCanBase->canidar6);\
  }

/* set 8bit acceptance mask 0 */
#define ioctlMSCAN_SET_ACC_IDR_8_0(pMSCanBase, param)   \
	periphMemWrite((UWord16)(param),&pMSCanBase->canidar0)

/* set 8bit acceptance mask 1 */
#define ioctlMSCAN_SET_ACC_IDR_8_1(pMSCanBase, param)   \
	periphMemWrite((UWord16)(param),&pMSCanBase->canidar1)

/* set 8bit acceptance mask 2 */
#define ioctlMSCAN_SET_ACC_IDR_8_2(pMSCanBase, param)   \
	periphMemWrite((UWord16)(param),&pMSCanBase->canidar2)

/* set 8bit acceptance mask 3 */
#define ioctlMSCAN_SET_ACC_IDR_8_3(pMSCanBase, param)   \
	periphMemWrite((UWord16)(param),&pMSCanBase->canidar3)

/* set 8bit acceptance mask 4 */
#define ioctlMSCAN_SET_ACC_IDR_8_4(pMSCanBase, param)   \
	periphMemWrite((UWord16)(param),&pMSCanBase->canidar4)

/* set 8bit acceptance mask 5 */
#define ioctlMSCAN_SET_ACC_IDR_8_5(pMSCanBase, param)   \
	periphMemWrite((UWord16)(param),&pMSCanBase->canidar5)

/* set 8bit acceptance mask 6 */
#define ioctlMSCAN_SET_ACC_IDR_8_6(pMSCanBase, param)   \
	periphMemWrite((UWord16)(param),&pMSCanBase->canidar6)

/* set 8bit acceptance mask 7 */
#define ioctlMSCAN_SET_ACC_IDR_8_7(pMSCanBase, param)   \
	periphMemWrite((UWord16)(param),&pMSCanBase->canidar7)

#define ioctlMSCAN_GET_RX_ERR_COUNT(pMSCanBase, param)   \
	periphMemRead(&pMSCanBase->canrxerr)

#define ioctlMSCAN_GET_TX_ERR_COUNT(pMSCanBase, param)   \
	periphMemRead(&pMSCanBase->cantxerr)

//////////////////////

/* Get pointer to Receiver's MB structure; NULL */
#define ioctlMSCAN_GET_RB_MODULE(pMSCanBase, param) 	\
  (&pMSCanBase->canrb)

/* Get pointer to Transmitter's MB structure; TB index (0,1,2) */
#define ioctlMSCAN_GET_TB_MODULE(pMSCanBase, param) 	\
  (&pMSCanBase->cantb[param])

//////////////////////
// MB commands

/* Get translated ID */
asm UWord32 ioctlMSCANMB_GET_ID(arch_sMSCAN_MB* pMSCanMB, void* param);

/* Get four IDR registers of given MB as a single 32 bit word */
/* unused bits are reset to 0 */
static asm UWord32 ioctlMSCANMB_GET_ID_RAW(arch_sMSCAN_MB* pMSCanMB, void* param)
{	lea		(SP)+
	move	X0,X:(SP)+
	move	Y0,X:(SP)
	
	move	#8,X0
	move	X:(R2+0),Y0
	asll	Y0,X0,A
	move	X:(R2+1),Y0
	or		Y0,A
	brclr	#8,A1,Finish
	
	move	X:(R2+2),Y0
	asll	Y0,X0,X0
	move	X:(R2+3),Y0
	or		X0,Y0
	move	Y0,A0

Finish:

	move	X:(SP)-,Y0
	move	X:(SP)-,X0
	rts
}

/* Get length field of the MB; NULL */
#define ioctlMSCANMB_GET_LEN(pMSCanMB, param) 	\
	periphMemRead(&pMSCanMB->dlr)

/* Return non-zero if RTR is set in given MB; NULL */
#define ioctlMSCANMB_GET_RTR(pMSCanMB, param) 	\
	( periphBitTest(MSCAN_MB_IDR1_IDE, &pMSCanMB->idr1) ?  \
	    periphBitTest(MSCAN_MB_IDR3_ERTR, &pMSCanMB->idr3) : \
	    periphBitTest(MSCAN_MB_IDR1_SRTR, &pMSCanMB->idr1) )\
	
#define ioctlMSCANMB_GET_DATAPTR(pMSCanMB, param) 	\
	((UWord16 *)(pMSCanMB->dsr))

//////////////////////

/* Set Data Length Register for given MB; len */
#define ioctlMSCANMB_SET_LEN(pMSCanMB, param) 	\
	periphMemWrite(param, &pMSCanMB->dlr)

/* Set Transmit Buffer Priority for given MB; ; priority */
#define ioctlMSCANMB_SET_TBP(pMSCanMB, param) 	\
	periphMemWrite(param, &pMSCanMB->tbpr)

/* Set four IDR registers of given MB using a single 32 bit word */
static asm void ioctlMSCANMB_SET_ID_RAW(arch_sMSCAN_MB* pMSCanMB, register UWord32 idr)
{	lea		(SP)+
	move	Y0,X:(SP)+
	move	Y1,X:(SP)
	
	move	A1,X:(R2+1)
	move	A0,X:(R2+3)

	move	#8,Y0
	lsrr	A1,Y0,Y1
	move	Y1,X:(R2+0)
	move	A0,Y1
	lsrr	Y1,Y0,Y0
	move	Y0,X:(R2+2)

	move	X:(SP)-,Y1
	move	X:(SP)-,Y0
	rts
}

inline void ioctlMSCANMB_SET_ID(arch_sMSCAN_MB* pMSCanMB, register UWord32 id)
{
	/* Note: we use inline function because mostly the ID is specified as constant so
          the ID->IDR conversion is done by compiler and the code is optimal */
   
	register UWord32 idr = MSCAN_Id2Idr(id);

	// set RTR ?
	if(id & MSCAN_ID_RTR)
		idr |= MSCAN_MB_IDR3_ERTR | (((UWord32)MSCAN_MB_IDR1_SRTR) << 16);
				
	// store into MB
	ioctl(pMSCanMB, MSCANMB_SET_ID_RAW, idr);	
}

/* same as above but done as function (to be used when ID passed is a variable) */
void ioctlMSCANMB_SET_ID_V(register arch_sMSCAN_MB* pMSCanMB, register UWord32 id);

/* Write given ID to the apropriate bits in given MB; MSCAN_ON/MSCAN_OFF */
#define ioctlMSCANMB_SET_RTR(pMSCanMB, param) 	\
  if(param) { \
    periphBitSet(MSCAN_MB_IDR1_SRTR, &pMSCanMB->idr1); \
    periphBitSet(MSCAN_MB_IDR3_ERTR, &pMSCanMB->idr3); \
  }			\
  else {	\
    if(periphBitTest(MSCAN_MB_IDR1_IDE, &pMSCanMB->idr1))  \
   	  periphBitClear(MSCAN_MB_IDR3_ERTR, &pMSCanMB->idr3); \
   	else \
   	  periphBitClear(MSCAN_MB_IDR1_SRTR, &pMSCanMB->idr1); \
  }


/********************************************************
 * End-of-life commands - do not use in new development
 *******************************************************/

#define ioctlMSCAN_ERINT_SELECT(pMSCanBase, param) 	\
  periphBitGrpRS (MSCAN_CANRIER_IE_MASK, param,&pMSCanBase->canrier)

#define ioctlMSCAN_TINT_SELECT(pMSCanBase, param) 	\
  periphBitGrpRS (MSCAN_CANTCR_IE_MASK, param,&pMSCanBase->cantcr)

#define ioctlMSCAN_READ_RXERR(pMSCanBase, param)   \
	periphMemRead(&pMSCanBase->canrxerr)

#define ioctlMSCAN_READ_TXERR(pMSCanBase, param)   \
	periphMemRead(&pMSCanBase->cantxerr)

#define ioctlMSCAN_GET_EINT_STATUS(pMSCanBase, param) 	\
  periphBitTest (MSCAN_CANRFLG_ERRF_MASK,&pMSCanBase->canrflg)

#define ioctlMSCAN_GET_TINT_STATUS(pMSCanBase, param) 	\
  periphBitTest (MSCAN_CANTFLG_TXEF_MASK,&pMSCanBase->cantflg)

#define ioctlMSCAN_CLEAR_EINT_STATUS(pMSCanBase, param) 	\
  periphMemWrite ((param) & MSCAN_CANRFLG_ERRF_MASK,&pMSCanBase->canrflg)

#define ioctlMSCAN_CLEAR_RINT_STATUS(pMSCanBase, param) 	\
  periphMemWrite (MSCAN_CANRFLG_RXF,&pMSCanBase->canrflg)

#define ioctlMSCAN_CLEAR_WINT_STATUS(pMSCanBase, param) 	\
  periphMemWrite (MSCAN_CANRFLG_WUPIF,&pMSCanBase->canrflg)


#ifdef __cplusplus
}
#endif

#endif
