/******************************************************************************
*
* Motorola Inc.
* (c) Copyright 2002 Motorola, Inc.
* ALL RIGHTS RESERVED.
*
*******************************************************************************
*
* FILE NAME: pwm.h
*
* DESCRIPTION: Header file for the PWM driver
*
* MODULES INCLUDED: pwmtarget.h
*
* NOTES:
*
*******************************************************************************/

#ifndef __PWM_H
#define __PWM_H

#if !defined(__ARCH_H) || !defined(__PERIPH_H)
#error arch.h and periph.h must be included before pwm.h
#endif

#if !defined(__MC_H)
#error mc.h must be included before pwm.h
#endif

#ifdef __cplusplus
extern "C" {
#endif

/******************************************************************************
*
*                      General Interface Description
*
*
* The Pulse Width Modulator (PWM) modules incorporate three complementary, individually
* programmable PWM signal outputs.
* Each module is also capable of supporting six independent PWM functions 
* to enhance motor control functionality. 
* Complementary operation permits programmable dead-time insertion,
* distortion correction via current sensing by software, and separate top and bottom output
* polarity control. The up-counter value is programmable to support a continuously variable
* PWM frequency. Both edge- and center-aligned synchronous pulse width control (full 0%
* to 100% modulation) are supported. 
*
* The device is capable of controlling most motor types: 
*      ACIM (AC Induction Motors)
*      BDC and BLDC (Brush and Brushless DC motors)
*      SRM and VRM (Switched and Variable Reluctance Motors)
*      Stepper motors
*
* The PWMs incorporate fault protection and cycle-by-cycle current limiting with sufficient
* output drive capability to directly drive standard opto-isolators.
*
* Write-once protection feature for key parameters is also included. A patented PWM
* waveform distortion correction circuit is also provided. Each PWM is double-buffered and
* includes interrupt controls to permit integral reload rates to be programmable from 1 to 16.
* The PWM modules also provide a reference output to synchronize the Analog-to-Digital
* Converters.
*
* For more information, please refer to the DSP56F80x User's Manual for a description
* of the PWM device.
*
* The PWM device driver interface uses "ioctl" call to control specific PWM functions
* during operation. For details see "ioctl" call.
*
******************************************************************************/

/************************************************************
* PWM module identifiers, correspond to module base address
*************************************************************/

#define PWM_A  (&ArchIO.PwmA)
#define PWM_B  (&ArchIO.PwmB)

/****************************************************
* Configurable items, i.e. defines, for appconfig.h
*****************************************************

  #define INT_VECTOR_ADDR_yy            pwmxISR   // specify interrupt handler routine
  #define INT_PRIORITY_LEVEL_yy  	// value 0-7 (0 = interrupt disabled)

  #define  PWM_x_PMCTL_INIT    	0x0000 
  #define  PWM_x_PMFCTL_INIT    0x0000
  #define  PWM_x_PMFSA_INIT    	0x0000
  #define  PWM_x_PMOUT_INIT    	0x0000
  #define  PWM_x_PWMCM_INIT    	0x0000
  #define  PWM_x_PMDEADTM_INIT  0x0000
  #define  PWM_x_PMDISMAP1_INIT 0x0000
  #define  PWM_x_PMDISMAP2_INIT 0x0000
  #define  PWM_x_PMCFG_INIT 	0x0000
  #define  PWM_x_PMCCR_INIT	    0x0000

  where:
         x  is PWM module (A or B)
         yy  is interrupt vector number
                61 - PWM A Fault
                60 - PWM B Fault
                59 - PWM A Reload
                58 - PWM B Reload
*/

/*****************************************************************
* ioctl() commands
*
*  Word16 ioctl( PWM_module_identifier, Command_name, Parameter );
*
******************************************************************

 COMMAND NAME                             PARAMETERS, COMMENTS
------------------------------------------------------------------ */
#define PWM_INIT                          /* NULL */

#define PWM_SET_RELOAD_FREQUENCY          /* PWM_RELOAD_OPPORTUNITY_X */ 
#define PWM_HALF_CYCLE_RELOAD             /* PWM_ENABLE / PWM_DISABLE  */
#define PWM_SET_CURRENT_POLARITY          /* (PWM_IPOL_0 | PWM_IPOL_1 | PWM_IPOL_2) / PWM_ZERO_MASK  */
#define PWM_SET_PRESCALER                 /* PWM_PRESCALER_DIV_X  */
#define PWM_RELOAD_INT                    /* PWM_ENABLE / PWM_DISABLE  */
#define PWM_SET_CURRENT_SENSING           /* PWM_CORRECTION_XX  */
#define PWM_DEVICE                        /* PWM_ENABLE / PWM_DISABLE  */
#define PWM_CLEAR_RELOAD_FLAG             /* NULL  */
#define PWM_LOAD_OK                       /* NULL  */

#define PWM_FAULT_INT_ENABLE              /* PWM_FAULT_x  */
#define PWM_FAULT_INT_DISABLE             /* PWM_FAULT_x  */

#define PWM_SET_AUTOMATIC_FAULT_CLEAR     /* PWM_FAULT_x  */
#define PWM_SET_MANUAL_FAULT_CLEAR        /* PWM_FAULT_x  */

#define PWM_CLEAR_FAULT_FLAG              /* PWM_FAULT_x  */

#define PWM_OUTPUT_PAD                    /* PWM_ENABLE / PWM_DISABLE  */
#define PWM_OUTPUT_SOFTWARE_CONTROL       /* mc_tPWMSignalMask  */
#define PWM_OUTPUT_CONTROL                /* mc_tPWMSignalMask  */
 
#define PWM_SET_MODULO                    /* value */
#define PWM_GET_MODULO                    /* NULL  */
         
#define PWM_SET_DEADTIME                  /* value  */
         
#define PWM_WRITE_DISABLE_MAPPING_REG1    /* value  */
#define PWM_WRITE_DISABLE_MAPPING_REG2    /* value  */

#define PWM_SET_ALIGNMENT                 /* PWM_ALIGN_EDGE / PWM_ALIGN_CENTER  */
#define PWM_SET_NEG_TOP_SIDE_POLARITY     /* (PWM_CHANNEL_45 | PWM_CHANNEL_23 | PWM_CHANNEL_01) / PWM_ZERO_MASK  */
#define PWM_SET_NEG_BOTTOM_SIDE_POLARITY  /* (PWM_CHANNEL_45 | PWM_CHANNEL_23 | PWM_CHANNEL_01) / PWM_ZERO_MASK  */
#define PWM_SET_INDEPENDENT_OPERATION     /* (PWM_CHANNEL_45 | PWM_CHANNEL_23 | PWM_CHANNEL_01) / PWM_ZERO_MASK */
#define PWM_SET_INDEPENDENT_MODE          /* PWM_CHANNEL_45 | PWM_CHANNEL_23 | PWM_CHANNEL_01 */
#define PWM_SET_COMPLEMENTARY_MODE        /* PWM_CHANNEL_45 | PWM_CHANNEL_23 | PWM_CHANNEL_01 */
#define PWM_SET_WRITE_PROTECT             /* NULL  */
          
#define PWM_HARDWARE_ACCELERATION         /* PWM_ENABLE / PWM_DISABLE  */
#define PWM_SET_CHANNEL_MASK              /* mc_tPWMSignalMask  */
#define PWM_SET_LOAD_MODE                 /* PWM_LOAD_xx  */
#define PWM_SET_SWAP                      /* (PWM_CHANNEL_45 | PWM_CHANNEL_23 | PWM_CHANNEL_01) / PWM_ZERO_MASK */

#define PWM_WRITE_VALUE_REG_0             /* value  */
#define PWM_WRITE_VALUE_REG_1             /* value  */
#define PWM_WRITE_VALUE_REG_2             /* value  */
#define PWM_WRITE_VALUE_REG_3             /* value  */
#define PWM_WRITE_VALUE_REG_4             /* value  */
#define PWM_WRITE_VALUE_REG_5             /* value  */
#define PWM_WRITE_VALUE_REGS_COMPL        /* pwm_sComplementaryValues*  */
#define PWM_WRITE_VALUE_REGS_INDEP        /* pwm_sIndependentValues*  */
#define PWM_READ_FAULT_STATUS_REG         /* NULL  */
#define PWM_READ_COUNTER_REG              /* NULL */
#define PWM_READ_CONTROL_REG              /* NULL  */
#define PWM_READ_PORT_REG                 /* NULL  */

#define PWM_SOFTWARE_OUTPUTS_CONTROL      /* pwm_sOutputControl*  */
#define PWM_UPDATE_VALUE_REG_0            /* value  */
#define PWM_UPDATE_VALUE_REG_1            /* value  */
#define PWM_UPDATE_VALUE_REG_2            /* value  */
#define PWM_UPDATE_VALUE_REG_3            /* value  */
#define PWM_UPDATE_VALUE_REG_4            /* value  */
#define PWM_UPDATE_VALUE_REG_5            /* value  */
#define PWM_UPDATE_VALUE_REGS_COMPL       /* pwm_sComplementaryValues*  */
#define PWM_UPDATE_VALUE_REGS_INDEP       /* pwm_sIndependentValues*  */  
#define PWM_UPDATE_VALUE_SET_VLMODE       /* pwm_sUpdateValueSetVlmode*  */    
#define PWM_CORRECT_DEAD_TIME_COMPL       /* pwm_sComplementaryValues*  */ 
#define PWM_SET_MASK_SWAP                 /* pwm_sChannelControl*  */    


/****************************************************************************
* PWM constants and type declarations use in the ioctl functions
*****************************************************************************/

#define PWM_ENABLE                      true
#define PWM_DISABLE                     false

#define PWM_ZERO_MASK                   0x0000   

#define PWM_IPOL_0                      PWM_PMCTL_IPOL_0
#define PWM_IPOL_1                      PWM_PMCTL_IPOL_1
#define PWM_IPOL_2                      PWM_PMCTL_IPOL_2

#define PWM_RELOAD_OPPORTUNITY_1        0
#define PWM_RELOAD_OPPORTUNITY_2        1
#define PWM_RELOAD_OPPORTUNITY_3        2
#define PWM_RELOAD_OPPORTUNITY_4        3
#define PWM_RELOAD_OPPORTUNITY_5        4
#define PWM_RELOAD_OPPORTUNITY_6        5
#define PWM_RELOAD_OPPORTUNITY_7        6
#define PWM_RELOAD_OPPORTUNITY_8        7
#define PWM_RELOAD_OPPORTUNITY_9        8
#define PWM_RELOAD_OPPORTUNITY_10       9
#define PWM_RELOAD_OPPORTUNITY_11       10
#define PWM_RELOAD_OPPORTUNITY_12       11
#define PWM_RELOAD_OPPORTUNITY_13       12
#define PWM_RELOAD_OPPORTUNITY_14       13
#define PWM_RELOAD_OPPORTUNITY_15       14
#define PWM_RELOAD_OPPORTUNITY_16       15

#define PWM_PRESCALER_DIV_1             0
#define PWM_PRESCALER_DIV_2             1
#define PWM_PRESCALER_DIV_4             2
#define PWM_PRESCALER_DIV_8             3

#define PWM_CORRECTION_NO               0
#define PWM_CORRECTION_SOFTWARE         1
#define PWM_CORRECTION_DURING_DEADTIME  2	/* not applicable for DSP56801 */
#define PWM_CORRECTION_DURING_CYCLE     3	/* not applicable for DSP56801 */

#define PWM_FAULT_0                     0	
#define PWM_FAULT_1                     1	/* not applicable for DSP56801 */
#define PWM_FAULT_2                     2	/* not applicable for DSP56801 */
#define PWM_FAULT_3                     3	/* not applicable for DSP56801/803 */

#define PWM_ALIGN_EDGE                  true
#define PWM_ALIGN_CENTER                false

#define PWM_CHANNEL_45                  0x0004
#define PWM_CHANNEL_23                  0x0002
#define PWM_CHANNEL_01                  0x0001

#define PWM_LOAD_INDEP                  0
#define PWM_LOAD_FROM_0_TO_5            1
#define PWM_LOAD_FROM_0_TO_3            2

//#define PWM_COMPLEMENTARY_MODE          0
//#define PWM_INDEPENDENT_MODE            1


/****************************************************************************/

/*  pwm_tPWMChannelSwap contains three control bits
	representing channel swapping of PWM signals.
	The bits can be combined in a numerical value
	that represents the union of the appropriate
	bits.  For example, the value 0x05 indicates
	that PWM channel 0 and 1 are swapped, channel 4 and 5 are swapped. 
*/
typedef UWord16 pwm_tPWMChannelSwap;    

typedef struct
{
	Word16  pwmChannel_0_Value;
	Word16  pwmChannel_2_Value;
	Word16  pwmChannel_4_Value;
}pwm_sComplementaryValues;

typedef struct
{
	Word16  pwmChannel_0_Value;
	Word16  pwmChannel_1_Value;
	Word16  pwmChannel_2_Value;
	Word16  pwmChannel_3_Value;
	Word16  pwmChannel_4_Value;
	Word16  pwmChannel_5_Value;
}pwm_sIndependentValues;

typedef struct
{
	mc_tPWMSignalMask   SoftwareControlled;
	mc_tPWMSignalMask   OutputControl;
}pwm_sOutputControl;

typedef struct
{
	Word16    DutyCycle;
	UWord16   Vlmode;
}pwm_sUpdateValueSetVlmode;

typedef struct
{
	mc_tPWMSignalMask     Mask;
	pwm_tPWMChannelSwap   Swap;
}pwm_sChannelControl;


/*****************************************************************
* PWM Register Bits Defines, used in ioct command implementation
******************************************************************/

/* PWM Control Register PMCTL */
#define PWM_PMCTL_LDFQ_MASK	0xf000   /* Load Frequency Bits */  
#define PWM_PMCTL_LDFQ_0    0x1000   /* Load Frequency Bit 0 */  
#define PWM_PMCTL_LDFQ_1    0x2000   /* Load Frequency Bit 1 */  
#define PWM_PMCTL_LDFQ_2    0x4000   /* Load Frequency Bit 2 */  
#define PWM_PMCTL_LDFQ_3    0x8000   /* Load Frequency Bit 3 */  
#define PWM_PMCTL_HALF      0x0800   /* Half Cycle Reload */
#define PWM_PMCTL_IPOL_MASK 0x0700   /* Current Polarity Bits */  
#define PWM_PMCTL_IPOL_0	0x0100   /* Current Polarity Bit 0 */  
#define PWM_PMCTL_IPOL_1	0x0200   /* Current Polarity Bit 1 */  
#define PWM_PMCTL_IPOL_2	0x0400   /* Current Polarity Bit 2 */  
#define PWM_PMCTL_PRSC_MASK 0x00c0   /* Prescaler Bits */  
#define PWM_PMCTL_PRSC_0    0x0040   /* Prescaler Bit 0 */  
#define PWM_PMCTL_PRSC_1    0x0080   /* Prescaler Bit 1 */  
#define PWM_PMCTL_PWMRIE    0x0020   /* Reload Interrupt Enable Bit */
#define PWM_PMCTL_PWMF      0x0010   /* Reload Flag */
#define PWM_PMCTL_ISENS_MASK 0x000c  /* Current Sense Bits */  
#define PWM_PMCTL_ISENS_0   0x0004   /* Current Sense Bit 0 */  
#define PWM_PMCTL_ISENS_1   0x0008   /* Current Sense Bit 1 */  
#define PWM_PMCTL_LDOK      0x0002   /* Load Okay Bit */
#define PWM_PMCTL_PWMEN     0x0001   /* PWM Enable Bit */

/* PWM Fault Control Register PMFCTL */
#define PWM_PMFCTL_FIE3		0x0080   /* Fault3 Pin Interrupt Enable Bit */
#define PWM_PMFCTL_FMODE3	0x0040   /* Fault3 Pin Clearing Mode Bit */
#define PWM_PMFCTL_FIE2		0x0020   /* Fault2 Pin Interrupt Enable Bit */
#define PWM_PMFCTL_FMODE2	0x0010   /* Fault2 Pin Clearing Mode Bit */
#define PWM_PMFCTL_FIE1		0x0008   /* Fault1 Pin Interrupt Enable Bit */
#define PWM_PMFCTL_FMODE1	0x0004   /* Fault1 Pin Clearing Mode Bit */
#define PWM_PMFCTL_FIE0		0x0002   /* Fault0 Pin Interrupt Enable Bit */
#define PWM_PMFCTL_FMODE0	0x0001   /* Fault0 Pin Clearing Mode Bit */

/* PWM Fault Status & Acknowledge Register PMFSA */
#define PWM_PMFSA_FPIN3     0x8000
#define PWM_PMFSA_FFLAG3    0x4000
#define PWM_PMFSA_FPIN2     0x2000
#define PWM_PMFSA_FFLAG2    0x1000
#define PWM_PMFSA_FPIN1     0x0800
#define PWM_PMFSA_FFLAG1    0x0400
#define PWM_PMFSA_FPIN0     0x0200
#define PWM_PMFSA_FFLAG0    0x0100
#define PWM_PMFSA_DT5	    0x0020
#define PWM_PMFSA_DT4	    0x0010
#define PWM_PMFSA_DT3	    0x0008
#define PWM_PMFSA_DT2	    0x0004
#define PWM_PMFSA_DT1	    0x0002
#define PWM_PMFSA_DT0	    0x0001
#define PWM_PMFSA_FTACK3    0x0040   /* Fault3 Pin Acknowledge Bit */
#define PWM_PMFSA_FTACK2    0x0010   /* Fault2 Pin Acknowledge Bit */
#define PWM_PMFSA_FTACK1    0x0004   /* Fault1 Pin Acknowledge Bit */
#define PWM_PMFSA_FTACK0    0x0001   /* Fault0 Pin Acknowledge Bit */

/* PWM Output Control Register PMOUT */
#define PWM_PMOUT_PAD_EN       0x8000   /* Output Pad Output Enable */
#define PWM_PMOUT_OUTCTL_MASK  0x3f00   /* Output Control Enables OUTCTL5-0 */
#define PWM_PMOUT_OUT_MASK     0x003f   /* Output Control Bits OUT5-0 */

/* PWM Config Register PMCFG */
#define PWM_PMCFG_EDG          0x1000   /* Edge-Aligned or Center-Aligned PWM's */
#define PWM_PMCFG_TOPNEG_MASK  0x0700   /* Top-side PWM Polarity Bits, channel 5-0 */
#define PWM_PMCFG_TOPNEG45     0x0400   /* Top-side PWM Polarity Bit, channel 4-5 */
#define PWM_PMCFG_TOPNEG23     0x0200   /* Top-side PWM Polarity Bit, channel 2-3 */
#define PWM_PMCFG_TOPNEG01     0x0100   /* Top-side PWM Polarity Bit, channel 0-1 */
#define PWM_PMCFG_BOTNEG_MASK  0x0070   /* Bottom-side PWM Polarity Bits, channel 5-0 */
#define PWM_PMCFG_BOTNEG45     0x0040   /* Bottom-side PWM Polarity Bit, channel 4-5 */
#define PWM_PMCFG_BOTNEG23     0x0020   /* Bottom-side PWM Polarity Bit, channel 2-3 */
#define PWM_PMCFG_BOTNEG01     0x0010   /* Bottom-side PWM Polarity Bit, channel 0-1 */
#define PWM_PMCFG_INDEP_MASK   0x000e   /* Independent or Complementary Pair, channel 5-0 */
#define PWM_PMCFG_INDEP45      0x0008   /* Independent or Complementary Pair Operation, channel 4-5 */
#define PWM_PMCFG_INDEP23      0x0004   /* Independent or Complementary Pair Operation, channel 2-3 */
#define PWM_PMCFG_INDEP01      0x0002   /* Independent or Complementary Pair Operation, channel 0-1 */
#define PWM_PMCFG_WP           0x0001   /* Write Protect Bit */

/* PWM Channel Control Register PMCCR */
#define PWM_PMCCR_ENHA         0x8000   /* Enable Hardware Accelerator */
#define PWM_PMCCR_MSK_MASK     0x3f00   /* Mask bits */
#define PWM_PMCCR_VLMODE_MASK  0x0030   /* Value Register Load Mode bits */
#define PWM_PMCCR_SWP_MASK     0x0007   /* Swap bits */

/*******************************************************************************
	Bit names of older SDK version, maintained here for backward compatibility.
*******************************************************************************/

/* Do not use for new development ! */
#define PWM_LDFQ_0    PWM_PMCTL_LDFQ_0
#define PWM_LDFQ_1    PWM_PMCTL_LDFQ_1
#define PWM_LDFQ_2    PWM_PMCTL_LDFQ_2
#define PWM_LDFQ_3    PWM_PMCTL_LDFQ_3
#define PWM_LDFQ      PWM_PMCTL_LDFQ_MASK
#define PWM_HALF      PWM_PMCTL_HALF     
#define PWM_IPOL      PWM_PMCTL_IPOL_MASK  
#define PWM_PRSC_0    PWM_PMCTL_PRSC_0 
#define PWM_PRSC_1    PWM_PMCTL_PRSC_1 
#define PWM_PRSC      PWM_PMCTL_PRSC_MASK
#define PWM_PWMRIE    PWM_PMCTL_PWMRIE
#define PWM_PWMF      PWM_PMCTL_PWMF  
#define PWM_ISENS_0   PWM_PMCTL_ISENS_0  
#define PWM_ISENS_1   PWM_PMCTL_ISENS_1  
#define PWM_ISENS     PWM_PMCTL_ISENS_MASK
#define PWM_LDOK      PWM_PMCTL_LDOK 
#define PWM_PWMEN     PWM_PMCTL_PWMEN
#define PWM_FIE_3     PWM_PMFCTL_FIE3	
#define PWM_FIE_2     PWM_PMFCTL_FIE2	
#define PWM_FIE_1     PWM_PMFCTL_FIE1	
#define PWM_FIE_0     PWM_PMFCTL_FIE0	
#define PWM_FMODE_3   PWM_PMFCTL_FMODE3
#define PWM_FMODE_2   PWM_PMFCTL_FMODE2
#define PWM_FMODE_1   PWM_PMFCTL_FMODE1
#define PWM_FMODE_0   PWM_PMFCTL_FMODE0
#define PWM_FTACK_3   PWM_PMFSA_FTACK3
#define PWM_FTACK_2   PWM_PMFSA_FTACK2
#define PWM_FTACK_1   PWM_PMFSA_FTACK1
#define PWM_FTACK_0   PWM_PMFSA_FTACK0
#define PWM_PAD_EN    PWM_PMOUT_PAD_EN     
#define PWM_OUTCTL    PWM_PMOUT_OUTCTL_MASK
#define PWM_OUT       PWM_PMOUT_OUT_MASK
#define PWM_EDG       PWM_PMCFG_EDG        
#define PWM_TOPNEG45  PWM_PMCFG_TOPNEG45   
#define PWM_TOPNEG23  PWM_PMCFG_TOPNEG23   
#define PWM_TOPNEG01  PWM_PMCFG_TOPNEG01   
#define PWM_TOPNEG    PWM_PMCFG_TOPNEG_MASK
#define PWM_BOTNEG45  PWM_PMCFG_BOTNEG45   
#define PWM_BOTNEG23  PWM_PMCFG_BOTNEG23   
#define PWM_BOTNEG01  PWM_PMCFG_BOTNEG01   
#define PWM_BOTNEG    PWM_PMCFG_BOTNEG_MASK
#define PWM_INDEP45   PWM_PMCFG_INDEP45    
#define PWM_INDEP23   PWM_PMCFG_INDEP23    
#define PWM_INDEP01   PWM_PMCFG_INDEP01    
#define PWM_INDEP     PWM_PMCFG_INDEP_MASK
#define PWM_WP        PWM_PMCFG_WP         
#define PWM_ENHA      PWM_PMCCR_ENHA

/***********************************************************************
* PWM init
***********************************************************************/

#define ioctlPWM_INIT(pPwmBase,param) pwmInit(pPwmBase)
void pwmInit(arch_sPWM *pPwmBase); /* function prototype - implemented in pwm.c */


/* PWM Control Register */

#define ioctlPWM_SET_RELOAD_FREQUENCY(pPwmBase,param) \
  periphBitGrpZS(PWM_PMCTL_LDFQ_MASK, (((UWord16)(param)) << 12),&pPwmBase->pmctl)

#define ioctlPWM_HALF_CYCLE_RELOAD(pPwmBase,param) \
  if (param) periphBitSet(PWM_PMCTL_HALF,&pPwmBase->pmctl); \
  else periphBitClear(PWM_PMCTL_HALF,&pPwmBase->pmctl)

#define ioctlPWM_SET_CURRENT_POLARITY(pPwmBase,param) \
  periphBitGrpRS(PWM_PMCTL_IPOL_MASK, ((UWord16)(param)),&pPwmBase->pmctl)

#define ioctlPWM_SET_PRESCALER(pPwmBase,param) \
  periphBitGrpZS(PWM_PMCTL_PRSC_MASK, (((UWord16)(param)) << 6),&pPwmBase->pmctl)
  
#define ioctlPWM_RELOAD_INT(pPwmBase,param) \
  if (param) periphBitSet(PWM_PMCTL_PWMRIE,&pPwmBase->pmctl); \
  else periphBitClear(PWM_PMCTL_PWMRIE,&pPwmBase->pmctl)

#define ioctlPWM_SET_CURRENT_SENSING(pPwmBase,param) \
  periphBitGrpZS(PWM_PMCTL_ISENS_MASK, (((UWord16)(param)) << 2),&pPwmBase->pmctl)

#define ioctlPWM_DEVICE(pPwmBase,param) \
  if (param) periphBitSet(PWM_PMCTL_PWMEN,&pPwmBase->pmctl); \
  else periphBitClear(PWM_PMCTL_PWMEN,&pPwmBase->pmctl)

#define ioctlPWM_CLEAR_RELOAD_FLAG(pPwmBase,param) \
  periphBitClear(PWM_PMCTL_PWMF,&pPwmBase->pmctl)

#define ioctlPWM_LOAD_OK(pPwmBase,param) \
  periphBitSet(PWM_PMCTL_LDOK,&pPwmBase->pmctl)

          
/* PWM Output Control Register */

#define ioctlPWM_OUTPUT_PAD(pPwmBase,param) \
  if (param) periphBitSet(PWM_PMOUT_PAD_EN,&pPwmBase->pmout); \
  else periphBitClear(PWM_PMOUT_PAD_EN,&pPwmBase->pmout)

#define ioctlPWM_OUTPUT_SOFTWARE_CONTROL(pPwmBase,param) \
  periphBitGrpRS(PWM_PMOUT_OUTCTL_MASK, (((UWord16)(param)) << 8),&pPwmBase->pmout)

#define ioctlPWM_OUTPUT_CONTROL(pPwmBase,param) \
  periphBitGrpRS(PWM_PMOUT_OUT_MASK, ((UWord16)(param)),&pPwmBase->pmout)

          
/* PWM Counter Modulo Register */

#define ioctlPWM_SET_MODULO(pPwmBase,param) \
  periphMemWrite((UWord16)(param),&pPwmBase->pwmcm)

#define ioctlPWM_GET_MODULO(pPwmBase,param) \
  periphMemRead(&pPwmBase->pwmcm)

         
/* PWM Deadtime Register */

#define ioctlPWM_SET_DEADTIME(pPwmBase,param) \
  periphMemWrite(((UWord16)(param)),&pPwmBase->pmdeadtm)


/* PWM Config Register */


#define ioctlPWM_SET_ALIGNMENT(pPwmBase,param) \
  if (param) periphBitSet(PWM_PMCFG_EDG,&pPwmBase->pmcfg); \
  else periphBitClear(PWM_PMCFG_EDG,&pPwmBase->pmcfg)

#define ioctlPWM_SET_NEG_TOP_SIDE_POLARITY(pPwmBase,param) \
  periphBitGrpRS(PWM_PMCFG_TOPNEG_MASK, (((UWord16)(param)) << 8),&pPwmBase->pmcfg)

#define ioctlPWM_SET_NEG_BOTTOM_SIDE_POLARITY(pPwmBase,param) \
  periphBitGrpRS(PWM_PMCFG_BOTNEG_MASK, (((UWord16)(param)) << 4),&pPwmBase->pmcfg)

#define ioctlPWM_SET_INDEPENDENT_OPERATION(pPwmBase,param) \
  periphBitGrpRS(PWM_PMCFG_INDEP_MASK, (((UWord16)(param)) << 1),&pPwmBase->pmcfg)

#define ioctlPWM_SET_INDEPENDENT_MODE(pPwmBase,param) \
  periphBitSet((((UWord16)(param)) << 1), &pPwmBase->pmcfg)

#define ioctlPWM_SET_COMPLEMENTARY_MODE(pPwmBase,param) \
  periphBitClear((((UWord16)(param)) << 1), &pPwmBase->pmcfg)

#define ioctlPWM_SET_WRITE_PROTECT(pPwmBase,param) \
  periphBitSet(PWM_PMCFG_WP,&pPwmBase->pmcfg)
          

/* PWM Channel Control Register */

#define ioctlPWM_HARDWARE_ACCELERATION(pPwmBase,param) \
  if (param) periphBitSet(PWM_PMCCR_ENHA,&pPwmBase->pmccr); \
  else periphBitClear(PWM_PMCCR_ENHA,&pPwmBase->pmccr)
          
#define ioctlPWM_SET_CHANNEL_MASK(pPwmBase,param) \
  periphBitGrpRS(PWM_PMCCR_MSK_MASK, (((UWord16)(param)) << 8),&pPwmBase->pmccr)
          
#define ioctlPWM_SET_LOAD_MODE(pPwmBase,param) \
  { periphBitClear(~(((((UWord16)(param)) << 4)) | (~PWM_PMCCR_VLMODE_MASK)),&pPwmBase->pmccr); \
    periphBitSet((((UWord16)(param)) << 4),&pPwmBase->pmccr); } 
/*
#define ioctlPWM_SET_LOAD_MODE(pPwmBase,param) \
  { periphBitSet(PWM_PMCCR_ENHA,&pPwmBase->pmccr);  \
    periphBitClear(~(((((UWord16)(param)) << 4)) | (~PWM_PMCCR_VLMODE_MASK)),&pPwmBase->pmccr); \
    periphBitSet(PWM_PMCCR_ENHA,&pPwmBase->pmccr);  \
    periphBitSet((((UWord16)(param)) << 4),&pPwmBase->pmccr); } 
*/
//  periphBitSet(PWM_PMCCR_ENHA,&pPwmBase->pmccr); -- needed in older PWM rev           
 
#define ioctlPWM_SET_SWAP(pPwmBase,param) \
  { periphBitClear(~((((UWord16)(param))) | (~PWM_PMCCR_SWP_MASK)),&pPwmBase->pmccr); \
    periphBitSet(((UWord16)(param)),&pPwmBase->pmccr); } 

/*
#define ioctlPWM_SET_SWAP(pPwmBase,param) \
  { periphBitSet(PWM_PMCCR_ENHA,&pPwmBase->pmccr);  \
    periphBitClear(~((((UWord16)(param))) | (~PWM_PMCCR_SWP_MASK)),&pPwmBase->pmccr); \
    periphBitSet(PWM_PMCCR_ENHA,&pPwmBase->pmccr);  \
    periphBitSet(((UWord16)(param)),&pPwmBase->pmccr); } 
*/          
//  periphBitSet(PWM_PMCCR_ENHA,&pPwmBase->pmccr); \ needed in older PWM rev 
 
 
/* PWM Fault Status & Acknowledge Register */

#define ioctlPWM_READ_FAULT_STATUS_REG(pPwmBase,param) \
  periphMemRead(&pPwmBase->pmfsa)

          
/* PWM Counter Register */

#define ioctlPWM_READ_COUNTER_REG(pPwmBase,param) \
  periphMemRead(&pPwmBase->pmcnt)
 
/* 
#define ioctlPWM_WRITE_COUNTER_REG(pPwmBase,param) \
  periphMemWrite(((UWord16)(param)),&pPwmBase->pmcnt)
*/ 

          
/* PWM Control Register */

#define ioctlPWM_READ_CONTROL_REG(pPwmBase,param) \
  periphMemRead(&pPwmBase->pmctl)

          
/* PWM Port Register */

#define ioctlPWM_READ_PORT_REG(pPwmBase,param) \
  periphMemRead(&pPwmBase->pmport)
          

/* PWM Value Registers */

#define ioctlPWM_WRITE_VALUE_REG_0(pPwmBase,param) \
  periphMemWrite(param,&pPwmBase->pwmval[0])
          
#define ioctlPWM_WRITE_VALUE_REG_1(pPwmBase,param) \
  periphMemWrite(param,&pPwmBase->pwmval[1])

#define ioctlPWM_WRITE_VALUE_REG_2(pPwmBase,param) \
  periphMemWrite(param,&pPwmBase->pwmval[2])

#define ioctlPWM_WRITE_VALUE_REG_3(pPwmBase,param) \
  periphMemWrite(param ,&pPwmBase->pwmval[3])

#define ioctlPWM_WRITE_VALUE_REG_4(pPwmBase,param) \
  periphMemWrite(param,&pPwmBase->pwmval[4])

#define ioctlPWM_WRITE_VALUE_REG_5(pPwmBase,param) \
  periphMemWrite(param,&pPwmBase->pwmval[5])

#define ioctlPWM_WRITE_VALUE_REGS_COMPL(pPwmBase,param) \
  { periphMemWrite( ((pwm_sComplementaryValues*)param)->pwmChannel_0_Value,&pPwmBase->pwmval[0]); \
    periphMemWrite( ((pwm_sComplementaryValues*)param)->pwmChannel_2_Value,&pPwmBase->pwmval[2]); \
    periphMemWrite( ((pwm_sComplementaryValues*)param)->pwmChannel_4_Value,&pPwmBase->pwmval[4]);}

#define ioctlPWM_WRITE_VALUE_REGS_INDEP(pPwmBase,param) \
  { periphMemWrite( ((pwm_sIndependentValues*)param)->pwmChannel_0_Value ,&pPwmBase->pwmval[0]); \
    periphMemWrite( ((pwm_sIndependentValues*)param)->pwmChannel_1_Value ,&pPwmBase->pwmval[1]); \
    periphMemWrite( ((pwm_sIndependentValues*)param)->pwmChannel_2_Value ,&pPwmBase->pwmval[2]); \
    periphMemWrite( ((pwm_sIndependentValues*)param)->pwmChannel_3_Value ,&pPwmBase->pwmval[3]); \
    periphMemWrite( ((pwm_sIndependentValues*)param)->pwmChannel_4_Value ,&pPwmBase->pwmval[4]); \
    periphMemWrite( ((pwm_sIndependentValues*)param)->pwmChannel_5_Value ,&pPwmBase->pwmval[5]);}

#define ioctlPWM_UPDATE_VALUE_REG_0(pPwmBase,param) \
  { UWord16 TempModulus;  \
    TempModulus = periphMemRead(&pPwmBase->pwmcm ); \
    periphMemWrite( mult_r( TempModulus, param ),&pPwmBase->pwmval[0]); \
    periphBitSet(PWM_PMCTL_LDOK,&pPwmBase->pmctl); } 

#define ioctlPWM_UPDATE_VALUE_REG_1(pPwmBase,param) \
  { UWord16 TempModulus;  \
    TempModulus = periphMemRead(&pPwmBase->pwmcm ); \
    periphMemWrite( mult_r( TempModulus, param ),&pPwmBase->pwmval[1]); \
    periphBitSet(PWM_PMCTL_LDOK,&pPwmBase->pmctl); } 

#define ioctlPWM_UPDATE_VALUE_REG_2(pPwmBase,param) \
  { UWord16 TempModulus;  \
    TempModulus = periphMemRead(&pPwmBase->pwmcm ); \
    periphMemWrite( mult_r( TempModulus, param ),&pPwmBase->pwmval[2]); \
    periphBitSet(PWM_PMCTL_LDOK,&pPwmBase->pmctl); } 

#define ioctlPWM_UPDATE_VALUE_REG_3(pPwmBase,param) \
  { UWord16 TempModulus;  \
    TempModulus = periphMemRead(&pPwmBase->pwmcm ); \
    periphMemWrite( mult_r( TempModulus, param ),&pPwmBase->pwmval[3]); \
    periphBitSet(PWM_PMCTL_LDOK,&pPwmBase->pmctl); } 

#define ioctlPWM_UPDATE_VALUE_REG_4(pPwmBase,param) \
  { UWord16 TempModulus;  \
    TempModulus = periphMemRead(&pPwmBase->pwmcm ); \
    periphMemWrite( mult_r( TempModulus, param ),&pPwmBase->pwmval[4]); \
    periphBitSet(PWM_PMCTL_LDOK,&pPwmBase->pmctl); } 

#define ioctlPWM_UPDATE_VALUE_REG_5(pPwmBase,param) \
  { UWord16 TempModulus;  \
    TempModulus = periphMemRead(&pPwmBase->pwmcm ); \
    periphMemWrite( mult_r( TempModulus, param ),&pPwmBase->pwmval[5]); \
    periphBitSet(PWM_PMCTL_LDOK,&pPwmBase->pmctl); } 

#define ioctlPWM_UPDATE_VALUE_REGS_COMPL(pPwmBase,param) \
  { UWord16 TempModulus;  \
    TempModulus = periphMemRead(&pPwmBase->pwmcm ); \
    periphMemWrite( mult_r( TempModulus, ((pwm_sComplementaryValues*)param)->pwmChannel_0_Value) ,&pPwmBase->pwmval[0]); \
    periphMemWrite( mult_r( TempModulus, ((pwm_sComplementaryValues*)param)->pwmChannel_2_Value) ,&pPwmBase->pwmval[2]); \
    periphMemWrite( mult_r( TempModulus, ((pwm_sComplementaryValues*)param)->pwmChannel_4_Value) ,&pPwmBase->pwmval[4]); \
    periphBitSet(PWM_PMCTL_LDOK,&pPwmBase->pmctl); }

#define ioctlPWM_UPDATE_VALUE_REGS_INDEP(pPwmBase,param) \
  { UWord16 TempModulus;  \
    TempModulus = periphMemRead(&pPwmBase->pwmcm ); \
    periphMemWrite( mult_r( TempModulus, ((pwm_sIndependentValues*)param)->pwmChannel_0_Value) ,&pPwmBase->pwmval[0]); \
    periphMemWrite( mult_r( TempModulus, ((pwm_sIndependentValues*)param)->pwmChannel_1_Value) ,&pPwmBase->pwmval[1]); \
    periphMemWrite( mult_r( TempModulus, ((pwm_sIndependentValues*)param)->pwmChannel_2_Value) ,&pPwmBase->pwmval[2]); \
    periphMemWrite( mult_r( TempModulus, ((pwm_sIndependentValues*)param)->pwmChannel_3_Value) ,&pPwmBase->pwmval[3]); \
    periphMemWrite( mult_r( TempModulus, ((pwm_sIndependentValues*)param)->pwmChannel_4_Value) ,&pPwmBase->pwmval[4]); \
    periphMemWrite( mult_r( TempModulus, ((pwm_sIndependentValues*)param)->pwmChannel_5_Value) ,&pPwmBase->pwmval[5]); \
    periphBitSet(PWM_PMCTL_LDOK,&pPwmBase->pmctl); }

          
/* PWM Output Control Register */

#define ioctlPWM_SOFTWARE_OUTPUTS_CONTROL(pPwmBase,param) \
  { UWord16 TempOutctl;  \
    TempOutctl = PWM_PMOUT_PAD_EN | ((pwm_sOutputControl*)param)->SoftwareControlled << 8 | ((pwm_sOutputControl*)param)->OutputControl; \
    periphMemWrite((TempOutctl),&pPwmBase->pmout); } 


/* PWM Channel Control Register */
          
#define ioctlPWM_SET_MASK_SWAP(pPwmBase,param) \
  { UWord16 TempChannelControl;  \
    TempChannelControl = periphMemRead(&pPwmBase->pmccr) & (PWM_PMCCR_VLMODE_MASK | PWM_PMCCR_ENHA); \
    TempChannelControl = (((pwm_sChannelControl*)param)->Mask << 8) | TempChannelControl | ((pwm_sChannelControl*)param)->Swap; \
    periphMemWrite((TempChannelControl),&pPwmBase->pmccr); } 

/*
#define ioctlPWM_SET_MASK_SWAP(pPwmBase,param) \
  { UWord16 TempChannelControl;  \
    TempChannelControl = periphMemRead((UWord16 *)(&((arch_sPWM*)pPwmBase)->pmccr)) & PWM_PMCCR_VLMODE_MASK; \
    TempChannelControl = PWM_PMCCR_ENHA | (((pwm_sChannelControl*)param)->Mask << 8) | TempChannelControl | ((pwm_sChannelControl*)param)->Swap; \
    periphBitSet(PWM_PMCCR_ENHA,&pPwmBase->pmccr); \
    periphMemWrite((TempChannelControl),&pPwmBase->pmccr); } 
*/ 
//  periphBitSet(PWM_PMCCR_ENHA,&pPwmBase->pmccr); \ needed in older PWM rev         


#define ioctlPWM_UPDATE_VALUE_SET_VLMODE(pPwmBase,param) \
  { UWord16 TempModulus;  \
    TempModulus = periphMemRead( &pPwmBase->pwmcm ); \
    periphBitClear(PWM_PMCCR_VLMODE_MASK,&pPwmBase->pmccr); \
    periphBitSet( ( (((pwm_sUpdateValueSetVlmode*)param)->Vlmode) << 4 ),&pPwmBase->pmccr); \
    periphMemWrite( mult_r( TempModulus, ((pwm_sUpdateValueSetVlmode*)param)->DutyCycle) ,&pPwmBase->pwmval[0]); \
    periphBitSet(PWM_PMCTL_LDOK,&pPwmBase->pmctl); }

/*
#define ioctlPWM_UPDATE_VALUE_SET_VLMODE(pPwmBase,param) \
  { UWord16 TempModulus;  \
    TempModulus = periphMemRead( (UWord16 *)(&((arch_sPWM*)pPwmBase)->pwmcm)); \
    periphBitSet(PWM_PMCCR_ENHA,&pPwmBase->pmccr); \
    periphBitClear(PWM_PMCCR_VLMODE_MASK,&pPwmBase->pmccr); \
    periphBitSet(PWM_PMCCR_ENHA,&pPwmBase->pmccr); \
    periphBitSet( ( (((pwm_sUpdateValueSetVlmode*)param)->Vlmode) << 4 ),&pPwmBase->pmccr); \
    periphMemWrite( mult_r( TempModulus, ((pwm_sUpdateValueSetVlmode*)param)->DutyCycle) ,&pPwmBase->pwmval[0]); \
    periphBitSet(PWM_PMCTL_LDOK,&pPwmBase->pmctl); }
*/
//  periphBitSet(PWM_PMCCR_ENHA,&pPwmBase->pmccr); \ needed in older PWM rev         


#define ioctlPWM_CORRECT_DEAD_TIME_COMPL(pPwmBase,param) \
  { UWord16 tmp0, tmp1, tmp2, modulo, dead, edge;  \
    modulo = periphMemRead( &pPwmBase->pwmcm ); \
    dead = periphMemRead( &pPwmBase->pmdeadtm ); \
    edge = periphMemRead( &pPwmBase->pmcfg ) & PWM_PMCFG_EDG; \
    if( edge == 0 ) dead = dead >> 1; \
    tmp0 = mult_r( modulo, ((pwm_sComplementaryValues*)param)->pwmChannel_0_Value ); \
    tmp1 = mult_r( modulo, ((pwm_sComplementaryValues*)param)->pwmChannel_2_Value ); \
    tmp2 = mult_r( modulo, ((pwm_sComplementaryValues*)param)->pwmChannel_4_Value ); \
    periphMemWrite( tmp0 + dead ,&pPwmBase->pwmval[0]); \
    periphMemWrite( tmp0 - dead ,&pPwmBase->pwmval[1]); \
    periphMemWrite( tmp1 + dead ,&pPwmBase->pwmval[2]); \
    periphMemWrite( tmp1 - dead ,&pPwmBase->pwmval[3]); \
    periphMemWrite( tmp2 + dead ,&pPwmBase->pwmval[4]); \
    periphMemWrite( tmp2 - dead ,&pPwmBase->pwmval[5]); \
    periphBitSet(PWM_PMCTL_LDOK,&pPwmBase->pmctl); }


/**********************************************************************
* Include of device specific PWM commands and defines
***********************************************************************/
#include "pwmtarget.h"  /* device specific defines */

#ifdef __cplusplus
}
#endif

#endif
