/*******************************************************************************
*
*  Motorola Inc.
*  (c) Copyright 2002 Motorola Inc.
*  ALL RIGHTS RESERVED.
*
********************************************************************************
*
*
*  File Name:   qtimer.h
*
*  Description: Header file for Quad Timer (QT) driver.
*
*
*  Modules Included: qtimertarget.h
*
*  Notes: 
*
*
*******************************************************************************/
#ifndef __QTIMER_H
#define __QTIMER_H

#if !defined(__ARCH_H) || !defined(__PERIPH_H)
#error arch.h and periph.h must be included before qtimer.h
#endif

#ifdef __cplusplus
extern "C" {
#endif


/*******************************************************************************
	Quad Timer module identifiers (see also qtimertarget.h)
*******************************************************************************/

#define QTIMER_A0	(&ArchIO.TimerA.Channel0)
#define QTIMER_A1	(&ArchIO.TimerA.Channel1)
#define QTIMER_A2	(&ArchIO.TimerA.Channel2)
#define QTIMER_A3	(&ArchIO.TimerA.Channel3)
#define QTIMER_B0	(&ArchIO.TimerB.Channel0)
#define QTIMER_B1	(&ArchIO.TimerB.Channel1)
#define QTIMER_B2	(&ArchIO.TimerB.Channel2)
#define QTIMER_B3	(&ArchIO.TimerB.Channel3)
#define QTIMER_C0	(&ArchIO.TimerC.Channel0)
#define QTIMER_C1	(&ArchIO.TimerC.Channel1)
#define QTIMER_C2	(&ArchIO.TimerC.Channel2)
#define QTIMER_C3	(&ArchIO.TimerC.Channel3)
#define QTIMER_D0	(&ArchIO.TimerD.Channel0)
#define QTIMER_D1	(&ArchIO.TimerD.Channel1)
#define QTIMER_D2	(&ArchIO.TimerD.Channel2)
#define QTIMER_D3	(&ArchIO.TimerD.Channel3)

/******************************************************************************
*
*                      General Interface Description
*
* There are up to four different possible quad timer modules on the different
* configurations of the DSP56F800 series. The DSP56F801 has two quad timer
* modules, C and D, while the DSP56F803/805/807 have four quad timer modules:
* A, B, C, and D. Timer modules C and D have dedicated pins. Timer module C has
* two dedicated pins on the 805/807. Timer modules A and B share pins with the
* quadrature decoder modules. Each timer module contains four identical
* counter/timer groups. Each 16-bit counter timer group contains a prescaler,
* a counter, a load register, a hold register, a capture register, two compare
* registers, a status and control register, and a control register. All of the
* registers except the prescaler are read/write registers.
* 
* Basic Features:
* - Each timer module consists of four 16-bit counters/timers
* - Count up/down
* - Counters are cascadable
* - Programmable count modulo
* - Max count rate equals peripheral clock/2 when counting external events
* - Max count rate equals peripheral clock when using internal clocks
* - Count once or repeatedly
* - Counters are preloadable
* - Counters can share available input pins
* - Each counter has a separate prescaler
* - Each counter has capture and compare capability
*
* For more information, please refer to the DSP56F80x User's Manual,
* for a description of the Quad Timer device.
*
* The Quad Timer device driver interface uses "ioctl" call to control specific
* Quad Timer functions during operation. For details see "ioctl" call.
*
******************************************************************************/

/*******************************************************************************
	Defines for appconfig.h for each timer/counter
********************************************************************************

	void qtxxISR(void);
	#define INT_VECTOR_ADDR_yy			qtxxISR
	#define INT_PRIORITY_LEVEL_yy		value 0 to 7 (0 means interrupt is disabled)

	#define QT_xx_CTRL_INIT  0x0000
	#define QT_xx_SCR_INIT   0x0000
	#define QT_xx_CMP1_INIT  0x0000
	#define QT_xx_CMP2_INIT  0x0000
	#define QT_xx_LOAD_INIT	 0x0000
	#define QT_xx_CNTR_INIT  0x0000

	where:
	xx is timer/counter:
		D0,D1,D2,D3,C0,C1,C2,C3,B0,B1,B2,B3,A0,A1,A2,A3
		
	yy is corresponding interrupt vector
		30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45
	
*/

/*******************************************************************************
	ioctl() commands
*******************************************************************************/

/*		command							parameters and commentary */
#define QT_INIT							/* NULL */
#define QT_SET_COUNT_MODE				/* see below QT_xxx_MODE */
#define QT_SET_PRIMARY_SOURCE			/* see below */
#define QT_SET_SECONDARY_SOURCE			/* one of QT_COUNTER0_INPUT, QT_COUNTER1_INPUT, QT_COUNTER2_INPUT, QT_COUNTER3_INPUT */

#define QT_SET_COUNT_ONCE				/* QT_COUNT_REPEATEDLY / QT_UNTIL_COMPARE_AND_STOP */
#define QT_SET_COUNT_LENGTH				/* QT_ROLL_OVER / QT_UNTIL_COMPARE_AND_REINIT */
#define QT_SET_COUNT_DIRECTION			/* QT_COUNT_UP / QT_COUNT_DOWN */
#define QT_CO_CHANNEL_INIT				/* QT_ENABLE / QT_DISABLE */
#define QT_SET_OUTPUT_MODE				/* see below */

//#define QT_SET_FLAG						/* QT_COMPARE_FLAG | QT_OVERFLOW_FLAG | QT_INPUT_EDGE_FLAG | QT_VAL_FLAG */
#define QT_CLEAR_FLAG					/* QT_COMPARE_FLAG | QT_OVERFLOW_FLAG | QT_INPUT_EDGE_FLAG | QT_VAL_FLAG */
#define QT_READ_FLAG					/* QT_COMPARE_FLAG | QT_OVERFLOW_FLAG | QT_INPUT_EDGE_FLAG | QT_VAL_FLAG, returns 0x0000 - bits are cleared, or returns 0x8000 (Timer Compare Flag is set) | 0x2000 (Timer Overflow Flag is set) | 0x0800 (Input Edge Flag is set) | 0x0008 (VAL Flag is set) */
#define QT_INT_ENABLE					/* QT_COMPARE_INT | QT_OVERFLOW_INT | QT_INPUT_EDGE_INT */
#define QT_INT_DISABLE					/* QT_COMPARE_INT | QT_OVERFLOW_INT | QT_INPUT_EDGE_INT */
#define QT_SET_INPUT_POLARITY			/* QT_NORMAL_POLARITY / QT_INVERTED_POLARITY */
#define QT_READ_EXT_INPUT_PIN			/* NULL, returns 0x0000 (input pin is 0) or 0x0100 (input pin is 1) */
#define QT_SET_CAPTURE_MODE				/* QT_CAPTURE_DISABLED / QT_RISING_EDGE / QT_FALLING_EDGE / QT_BOTH_EDGES */
#define QT_MASTER_MODE					/* QT_ENABLE / QT_DISABLE */
#define QT_EXT_OFLAG_FORCE				/* QT_ENABLE / QT_DISABLE */
#define QT_FORCE_OFLAG					/* QT_ONE / QT_ZERO0 - (force OFLAG to 1) / (force OFLAG to 0), use this command only if timer is disabled */
#define QT_SET_OUTPUT_POLARITY			/* QT_NORMAL_POLARITY / QT_INVERTED_POLARITY */
#define QT_OUTPUT_ON_EXT_PIN			/* QT_ENABLE / QT_DISABLE */

#define QT_WRITE_CONTROL_REG			/* 16 bit value */
#define QT_WRITE_STATUS_CONTROL_REG		/* 16 bit value */
#define QT_WRITE_COMPARE_REG1			/* 16 bit value */
#define QT_WRITE_COMPARE_REG2			/* 16 bit value */
#define QT_WRITE_LOAD_REG				/* 16 bit value */
#define QT_WRITE_COUNTER_REG			/* 16 bit value */

#define QT_READ_CONTROL_REG				/* NULL, returns UWord16 */
#define QT_READ_STATUS_CONTROL_REG		/* NULL, returns UWord16 */
#define QT_READ_COMPARE_REG1			/* NULL, returns UWord16 */
#define QT_READ_COMPARE_REG2			/* NULL, returns UWord16 */
#define QT_READ_LOAD_REG				/* NULL, returns UWord16 */
#define QT_READ_COUNTER_REG				/* NULL, returns UWord16 */
#define QT_READ_CAPTURE_REG				/* NULL, returns UWord16 */
#define QT_READ_HOLD_REG				/* NULL, returns UWord16 */


/*******************************************************************************
	QT constants used in the ioctl() as parameters
*******************************************************************************/

/* enable/disable */
#define	QT_DISABLE						0
#define	QT_ENABLE						1

/* Modes (used in QT_SET_COUNT_MODE command) */

#define QT_NO_OPERATION					0x0000
#define QT_COUNT_RISING_EDGES_MODE		0x2000
#define QT_COUNT_BOTH_EDGES_MODE		0x4000
#define QT_GATED_COUNT_MODE				0x6000
#define QT_QUADRATURE_COUNT_MODE		0x8000
#define QT_SIGNED_COUNT_MODE			0xA000
#define QT_TRIGGERED_COUNT_MODE			0xC000
#define QT_CASCADE_COUNT_MODE			0xE000

 /* special modes */						   /* Mask */
#define QT_ONE_SHOT_MODE			0xC065 /* COUNTMODE|ONCE|LENGTH|OUTMODE */
#define QT_PULSE_OUTPUT_MODE			0x2047 /* COUNTMODE|ONCE|OUTMODE */
#define QT_FIXED_FREQ_PWM_MODE			0x2006 /* COUNTMODE|ONCE|LENGTH|OUTMODE */
#define QT_VARIABLE_FREQ_PWM_MODE		0x2024 /* COUNTMODE|ONCE|LENGTH|OUTMODE */

/* Primary Count Sources (used in QT_SET_PRIMARY_SOURCE command).
   Secondary Count Sources (used in QT_SET_SECONDARY_SOURCE command). */
   
#define QT_COUNTER0_INPUT				0	/*0x0000*/
#define QT_COUNTER1_INPUT				1	/*0x0200*/
#define QT_COUNTER2_INPUT				2	/*0x0400*/
#define QT_COUNTER3_INPUT				3	/*0x0600*/
#define QT_COUNTER0_OUTPUT				4	/*0x0800*/
#define QT_COUNTER1_OUTPUT				5	/*0x0A00*/
#define QT_COUNTER2_OUTPUT				6	/*0x0C00*/
#define QT_COUNTER3_OUTPUT				7	/*0x0E00*/
#define QT_IPBUS_DIV_1				8	/*0x1000*/
#define QT_IPBUS_DIV_2				9	/*0x1200*/
#define QT_IPBUS_DIV_4				10	/*0x1400*/
#define QT_IPBUS_DIV_8				11	/*0x1600*/
#define QT_IPBUS_DIV_16				12	/*0x1800*/
#define QT_IPBUS_DIV_32				13	/*0x1A00*/
#define QT_IPBUS_DIV_64				14	/*0x1C00*/
#define QT_IPBUS_DIV_128			15	/*0x1E00*/


/* END OF LIFE   begin block */
/* deprecated names of parameters, to be removed in future versions */
#define QT_PRESCALER_DIV_1				8	/*0x1000*/
#define QT_PRESCALER_DIV_2				9	/*0x1200*/
#define QT_PRESCALER_DIV_4				10	/*0x1400*/
#define QT_PRESCALER_DIV_8				11	/*0x1600*/
#define QT_PRESCALER_DIV_16				12	/*0x1800*/
#define QT_PRESCALER_DIV_32				13	/*0x1A00*/
#define QT_PRESCALER_DIV_64				14	/*0x1C00*/
#define QT_PRESCALER_DIV_128			15	/*0x1E00*/
/* END OF LIFE   end block */

/* Continuous or one shot counting modes (used in QT_SET_COUNT_ONCE command) */
#define QT_COUNT_REPEATEDLY				0
#define QT_UNTIL_COMPARE_AND_STOP		1

/* Length (used in QT_SET_COUNT_LENGTH command) */
#define QT_ROLL_OVER					0
#define QT_UNTIL_COMPARE_AND_REINIT		1

/* Count Directions (used in QT_SET_COUNT_DIRECTION command) */
#define QT_COUNT_UP						0
#define QT_COUNT_DOWN					1

/* Output Modes (used in QT_SET_OUTPUT_MODE command) */
#define QT_SET_WHILE_ACTIVE				0
#define QT_CLEAR_ON_COMPARE				1
#define QT_SET_ON_COMPARE				2
#define QT_TOGGLE_ON_COMPARE			3
#define QT_TOGGLE_USING_ALT_COMPARE		4
#define QT_CLEAR_ON_SECONDARY			5
#define QT_CLEAR_ON_COUNTER_ROLLOVER	6
#define QT_GATED_CLOCK_WHILE_ACTIVE		7

/* deprecated name, to be removed in future versions */
#define QT_ASSERT_ON_GATED_CLOCK		7

/* Quad Timer Interrupts Flags and Interrupt Enable bits */
#define QT_COMPARE_FLAG					QT_SCR_TCF
#define QT_OVERFLOW_FLAG				QT_SCR_TOF
#define QT_INPUT_EDGE_FLAG				QT_SCR_IEF
#define QT_VAL_FLAG						QT_SCR_VAL

#define QT_COMPARE_INT					QT_SCR_TCFIE
#define QT_OVERFLOW_INT					QT_SCR_TOFIE
#define QT_INPUT_EDGE_INT				QT_SCR_IEFIE

/* Input and Output polarity select (used in QT_INPUT_POLARITY command) */
#define	QT_NORMAL_POLARITY				0
#define	QT_INVERTED_POLARITY			1

/* Input Capture Mode (used in QT_SET_CAPTURE_MODE command) */
#define QT_CAPTURE_DISABLED				0	/*0x0000*/
#define QT_RISING_EDGE					1	/*0x0040*/
#define QT_FALLING_EDGE					2	/*0x0080*/
#define QT_BOTH_EDGES					3	/*0x00C0*/

/* forced value */
#define	QT_ONE							1
#define	QT_ZERO  						0

/*******************************************************************************
	Register and bit names
*******************************************************************************/

/* CONTROL REGISTER */
#define QT_CTRL_COUNTMODE_MASK	0xe000
#define QT_CTRL_PRIMSOURCE_MASK	0x1e00
#define QT_CTRL_SECSOURCE_MASK	0x0180
#define QT_CTRL_ONCE			0x0040
#define QT_CTRL_LENGTH			0x0020
#define QT_CTRL_DIR				0x0010
#define QT_CTRL_CO_INIT			0x0008
#define QT_CTRL_OUTMODE_MASK	0x0007

/* STATUS CONTROL REGISTER*/
#define QT_SCR_TCF				0x8000
#define QT_SCR_TCFIE			0x4000
#define QT_SCR_TOF				0x2000
#define QT_SCR_TOFIE			0x1000
#define QT_SCR_IEF				0x0800
#define QT_SCR_IEFIE			0x0400
#define QT_SCR_IPS				0x0200
#define QT_SCR_INPUT			0x0100
#define QT_SCR_CAPMODE_MASK		0x00c0
#define QT_SCR_MSTR				0x0020
#define QT_SCR_EEOF				0x0010
#define QT_SCR_VAL				0x0008
#define QT_SCR_FORCE			0x0004
#define QT_SCR_OPS				0x0002
#define QT_SCR_OEN				0x0001

/*******************************************************************************
	Bit names of older SDK version, maintained here for backward compatibility.
*******************************************************************************/

/* Do not use for new development ! */
#define QT_COUNT_ONCE_BIT				QT_CTRL_ONCE
#define QT_COUNT_LENGTH_BIT				QT_CTRL_LENGTH
#define QT_COUNT_DIRECTION_BIT			QT_CTRL_DIR
#define QT_CO_INIT_BIT					QT_CTRL_CO_INIT
#define QT_INPUT_POLARITY_BIT			QT_SCR_IPS
#define QT_EXT_INPUT_PIN				QT_SCR_INPUT
#define QT_MASTER_MODE_BIT				QT_SCR_MSTR
#define QT_ENABLE_OFLAG_FORCE_BIT		QT_SCR_EEOF
#define QT_OFLAG_VALUE_BIT				QT_SCR_VAL
#define QT_FORCE_OFLAG_BIT				QT_SCR_FORCE
#define QT_OUTPUT_POLARITY_BIT			QT_SCR_OPS
#define QT_OUTPUT_ENABLE_BIT			QT_SCR_OEN

/*******************************************************************************
	Implementations of Quad Timer ioctl() commands
*******************************************************************************/

void qtInitQTIMER_A0(); void qtInitQTIMER_B0(); void qtInitQTIMER_C0(); void qtInitQTIMER_D0();
void qtInitQTIMER_A1(); void qtInitQTIMER_B1(); void qtInitQTIMER_C1(); void qtInitQTIMER_D1();
void qtInitQTIMER_A2(); void qtInitQTIMER_B2(); void qtInitQTIMER_C2(); void qtInitQTIMER_D2();
void qtInitQTIMER_A3(); void qtInitQTIMER_B3(); void qtInitQTIMER_C3(); void qtInitQTIMER_D3();

#define ioctlQT_INIT(pTimerBase,param)	qtInit ## pTimerBase ()

/* Control Register */
#define ioctlQT_SET_COUNT_MODE(pTimerBase, param) \
  if(param==QT_PULSE_OUTPUT_MODE)\
   periphBitGrpZS(QT_CTRL_COUNTMODE_MASK|QT_CTRL_ONCE|QT_CTRL_OUTMODE_MASK, param,&pTimerBase->ctrl)\
  else if(param==QT_FIXED_FREQ_PWM_MODE || param==QT_VARIABLE_FREQ_PWM_MODE || param==QT_ONE_SHOT_MODE)\
   periphBitGrpZS(QT_CTRL_COUNTMODE_MASK|QT_CTRL_ONCE|QT_CTRL_LENGTH|QT_CTRL_OUTMODE_MASK, param,&pTimerBase->ctrl)\
  else periphBitGrpZS(QT_CTRL_COUNTMODE_MASK, param,&pTimerBase->ctrl)

#define ioctlQT_SET_PRIMARY_SOURCE(pTimerBase,param) \
  periphBitGrpZS(QT_CTRL_PRIMSOURCE_MASK, (param) << 9,&pTimerBase->ctrl)

#define ioctlQT_SET_SECONDARY_SOURCE(pTimerBase,param) \
  periphBitGrpZS(QT_CTRL_SECSOURCE_MASK, (param) << 7,&pTimerBase->ctrl)

#define ioctlQT_SET_COUNT_ONCE(pTimerBase, param) \
  if (param) periphBitSet(QT_CTRL_ONCE,&pTimerBase->ctrl); \
  else		 periphBitClear(QT_CTRL_ONCE,&pTimerBase->ctrl)
				
#define ioctlQT_SET_COUNT_LENGTH(pTimerBase, param) \
  if (param) periphBitSet(QT_CTRL_LENGTH,&pTimerBase->ctrl); \
  else		 periphBitClear(QT_CTRL_LENGTH,&pTimerBase->ctrl)

#define ioctlQT_SET_COUNT_DIRECTION(pTimerBase, param) \
  if (param) periphBitSet(QT_CTRL_DIR,&pTimerBase->ctrl); \
  else		 periphBitClear(QT_CTRL_DIR,&pTimerBase->ctrl)

#define ioctlQT_CO_CHANNEL_INIT(pTimerBase, param) \
  if (param) periphBitSet(QT_CTRL_CO_INIT,&pTimerBase->ctrl); \
  else		 periphBitClear(QT_CTRL_CO_INIT,&pTimerBase->ctrl)

#define ioctlQT_SET_OUTPUT_MODE(pTimerBase,param) \
  periphBitGrpZS(QT_CTRL_OUTMODE_MASK, param,&pTimerBase->ctrl)

/* Status and Control Register */

/*
#define ioctlQT_SET_FLAG(pTimerBase,param) \
  periphBitSet(param,&pTimerBase->scr)
*/

#define ioctlQT_CLEAR_FLAG(pTimerBase,param) \
  periphBitClear(param,&pTimerBase->scr)

#define ioctlQT_READ_FLAG(pTimerBase,param) \
  periphBitTest(param,&pTimerBase->scr)

#define ioctlQT_ENABLE_INT(pTimerBase,param) \
  periphBitSet(param,&pTimerBase->scr)

#define ioctlQT_INT_ENABLE(pTimerBase,param) \
  periphBitSet(param,&pTimerBase->scr)

#define ioctlQT_DISABLE_INT(pTimerBase,param) \
  periphBitClear(param,&pTimerBase->scr)

#define ioctlQT_INT_DISABLE(pTimerBase,param) \
  periphBitClear(param,&pTimerBase->scr)

#define ioctlQT_SET_INPUT_POLARITY(pTimerBase, param) \
  if (param) periphBitSet(QT_SCR_IPS,&pTimerBase->scr); \
  else		 periphBitClear(QT_SCR_IPS,&pTimerBase->scr)

#define ioctlQT_READ_EXT_INPUT_PIN(pTimerBase,param) \
  periphBitTest(QT_SCR_INPUT,&pTimerBase->scr)

#define ioctlQT_SET_CAPTURE_MODE(pTimerBase, param)	\
  periphBitGrpZS(QT_SCR_CAPMODE_MASK, (param) << 6,&pTimerBase->scr)

#define ioctlQT_MASTER_MODE(pTimerBase, param) \
  if (param) periphBitSet(QT_SCR_MSTR,&pTimerBase->scr); \
  else		 periphBitClear(QT_SCR_MSTR,&pTimerBase->scr)

#define ioctlQT_EXT_OFLAG_FORCE(pTimerBase, param) \
  if (param) periphBitSet(QT_SCR_EEOF,&pTimerBase->scr); \
  else		 periphBitClear(QT_SCR_EEOF,&pTimerBase->scr)

#define ioctlQT_FORCE_OFLAG(pTimerBase, param) \
  { if(param)	periphBitSet(QT_SCR_VAL,&pTimerBase->scr); \
    else		periphBitClear(QT_SCR_VAL,&pTimerBase->scr); \
    periphBitSet(QT_SCR_FORCE,&pTimerBase->scr); }

/* alias for QT_FORCE_OFLAG */
#define ioctlQT_SET_OUTPUT_SIGNAL(pTimerBase, param) \
  { if(param)	periphBitSet(QT_SCR_VAL,&pTimerBase->scr); \
    else		periphBitClear(QT_SCR_VAL,&pTimerBase->scr); \
    periphBitSet(QT_SCR_FORCE,&pTimerBase->scr); }

#define ioctlQT_SET_OUTPUT_POLARITY(pTimerBase, param) \
  if (param) periphBitSet(QT_SCR_OPS,&pTimerBase->scr); \
  else		 periphBitClear(QT_SCR_OPS,&pTimerBase->scr)

#define ioctlQT_OUTPUT_ON_EXT_PIN(pTimerBase, param) \
  if (param) periphBitSet(QT_SCR_OEN,&pTimerBase->scr); \
  else       periphBitClear(QT_SCR_OEN,&pTimerBase->scr)


/* Reading(writing) from(to) registers */
#define ioctlQT_WRITE_CONTROL_REG(pTimerBase,param) \
  periphMemWrite(param,&pTimerBase->ctrl)

#define ioctlQT_WRITE_STATUS_CONTROL_REG(pTimerBase,param) \
  periphMemWrite(param,&pTimerBase->scr)

#define ioctlQT_WRITE_COMPARE_REG1(pTimerBase,param) \
  periphMemWrite(param,&pTimerBase->cmp1)

#define ioctlQT_WRITE_COMPARE_REG2(pTimerBase,param) \
  periphMemWrite(param,&pTimerBase->cmp2)

#define ioctlQT_WRITE_LOAD_REG(pTimerBase,param) \
  periphMemWrite(param,&pTimerBase->load)

#define ioctlQT_WRITE_COUNTER_REG(pTimerBase,param) \
  periphMemWrite(param,&pTimerBase->cntr)

#define ioctlQT_READ_CONTROL_REG(pTimerBase,param) \
  periphMemRead(&pTimerBase->ctrl)

#define ioctlQT_READ_STATUS_CONTROL_REG(pTimerBase,param) \
  periphMemRead(&pTimerBase->scr)

#define ioctlQT_READ_COMPARE_REG1(pTimerBase,param) \
  periphMemRead(&pTimerBase->cmp1)

#define ioctlQT_READ_COMPARE_REG2(pTimerBase,param) \
  periphMemRead(&pTimerBase->cmp2)

#define ioctlQT_READ_LOAD_REG(pTimerBase,param) \
  periphMemRead(&pTimerBase->load)

#define ioctlQT_READ_COUNTER_REG(pTimerBase,param) \
  periphMemRead(&pTimerBase->cntr)

#define ioctlQT_READ_CAPTURE_REG(pTimerBase,param) \
  periphMemRead(&pTimerBase->cap)

#define ioctlQT_READ_HOLD_REG(pTimerBase,param) \
  periphMemRead(&pTimerBase->hold)


/**********************************************************************
* Include of device specific QTIMER commands and defines
***********************************************************************/
#include "qtimertarget.h"  /* device specific defines */

#ifdef __cplusplus
}
#endif

#endif
