/*****************************************************************************
*
* Motorola Inc.
* (c) Copyright 2002 Motorola, Inc.
* ALL RIGHTS RESERVED.
*
******************************************************************************
*
* File Name: spi.h
*
* Description: Header file for the SPI driver (macros and definitions)
*
* Modules Included: spitarget.h
*
*****************************************************************************/

#ifndef __SPI_H
#define __SPI_H

#if !defined(__ARCH_H) || !defined(__PERIPH_H)
#error arch.h and periph.h must be included before spi.h
#endif

#ifdef __cplusplus
extern "C" {
#endif

/************************************************************
* SPI module identifiers, correspond to module base address
*************************************************************/

#define SPI_0 (&ArchIO.Spi0)

/******************************************************************************
*
*                      General Interface Description
*
*
* The Serial Peripheral Interface (SPI) module allows full-duplex, synchronous,
* serial communication between the DSP and peripheral devices, including other DSPs.
* Software can poll SPI status flags, or SPI operation can be interrupt
* driven. This block contains four 16-bit memory mapped registers for control
* parameters, status, and data transfer.
* 
* Features of the SPI module include:
*    Full-duplex operation
*    Master and slave modes
*    Double-buffered operation with separate transmit and receive registers
*    Programmable length transmissions (2 to 16 bits)
*    Programmable transmit and receive shift order (MSB first or last bit transmitted)
*    Eight master mode frequencies (maximum = bus frequency  2)
*    Maximum slave mode frequency = bus frequency  2
*    Clock ground for reduced Radio Frequency (RF) interference
*    Serial clock with programmable polarity and phase
*    Two separately enabled interrupts
*       SPI Receiver Full (SPRF)
*       SPI Transmitter Empty (SPTE)
*    Mode fault error flag interrupt capability
*
* For more information, please refer to the user manual for the particular
* chip that you are using, such as the DSP56F80x User's Manual,
* for a description of the SPI device.
*
* The SPI device driver interface uses "ioctl" call to control specific
* SPI functions during operation. For details see "ioctl" call.
*
******************************************************************************/

/******************************************************************************
* type of SPI characters and buffers (can be defined as "char" in appconfig.h) 
******************************************************************************/

#define SPI0CHAR UWord16	/* HawkV1 compatible 16bit characters by default */

/****************************************************
* Configurable items, i.e. defines, for appconfig.h
****************************************************

	#define INT_VECTOR_ADDR_yy isr_routine 		// specify interrupt handler routine
	#define INT_PRIORITY_LEVEL_yy INTC_LEVEL1 	// interrupt level : (INTC_DISABLED, INTC_LEVEL0,...INTC_LEVEL2)
  
	#define SPI_0_SPSCR_INIT  0	 		// SPI0 Status Control Reg (SPSCR)
	#define SPI_0_SPDSCR_INIT 0x000F 	// SPI0 Data Size Reg (SPDSR) num=(num_bits-1) 

	dtto for SPI_1
 
  where:
         yy  is interrupt vector number
         
         	e.g. on DSP56F800:
                25 - SPI 0 Receiver Full
                24 - SPI 0 Transmitter Empty
                
                
  To enable NON_BLOCKING SPI_0 read & write functions, do:
  
	#define INT_VECTOR_ADDR_25 Spi0RxFullISR
	#define INT_VECTOR_ADDR_24 Spi0TxEmptyISR
	#define INT_PRIORITY_LEVEL_25 1
	#define INT_PRIORITY_LEVEL_24 1

  To install read & write hooks, define
  
	#define SPI_0_RX_CHAR_HOOK       rx_char_hook		// all operations
	#define SPI_0_RX_ERROR_HOOK      rx_error_hook		// all operations
	#define SPI_0_TX_CHAR_HOOK       tx_char_hook		// all operations
	#define SPI_0_RX_FINISHED_HOOK   rx_finished_hook	// NON_BLOCKING operation only 
	#define SPI_0_TX_FINISHED_HOOK   tx_finished_hook	// NON_BLOCKING operation only 

  where:
  
    UWord16 rx_char_hook(UWord16 rxchar)  // is called for each character received (both bl/nobl), 
                                          // return nonzero to discard received character
                                          // return zero to accept received character
                                          // you can use SPI_READ_CANCEL command to abort operation
                                          
	void rx_finished_hook(void)	          // called when non-blocking read finishes
	
	void rx_error_hook(void)              // called when error occures during read (both bl/nonbl)
										  // read SPI_x->scisr to get status register
	
    UWord16 tx_char_hook(UWord16 txchar)  // called after each character transmitted (both bl/nonbl)
                                          // return nonzero to not to transmit next buffered character 
                                                (you have to write SCIDR yourself using SPI_WRITE_DATA)
                                          // return zero to continue with transmission
                                          // you can use SPI_WRITE_CANCEL command to abort operation
                                          
    void tx_finished_hook(void)           // called when non-blocking write finishes


  In any hook function, you can issue new (NON_BLOCKING) read or write operation. 
 
 */


/***************************************************
* SPI Ioctl commands
****************************************************/
/* command                    |        Param       */

/* SPI status and control register commands */
#define SPI_INIT                /* NULL */
#define SPI_DEVICE				/* SPI_ENABLE / SPI_DISABLE */
#define SPI_SET_MODE			/* SPI_MASTER(1) / SPI_SLAVE(0) */
#define SPI_SET_ORDER	 	    /* SPI_LSB_FIRST(1) / SPI_MSB_FIRST(0) - DSO bit*/
#define SPI_SET_CLOCK_POLARITY  /* SPI_RISING_EDGE(1) / SPI_FALLING_EDGE(0) - CPOL bit */
#define SPI_SET_CLOCK_PHASE	    /* SPI_SCLK_EDGE(1) / SPI_SS_EDGE - CPHA bit (which edge starts slave transmission) */
#define SPI_SET_MODE_FAULT		/* SPI_ENABLE / SPI_DISABLE - MODFEN bit (ena/dis mode fault) */
#define SPI_SET_BAUD_DIV 		/* one of SPI_DIVx constants */

/* interrrupts enable disable */
#define SPI_RX_FULL_INT			/* SPI_ENABLE / SPI_DISABLE - receive interrupt */
#define SPI_TX_EMPTY_INT		/* SPI_ENABLE / SPI_DISABLE - transmit interrupt */
#define SPI_ERROR_INT			/* SPI_ENABLE / SPI_DISABLE - error (overflow/mode fault) interrupt */
#define SPI_INT_ENABLE          /* SPI_TX_EMPTY | SPI_RX_FULL | SPI_ERROR */
#define SPI_INT_DISABLE         /* SPI_TX_EMPTY | SPI_RX_FULL | SPI_ERROR */

/* SPI data and control register commands */
#define SPI_SET_TX_DATA_SIZE 	/* data size (2..16) - writes data_size-1 to SPDSCR) */

/* data registers */
#define SPI_WRITE_DATA			/* UWord16: transmit data 2-16bits */
#define SPI_READ_DATA			/* NULL */

/* test SPI status bits - can be used directly in condition */
/* if it is used in assignement then it returns bit value in original bit position */
#define SPI_GET_TX_EMPTY		/* NULL, test SPTE bit */
#define SPI_GET_RX_FULL			/* NULL, test SPRF bit */
#define SPI_GET_RX_OVERFLOW		/* NULL, test OVRF bit */
#define SPI_GET_MODE_FAULT		/* NULL, test MODF bit */

#define SPI_READ_CONTROL_REG    /* NULL */
#define SPI_WRITE_CONTROL_REG   /* UWord16 */

#define SPI_GET_ERROR	        /* NULL */
#define SPI_ERROR_IS_OVERFLOW   /* NULL */
#define SPI_ERROR_IS_MODEFAULT  /* NULL */

#define SPI_CLEAR_MODE_FAULT	/* NULL */
//#define SPI_CLEAR_RX_OVERFLOW	/* NULL */

/* commands for high-level operations (BLOCKING/NON_BLOCKING read, write) */
#define SPI_CLEAR_EXCEPTION		/* NULL */
#define SPI_GET_STATUS			/* NULL */
#define SPI_WRITE_CANCEL		/* NULL */
#define SPI_READ_CANCEL			/* NULL */

/****************************************************************************
* read/write modes of operation
*****************************************************************************/

#define BLOCKING      /* blocking mode for read/write functions */
#define NON_BLOCKING  /* non-blocking mode for read/write functions */

/****************************************************************************
* SPI constants and type declarations used in the ioctl functions
*****************************************************************************/

#define SPI_ENABLE	1
#define SPI_DISABLE	0

#define SPI_MASTER   1
#define SPI_SLAVE    0

#define SPI_MSB_FIRST   0
#define SPI_LSB_FIRST   1

#define SPI_RISING_EDGE		1
#define SPI_FALLING_EDGE 	0

#define SPI_SCLK_EDGE 	1
#define SPI_SS_EDGE 	0


/* SPI_SET_BAUD_DIV */
#define SPI_DIV2   	0  
#define SPI_DIV8   	SPI_SPSCR_SPR0
#define SPI_DIV16  	SPI_SPSCR_SPR1
#define SPI_DIV32  	SPI_SPSCR_SPR0|SPI_SPSCR_SPR1

/* interrupts */
#define SPI_TX_EMPTY	SPI_SPSCR_SPTIE
#define SPI_RX_FULL		SPI_SPSCR_SPRIE
#define SPI_ERROR		SPI_SPSCR_ERRIE

/* status generated by read/write functions -> check by SPI0_GET_STATUS command */

#define SPI_STATUS_WRITE_INPROGRESS		0x0001    /* write to SPI is in progress */
#define SPI_STATUS_READ_INPROGRESS		0x0002    /* read from SPI is in progress */
#define SPI_STATUS_EXCEPTION_EXIST		0x0010    /* exception exists - HW exception if only this flag is set
                                                           - SW exception if other exception flags are set */
#define SPI_EXCEPTION_BUFFER_OVERFLOW	0x0020    /* buffer overflow - some bytes were thrown away */
/* exception codes */
#define SPI_EXCEPTION_OVERRUN   		0x0040
#define SPI_EXCEPTION_MODE_FAULT   		0x0080

/*****************************************************************
* SPI Register Bits Defines, used in ioctl command implementation
******************************************************************/

/* SPI Status and Control Register (SPSCR) */
#define SPI_SPSCR_DSO		0x4000
#define SPI_SPSCR_SPRF		0x2000 /* read only */
#define SPI_SPSCR_ERRIE		0x1000
#define SPI_SPSCR_OVRF		0x0800 /* read only */
#define SPI_SPSCR_MODF		0x0400 /* read only, cleared by writing "1" */
#define SPI_SPSCR_SPTE		0x0200 /* read only */
#define SPI_SPSCR_MODFEN	0x0100
#define SPI_SPSCR_SPR_MASK	0x00C0
#define SPI_SPSCR_SPR1		0x0080
#define SPI_SPSCR_SPR0		0x0040
#define SPI_SPSCR_SPRIE		0x0020
#define SPI_SPSCR_SPMSTR	0x0010
#define SPI_SPSCR_CPOL		0x0008
#define SPI_SPSCR_CPHA		0x0004
#define SPI_SPSCR_SPE		0x0002
#define SPI_SPSCR_SPTIE		0x0001

/* SPI DataSize and Control Register (SPDSR) */
#define SPI_SPDSR_DS_MASK		0x000f

#define SPI_SPDSR_DS2			0x0001
#define SPI_SPDSR_DS3			0x0002
#define SPI_SPDSR_DS4			0x0003
#define SPI_SPDSR_DS5			0x0004
#define SPI_SPDSR_DS6			0x0005
#define SPI_SPDSR_DS7			0x0006
#define SPI_SPDSR_DS8			0x0007
#define SPI_SPDSR_DS9			0x0008
#define SPI_SPDSR_DS10			0x0009
#define SPI_SPDSR_DS11			0x000a
#define SPI_SPDSR_DS12			0x000b
#define SPI_SPDSR_DS13			0x000c
#define SPI_SPDSR_DS14			0x000d
#define SPI_SPDSR_DS15			0x000e
#define SPI_SPDSR_DS16			0x000f


/*******************************************************************************
	Bit names of older SDK version, maintained here for backward compatibility.
*******************************************************************************/

/* Do not use for new development ! */
#define SPI_SPE		SPI_SPSCR_SPE
#define SPI_SPTIE	SPI_SPSCR_SPTIE
#define SPI_CPHA	SPI_SPSCR_CPHA
#define SPI_CPOL	SPI_SPSCR_CPOL
#define SPI_SPMSTR	SPI_SPSCR_SPMSTR
#define SPI_SPRIE	SPI_SPSCR_SPRIE
#define SPI_MODFEN	SPI_SPSCR_MODFEN
#define SPI_ERRIE	SPI_SPSCR_ERRIE
#define SPI_DSO		SPI_SPSCR_DSO
#define SPI_SPTE	SPI_SPSCR_SPTE
#define SPI_MODF	SPI_SPSCR_MODF
#define SPI_OVRF	SPI_SPSCR_OVRF
#define SPI_SPRF	SPI_SPSCR_SPRF


/****************************************************************************
* SPI function prototypes
*****************************************************************************/

extern void spiInit(arch_sSPI *pSpiBase);

extern void Spi0TxEmptyISR(void);
extern void Spi0RxFullISR(void);
extern void write_SPI_0_NON_BLOCKING(const SPI0CHAR *buf, UWord16 size);
extern void read_SPI_0_NON_BLOCKING(SPI0CHAR *buf, UWord16 size);
extern void write_SPI_0_BLOCKING(const SPI0CHAR *buf, UWord16 size);
extern void read_SPI_0_BLOCKING(SPI0CHAR *buf, UWord16 size);

/***********************************************************************
* status variables for read/wrire functions
***********************************************************************/

extern volatile UWord16 spi0_status;
extern volatile UWord16 spi0_RxCounter;
extern volatile UWord16 spi0_TxCounter;

/***********************************************************************
* ioctl implementation
***********************************************************************/

/* module names for internal use in ioctl implementation */

#define __SPI_0 SPI_0

/***********************************************************************
* SPI init
***********************************************************************/

#define ioctlSPI_INIT(pSpiBase,param) spiInit(pSpiBase)

/***********************************************************************
* SPI control command macros
***********************************************************************/

/* SPI peripheral enable/disable */
#define ioctlSPI_DEVICE(pSpiBase,param) \
  if (param) periphBitSet(SPI_SPSCR_SPE,&pSpiBase->spscr); \
  else periphBitClear(SPI_SPSCR_SPE,&pSpiBase->spscr)

/* SPI MASTER/SLAVE mode set */
#define ioctlSPI_SET_MODE(pSpiBase,param) \
  if (param) periphBitSet(SPI_SPSCR_SPMSTR,&pSpiBase->spscr); \
  else periphBitClear(SPI_SPSCR_SPMSTR,&pSpiBase->spscr)

/* SPI data shift order set */
#define ioctlSPI_SET_ORDER(pSpiBase,param) \
  if (param) periphBitSet(SPI_SPSCR_DSO,&pSpiBase->spscr); \
  else periphBitClear(SPI_SPSCR_DSO,&pSpiBase->spscr)
 
/* SPI clock polarity set */
#define ioctlSPI_SET_CLOCK_POLARITY(pSpiBase,param) \
  if (param) periphBitSet(SPI_SPSCR_CPOL,&pSpiBase->spscr); \
  else periphBitClear(SPI_SPSCR_CPOL,&pSpiBase->spscr)

/* SPI clock phase set */
#define ioctlSPI_SET_CLOCK_PHASE(pSpiBase,param) \
  if (param) periphBitSet(SPI_SPSCR_CPHA,&pSpiBase->spscr); \
  else periphBitClear(SPI_SPSCR_CPHA,&pSpiBase->spscr)

/* SPI mode fault detect set */
#define ioctlSPI_SET_MODE_FAULT(pSpiBase,param) \
  if (param) periphBitSet(SPI_SPSCR_MODFEN,&pSpiBase->spscr); \
  else periphBitClear(SPI_SPSCR_MODFEN,&pSpiBase->spscr)

/* SPI receive interrupt enable/disable */
#define ioctlSPI_RX_FULL_INT(pSpiBase,param) \
  if (param) periphBitSet(SPI_SPSCR_SPRIE,&pSpiBase->spscr); \
  else periphBitClear(SPI_SPSCR_SPRIE,&pSpiBase->spscr)

/* SPI transmit interrupt enable/disable */
#define ioctlSPI_TX_EMPTY_INT(pSpiBase,param) \
  if (param) periphBitSet(SPI_SPSCR_SPTIE,&pSpiBase->spscr); \
  else periphBitClear(SPI_SPSCR_SPTIE,&pSpiBase->spscr)

/* SPI transmit interrupt enable/disable */
#define ioctlSPI_ERROR_INT(pSpiBase,param) \
  if (param) periphBitSet(SPI_SPSCR_ERRIE,&pSpiBase->spscr); \
  else periphBitClear(SPI_SPSCR_ERRIE,&pSpiBase->spscr)

#define ioctlSPI_INT_ENABLE(pSpiBase, param)		\
  periphBitSet (param,&pSpiBase->spscr)

#define ioctlSPI_INT_DISABLE(pSpiBase, param)		\
  periphBitClear (param,&pSpiBase->spscr)
  
/* set SPI baud divisor */
#define ioctlSPI_SET_BAUD_DIV(pSpiBase,param) \
  periphBitGrpRS(SPI_SPSCR_SPR_MASK, param,&pSpiBase->spscr)

/* set data size */
#define ioctlSPI_SET_TX_DATA_SIZE(pSpiBase,param) \
  periphMemWrite(((param)-1)& 0x000F,&pSpiBase->spdsr)
  
/* write SPI Tx register */
#define ioctlSPI_WRITE_DATA(pSpiBase,param) \
  periphMemWrite(param,&pSpiBase->spdtr)
#define ioctlSPI_WRITE(pSpiBase,param) /* maintained for compatibility */ \
  periphMemWrite(param,&pSpiBase->spdtr)
  
/* reads SPI Rx register */
#define ioctlSPI_READ_DATA(pSpiBase,param) \
  periphMemRead(&pSpiBase->spdrr)
#define ioctlSPI_READ(pSpiBase,param) /* maintained for compatibility */ \
  periphMemRead(&pSpiBase->spdrr)

/* test SPI Tx reg. empty bit */
#define ioctlSPI_GET_TX_EMPTY(pSpiBase,param) \
  periphBitTest(SPI_SPSCR_SPTE,&pSpiBase->spscr)

/* test SPI Rx reg. full bit */
#define ioctlSPI_GET_RX_FULL(pSpiBase,param) \
  periphBitTest(SPI_SPSCR_SPRF,&pSpiBase->spscr)

/* test SPI overflow bit */
#define ioctlSPI_GET_RX_OVERFLOW(pSpiBase,param) \
  periphBitTest(SPI_SPSCR_OVRF,&pSpiBase->spscr)
  
/* test SPI mode fault occurence bit */
#define ioctlSPI_GET_MODE_FAULT(pSpiBase,param) \
  periphBitTest(SPI_SPSCR_MODF,&pSpiBase->spscr)

/* test SPI two error bits */
#define ioctlSPI_GET_ERROR(pSpiBase,param) \
  periphBitTest(SPI_SPSCR_MODF | SPI_SPSCR_OVRF,&pSpiBase->spscr)

/* test SPI overflow error bit */
#define ioctlSPI_ERROR_IS_OVERFLOW(pSpiBase,param) \
  periphBitTest(SPI_SPSCR_OVRF,&pSpiBase->spscr)

/* test SPI mode fault error bit */
#define ioctlSPI_ERROR_IS_MODEFAULT(pSpiBase,param) \
  periphBitTest(SPI_SPSCR_MODF,&pSpiBase->spscr)

/* clear SPI mode fault occurence bit - cleared by writing "1" to MODF */
/* note: it is the only clear-by-write-one bit in SPSCR, we don't need to use periphSafeAckByOne() */
#define ioctlSPI_CLEAR_MODE_FAULT(pSpiBase,param) \
  periphBitSet(SPI_SPSCR_MODF,&pSpiBase->spscr)

/***********************************************************************
* End-of-life commands
***********************************************************************/

#define ioctlSPI_SET_DATA_SIZE(pSpiBase,param) \
  periphMemWrite(((param)-1)& 0x000F,&pSpiBase->spdsr)

#define ioctlSPI_INT_SELECT(pSpiBase, param)		\
  periphBitGrpRS (SPI_SPSCR_SPTIE | SPI_SPSCR_SPRIE | SPI_SPSCR_ERRIE, param,&pSpiBase->spscr)


/***********************************************************************
* SPI_CLEAR_EXCEPTION
***********************************************************************/

#define ioctlSPI_CLEAR_EXCEPTION(pSpiBase,param)  \
  periphBitClear(SPI_STATUS_EXCEPTION_EXIST | SPI_EXCEPTION_BUFFER_OVERFLOW |\
          SPI_EXCEPTION_OVERRUN | SPI_EXCEPTION_MODE_FAULT, &spi0_status)

/***********************************************************************
* SPI_GET_STATUS
***********************************************************************/

#define ioctlSPI_GET_STATUS(pSpiBase,param) periphMemRead(&spi0_status);

/***********************************************************************
* SPI_WRITE_CANCEL
***********************************************************************/

#define ioctlSPI_WRITE_CANCEL(pSpiBase,param)  {\
    periphBitClear(SPI_SPSCR_SPTIE,&pSpiBase->spscr); \
    periphMemWrite(0x0000, &spi0_TxCounter); \
    periphBitClear(SPI_STATUS_WRITE_INPROGRESS, &spi0_status);}
    
/***********************************************************************
* SPI_READ_CANCEL
***********************************************************************/

#define ioctlSPI_READ_CANCEL(pSpiBase,param)   {\
    periphBitClear(SPI_SPSCR_SPRIE,&pSpiBase->spscr); \
    periphMemWrite(0x0000, &spi0_RxCounter); \
	periphBitClear(SPI_STATUS_READ_INPROGRESS, &spi0_status);}


#ifdef __cplusplus
}
#endif

#endif
