/******************************************************************************
*
* Motorola Inc.
* (c) Copyright 2001 Motorola, Inc.
* ALL RIGHTS RESERVED.
*
*******************************************************************************
*
* FILE NAME: pcmaster.c   
*
* DESCRIPTION: Source file for PC Master
*
* MODULES INCLUDED: string.h
*					types.h
*					arch.h
*					periph.h
*					sci.h
*					itcn.h
*					pcmaster.h
*
* NOTES:
*
*******************************************************************************/

#include <string.h>

#include "types.h"
#include "arch.h"
#include "periph.h"

#include "appconfig.h"
#include "sci.h"
#include "pcmaster.h"

/*-----------------------------------------------------------------------------
	TYPES	
  ---------------------------------------------------------------------------*/
  
/* recorder settings structure (recorder settings and 
temporary variables are stored in this structure) */
typedef struct{
	UWord16 trgMode;		/* 0x00 - manual, 0x01 - rising edge, 
											  0x02 - falling edge */
	UWord16 totalSmps;		/* number of samples required */
	UWord16 postTrigger;	/* samples after trigger */
	UWord16 timeDiv;		/* time div */
	UWord16 trgVarAddr;		/* address of trigger variable */
	UWord16 trgVarSize;		/* size of variable (bytes) */
	UWord16 trgVarSigned;	/* 0x00 - unsigned, 0x01 - signed */
	
	union{						
	/* union is used to access various types of treshold */
		UWord16	uw;
		UWord32	ud;
		Word16	sw;
		Word32	sd;
	} trgTreshold;		/* trigger treshold */
	
	UWord16 varCnt;	    /* number of variables */
	struct{
		UWord16 varSize;		/* size of variable */
		UWord16 varAddr;		/* address of variable */
	} varDef[8];
	
	/* position in recorder buffer - position of the next samples 
	(incremented with RecSetLen) */
	UWord16 recPos;		
	/* length of required set of variables (in words) */
	UWord16 recSetLen;	
	/* position to end of buffer (the variable is decremented after trigger 
	and recorder stops at 0) */
	UWord16 recToEnd;
	/* time div of Recorder */
	UWord16 recTime;

	/* recorder last value (last value the triggering variable 
	is stored after launch of Recorder routine)  */
	union{
		UWord16	uw;
		UWord32	ud;
		Word16	sw;
		Word32	sd;
	} recLastVal;				/* last value of synchronizing variable */
} sRecorder;

/* scope settings buffer (scope settings are stored in this structure) */
typedef struct{
UWord16	varCnt;
struct{
	UWord16 varSize;			/* size of scope variable */
	UWord16 varAddr;			/* address of scope variable */
	} varDef[8];				/* maximum number of variables is 8 */
} sScope;

/*-----------------------------------------------------------------------------
	CONSTANTS	
  ---------------------------------------------------------------------------*/

/* call user application command status */
/* no application command called (board after reset) */
#define PCMASTER_APPCMD_NOCMD		0xFF	
/* application command not finished */
#define PCMASTER_APPCMD_RUNNING	0xFE	

/* status byte masks (used with 'status' variable) */
/* receiving started, beginning of message already detected ('+') */
#define ST_STARTED				0x0010		
/* last received char was '+' */
#define ST_ST_CHAR_REC			0x0020		
/* received message is standard command (for length decoding) */
#define ST_STD_CMD				0x0040		
/* recorder is running (trigger already detected) */
#define ST_REC_RUNNING			0x0100		
/* recorder is activated (waiting for trigger) */
#define ST_REC_ACTIVATED        0x0200		
/* read pretrigger samples before waiting for trigger */
#define ST_REC_READ_PRETRIG	    0x0400		
/* response is being send to PC */
#define ST_SENDING				0x1000		
/* last sent char was '+' */
#define ST_ST_SENT				0x2000		
/* checksum already added to the message */
#define ST_CS_ADDED				0x4000		

#define START 	'+'			/* start of message */

/* recorder configuration */
#define REC_WORD_UNSIGNED       0x0004
#define REC_WORD_SIGNED         0x0005
#define REC_DWORD_UNSIGNED      0x0008
#define REC_DWORD_SIGNED        0x0009

/*-----------------------------------------------------------------------------
	MACROS	
  ---------------------------------------------------------------------------*/

/* save data for response to PC */
#define respPrepare(sts, len) 	{response.status = sts;	\
								 response.length = len;}	

/*-----------------------------------------------------------------------------
	GLOBAL VARIABLES	
  ---------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------
	STATIC VARIABLES	
  ---------------------------------------------------------------------------*/

/* input/output communication buffer */
static UWord16 dataBuff[PCMASTER_BUFFER_SIZE + 1];  

#ifdef PCMASTER_INCLUDE_CMD_SCOPE
  /* scope config data */
  static sScope scope;		
#endif

#ifdef PCMASTER_INCLUDE_CMD_RECORDER
  /* recorder config and temp data */
  static sRecorder recorder;	
  /* recorder buffer */
  static UWord16 recorderBuff[PCMASTER_REC_BUFF_LEN];
#endif

#ifdef PCMASTER_INCLUDE_CMD_APPCMD
  /* application command call status */
  static unsigned int appCmdSts;
  /* application command data buffer */
  extern UWord16 appCmdBuff[];
#endif

static unsigned int status;					/* status word of receiver */

/* currently read input char (it contains checksum at the end of message) */
static unsigned int inChar;		
/* position in buffer (0,1,2,...) */
static unsigned int pos;		
/* length of data in a message used for receiving and transmitting 
												(includes checksum) */
static unsigned int length;		

/* variable for checksum accumulation */
static unsigned int checkSum;					

/* response structure type (this structure is used to save 
	the parameters of response to be sent to PC) */
typedef struct{
	unsigned int status;		/* status byte of response */
	unsigned int length;		/* length of the whole response */
} sResponse;

static sResponse response;		/* variable with response data */
static UWord16 recPretrigCount; /* recorder pretrigger counter */

/*-----------------------------------------------------------------------------
	STATIC FUNCTIONS	
  ---------------------------------------------------------------------------*/

/* send one byte from output buffer */
static 		void inline	sendBuffer(void);								
/* prepare data for transmitting 
(response.status -> status code, response.len -> length of data) */
static 		void		sendResponse(sResponse *resp) ;					
/* sample recorder data */
static asm 	void 		readSample( UWord16 *Addr, UWord16 *DestAddr);			
/* decoding of incoming message after the last byte was received */
static 		void        messageDecode(void);		/* PC MASTER of Level 1 */
/* routine callled after filtering doubled SOM */
static 		void 		messageData(UWord16 startOfMessage);

/* all functions that begins with Cmd execute 
   the command and place proper data in dataBuff */
   
/* command - read block of memory from address *Addr to *DestAddr */
static asm 	void 		cmdReadMem(UWord16 *Addr, UWord16 *DestAddr);			
/* command - get info */
static void 	 inline	cmdGetInfo(void);								
/* command - get brief info */
static 		void inline	cmdGetInfoBrief(void);								
/* command - write block of memory */
static asm 	void inline	cmdWriteMem(UWord16 *Addr, UWord16 *Data);				
/* command - write block of memory with mask */
static asm 	void inline	cmdWriteMemMask(UWord16 *AddrData, UWord16 *AddrMask);	
/* command - scope setup */
static asm 		 inline	cmdScopeInit(UWord16 *Addr,UWord16 *DestAddr);			
/* command - read scope data */
static asm 	void inline	cmdReadScope(UWord16 *Addr, UWord16 *DestAddr);			
/* command - call application command */
static asm 	void inline	cmdCallAppCmd(UWord16 *Addr, UWord16 *Data);			
/* command - recorder setup */
static asm 	void inline	cmdRecInit(UWord16 *Addr, UWord16 *DestAddr);			

/*-----------------------------------------------------------------------------
	FUNCTION IMPLEMENTATIONS
  ---------------------------------------------------------------------------*/

Word16 pcmasterInit(void)
{
	status=0;						/* reset receiver */

	#ifdef PCMASTER_INCLUDE_CMD_SCOPE
	  scope.varCnt=0;			/* reset scope */
	#endif

	#ifdef PCMASTER_INCLUDE_CMD_RECORDER
	  recorder.varCnt=0;		/* reset recorder */
	#endif
	
	#ifdef PCMASTER_INCLUDE_CMD_APPCMD
	  /* initialize application command status */
	  appCmdSts = PCMASTER_APPCMD_NOCMD; 
	#endif
	
	return(0);
}

/*------------------------------------------------------------------------------*/

#ifdef PCMASTER_INCLUDE_CMD_APPCMD
UWord16 pcmasterGetAppCmdSts(void)
/* return state of application command call */
{
	if (appCmdSts == PCMASTER_APPCMD_RUNNING)
		return(1);		/* new application command received */
	else
		return(0);		/* no new application command */
}
#endif

/*------------------------------------------------------------------------------*/

#ifdef PCMASTER_INCLUDE_CMD_APPCMD
Word16 pcmasterWriteAppCmdSts(UWord16 state)
/* write state of application command call */
{
	appCmdSts = state;
	return(0);			/* OK */
}
#endif

/*------------------------------------------------------------------------------*/
#undef add

static asm void cmdReadMem(UWord16 *Addr, UWord16 *DestAddr)
/* read 16bit data from 16bit address and send it to PC 
	(length is length of data block) 
 Addr - address of address 
 DestAddr - start address of data in buffer 
 using length */
{
	move x:(r2)+,y1				/* low byte of address */
	move x:(r2)+,y0				/* high byte of address */
	move #8,x0					/* shifting value */
	lsll y0,x0,y0				/* high byte shifted in Y0 */
	add y0,y1					/* complete address */

	/* read data */
	move y1,r2					/* initialize R2 register */
	move x0,y0					/* shifting value */

	move length,x0				/* use length of message */
	lsr x0

	do x0,EndRead
	move x:(r2)+,a1				/* read the requested variable */
	
	/* disassemble in two bytes */
	move a1,a0
	bfclr #0xff00,a0			/* low byte -> a0 */
	move a0,x:(r3)+				/* write low byte into memory */
	bfclr #0x00ff,a1			/* high byte -> a1 */
	lsrr a1,y0,a
	move a1,x:(r3)+				/* write high byte into memory */

EndRead:
	rts
}	

/*------------------------------------------------------------------------------*/

static asm cmdScopeInit(UWord16 *Addr,UWord16 *DestAddr)
/* initialize scope/recorder (copy data from dataBuff to Scope or 
	Recorder structure)
 Addr - address of data in input buffer
 DestAddr - address of Scope/Recorder buffer with settings */
{
	move x:(r2)+,lc				/* number of scope channels */
	move lc,x:(r3)+				/* write number of channels */
	move #8,x0					/* shifting value */

	do lc,EndDo
	move x:(r2)+,a1				/* size of scope variable */
	move a1,x:(r3)+				/* write size of scope variable */
	move x:(r2)+,y1				/* low byte of address */
	move x:(r2)+,y0				/* high byte of address */
	lsll y0,x0,y0				/* high byte shifted in Y0 */
	add y1,y0					/* complete address */
	move y0,x:(r3)+				/* write address */
EndDo:

	rts
}

/*------------------------------------------------------------------------------*/

static asm void cmdReadScope(UWord16 *Addr, UWord16 *DestAddr)
/* read scope data (data will be copied to dataBuff, 
					length is computed during copying and will contain 
					number of bytes of scope data)
 Addr - address of scope settings buffer
 DestAddr - address of io buffer
 modifying length */
{
	move x:(r2)+,lc				/* number of channels */
	clr y1 

	do lc,EndChannel			/* loop for channels */
	move x:(r2)+,x0				/* read length of variable */
	add x0,y1					/* compute length */
	lsr x0						/* number of words */
	move x:(r2)+,r1				/* address of scope variable */
	move #8,y0					/* shifting value */
	push x0						/* loop counter */

Word:
	move x:(r1)+,a1				/* read scope variable */
	move a1,a0
	bfclr #0x00ff,a1			/* high byte of scope variable -> a1 */
	bfclr #0xff00,a0			/* low byte of scope variable -> a0 */
	move a0,x:(r3)+				/* low byte */
	lsrr a1,y0,a
	move a1,x:(r3)+				/* high byte */

	pop a						/* loop for all variables */
	dec a
	tst a
	push a
	bgt Word
	pop a
	nop							/* needed for correct DO execution */

EndChannel:

	inc y1
	move y1,length				/* store length */
	rts
}

/*------------------------------------------------------------------------------*/

static asm void cmdWriteMem(UWord16 *Addr, UWord16 *Data)
/* write data in data memory
 Addr - address of address
 Data - address of data to be written
 using length (number of bytes to be written) */
{
	move x:(r2)+,y1				/* low byte */
	move x:(r2)+,y0				/* high byte */
	move #8,x0
	lsll y0,x0,y0				/* high byte shifted in Y0 */
	add y0,y1					/* complete address */
	move y1,r2					/* use R2 */

	move length,y0
	lsr y0

	do y0,EndDo
	move x:(r3)+,y0				/* low byte of data */
	move x:(r3)+,y1				/* high byte of data */
	lsll y1,x0,y1				/* shift high byte */
	add y1,y0					/* complete data */
	move y0,x:(r2)+				/* write data to destination */
EndDo:
	rts
}

/*------------------------------------------------------------------------------*/

static asm void cmdWriteMemMask(UWord16 *AddrData, UWord16 *AddrMask)
/* write data in data memory using mask
 Addr - address of address
 Data - address of data to be written
 using length (number of bytes to be written) */
{
	move x:(r2)+,y1				/* low byte */
	move x:(r2)+,y0				/* high byte */
	move #8,x0
	lsll y0,x0,y0				/* high byte shifted in Y0 */
	add y0,y1					/* complete address */
	move y1,r1					/* use R1 */

	move length,y0
	lsr y0

	do y0,EndDo
	move x:(r2)+,y0				/* low byte of data */
	move x:(r2)+,y1				/* high byte of data */
	lsll y1,x0,y1				/* shift upper byte */
	add y1,y0					/* complete data -> Y0 */
	move y0,a1					/* store data -> A1 */
	move x:(r3)+,y0				/* low byte of mask */
	move x:(r3)+,y1				/* high byte of mask */
	lsll y1,x0,y1				/* shift high byte */
	add y1,y0					/* complete mask -> Y0 */
	move y0,b1					/* store mask -> B1 */
	and a1,y0					/* data & mask -> Y0 */
	neg b						/* negate mask */
	move b1,y1
	
	move x:(r1),a1				/* read memory -> A1 */
	and a1,y1					/* origdata & neg(mask) -> Y1 */
	or y1,y0					/* (data & mask) or (origdata & neg(mask)) -> Y0 */
	move y0,x:(r1)+				/* write back */

EndDo:
	rts
}

/*------------------------------------------------------------------------------*/

static asm void cmdCallAppCmd(UWord16 *Addr, UWord16 *Data)
/* write data in application command buffer
 Addr - address of beginning of Application Command data in the message (dataBuff)
 Data - address of Application Command buffer */
{
	move x:(r2)+,a				/* length */
	lsr a

	jeq NoData					/* jump if command contains no data */

	move x:(r2)+,x0
	move x0,x:(r3)+				/* write application command code */
	
	move #8,x0					/* shift value */
	
	do a,EndDo
	move x:(r2)+,y0				/* low byte of data */
	move x:(r2)+,y1				/* high byte of data */
	lsll y1,x0,y1				/* shift high byte */
	add y1,y0					/* complete data */
	move y0,x:(r3)+				/* write data to destination */
EndDo:
	rts
	
NoData:
	move x:(r2)+,x0
	move x0,x:(r3)+				/* write application command code */
	rts
}

/*------------------------------------------------------------------------------*/

static asm void cmdRecInit(UWord16 *Addr, UWord16 *DestAddr)
/* initialize recorder (copy data from dataBuff to Recorder structure)
 Addr - address of Recorder settings data in the message (dataBuff)
 DestAddr - address of Recorder structure with settings (Recorder) */
{
	/* trigger mode */
	move x:(r2)+,a
	move a,x:(r3)+			
	move #8,x0

	/*  total samples
	 	post trigger
	 	timediv
	 	trigger variable address */
	do #4,EndDo
	move x:(r2)+,y1				/* low byte */
	move x:(r2)+,y0				/* high byte */
	lsll y0,x0,y0
	add y1,y0					/* complete word */
	move y0,x:(r3)+				/* write word to memory */
	EndDo:
	
	/* trigger variable size */
	move x:(r2)+,a				/* read */
	move a,x:(r3)+				/* write to memory */

	/* trigger variable signed */
	move x:(r2)+,a				/* read */
	move a,x:(r3)+				/* write to memory */
	
	/* trigger treshold */
	do #2,EndSize
	move x:(r2)+,y1				/* low byte */
	move x:(r2)+,y0				/* high byte */
	lsll y0,x0,y0
	add y1,y0					/* complete word */
	move y0,x:(r3)+				/* write word to memory */
EndSize:
	rts
}

/*------------------------------------------------------------------------------*/

static asm void inline readSample(UWord16 *Addr, UWord16 *DestAddr)
/* read recorder data (data will be copied to dataBuff)
 Addr - address of recorder settings buffer
 DestAddr - address of recorder data buffer */
{
	move x:(r2)+,lc				/* number of channels */

	do lc,EndChannel			
	move x:(r2)+,x0
	lsr x0						/* number of words */
	move x:(r2)+,r1				/* address of scope variable */
	move #8,y0
	push x0						/* loop counter */
	Word:
	move x:(r1)+,a1
	move a1,x:(r3)+				/* write value to buffer */
	
	pop a
	dec a
	tst a
	push a
	bgt Word
	pop a
	nop
	EndChannel:

	rts
}

#define add
/*------------------------------------------------------------------------------*/

static void cmdGetInfo(void)
/* get information about hardware 
	(initialization sequence before any other communication)
 data are transfered to dataBuff */
{
	int i;				/* temporary variable for string copy */
	
	dataBuff[1] = 1;                                /* protocol version	*/
	dataBuff[2] = PCMASTER_CFG_FLAFGS;	            /* CFG_FLAFGS */
	dataBuff[3] = 2;                                /* dataBusWdt */
	dataBuff[4] = PCMASTER_GLOB_VERSION_MAJOR;      /* version */
	dataBuff[5] = PCMASTER_GLOB_VERSION_MINOR;      /* version */
	dataBuff[6] = PCMASTER_BUFFER_SIZE - 1;     /* size of input buffer 
	                                                   (without CMD, LENGTH) */
    /* size in bytes is transfered to PC */
	dataBuff[7] = (PCMASTER_REC_BUFF_LEN*2) & 0x00ff; /* size of 
	                                                     recorder buffer */
	dataBuff[8] = (PCMASTER_REC_BUFF_LEN*2) >> 8;
	dataBuff[9] = PCMASTER_RECORDER_TIME_BASE & 0x00ff; /* period of Recorder 
                                                           routine launch */
	dataBuff[10]= PCMASTER_RECORDER_TIME_BASE >> 8;
	
	strcpy((char*)&(dataBuff[11]),PCMASTER_IDT_STRING);
}

/*------------------------------------------------------------------------------*/

static void cmdGetInfoBrief(void)
/* get brief information about hardware
   (initialization sequence before any other communication)
   data are transfered to dataBuff */
{
	dataBuff[1] = PCMASTER_PROT_VER;					/* protocol version	*/
	dataBuff[2] = PCMASTER_CFG_FLAFGS;				/* CFG_FLAFGS */
	dataBuff[3] = PCMASTER_DATABUSWDT;				/* dataBusWdt */
	dataBuff[4] = PCMASTER_GLOB_VERSION_MAJOR; 		/* version */
	dataBuff[5] = PCMASTER_GLOB_VERSION_MINOR;		/* version */
	
	/* size of input buffer	(without CMD, LENGTH) */
	dataBuff[6] = PCMASTER_BUFFER_SIZE - 1;	
}

/*------------------------------------------------------------------------------*/

static void sendBuffer(void)
/* sends '+', data of message (dataBuff) and checksum (dataBuff)
 using length */
{
	if (pos <= length)						/* is it end of message ? */
	{
		inChar = ioctl(SCI_0, SCI_GET_STATUS_REG, NULL);
		ioctl(SCI_0, SCI_WRITE_DATA, dataBuff[pos]);
			
		if (dataBuff[pos] != '+') 		/* current character is not '+' */
		{
			checkSum += dataBuff[pos];	/* accumulate checksum */
			pos++;
		}
		else 								/* current character is '+' */
		{
			if (status & ST_ST_SENT)		/* the last sent char was '+' */
			{
				periphBitClear(ST_ST_SENT,&status);
				checkSum += dataBuff[pos];	/* accumulate checksum */
				pos++;
			}
			else							/* the last sent byte was not '+' */
			{
				periphBitSet(ST_ST_SENT,&status);	
			}
		}
		
		if ((pos == length) && !(status & ST_CS_ADDED))		/* the last byte 
								before cs was sent, now add the checksum */
		{
			checkSum = (-checkSum) & 0x00FF;	/* compute checksum */
			dataBuff[pos] = checkSum;
			periphBitSet(ST_CS_ADDED,&status);			/* set flag */
		}
	}
	else		/* end of transmitting */
	{
		ioctl(SCI_0, SCI_TX_EMPTY_INT, SCI_DISABLE); 	/* disable Tx Empty Interrupt */
		ioctl(SCI_0, SCI_RX_FULL_INT, SCI_ENABLE);  	/* enable Rx Full Interrupt */

		periphBitClear(ST_SENDING | ST_CS_ADDED,&status);		/* reset transmitter 
												(switch to receiving mode) */
	}
}

/*------------------------------------------------------------------------------*/

static void sendResponse(sResponse *resp) 
/* put all data to dataBuff, computes a checksum and transmit '+' as 
	beginning of message */
{
	periphBitSet(ST_SENDING,&status);			/* set flag */
	dataBuff[0]=resp->status;		/* status of trasmitted message */
	length=resp->length;				/* length of message */
	pos=0;								/* position in the message */
	
	/* send start of message */
	inChar = ioctl(SCI_0, SCI_GET_STATUS_REG, NULL);
	
	ioctl(SCI_0, SCI_WRITE_DATA, '+');          /* write SOB character */
	ioctl(SCI_0, SCI_TX_EMPTY_INT, SCI_ENABLE); /* enable Tx Empty Interrupt */

	checkSum = 0;						/* reset checksum */
}

/*------------------------------------------------------------------------------*/

static void messageDecode(void)
/* decoding of received message
 proper command will be executed and response sent to PC */
{
int *varaddr;
unsigned int i;

	switch(dataBuff[0])
	{
		/* -------------------------
		    special format commands
		   ------------------------- */
		case PCMASTER_CMD_READVAR16:					/* read 16-bit variable */
		{
			length=2;
			/* execute command */
			cmdReadMem(&dataBuff[1],&dataBuff[1]);	
			respPrepare(PCMASTER_STC_OK,3);			/* OK */
		}break;
		case PCMASTER_CMD_READVAR32:					/* read 32-bit variable */
		{
			length=4;
			/* execute command */
			cmdReadMem(&dataBuff[1],&dataBuff[1]);	
			respPrepare(PCMASTER_STC_OK,5);			/* OK */
		}break;
		#ifdef PCMASTER_INCLUDE_CMD_SCOPE
		case PCMASTER_CMD_READSCOPE:					/* read scope variables */
		{
			/* scope not configured */
			if (scope.varCnt==0)			
			{
				respPrepare(PCMASTER_STC_NOTINIT,1);	/* scope not initialized */
			}		
			else									/* scope configured */
			{
				/* execute command */
				cmdReadScope(&(scope.varCnt),	&dataBuff[1]); 
				respPrepare(PCMASTER_STC_OK,length);	/* OK */
			}
		}break;
		#endif
		#ifdef PCMASTER_INCLUDE_CMD_RECORDER
		case PCMASTER_CMD_GETRECSTS:					/* get recorder status */
		{
			if (recorder.varCnt)		/* recorder is initialized */
			{
				if (!(status & ST_REC_ACTIVATED))
				{
					respPrepare(PCMASTER_STC_RECDONE,1);	/* recorder finished */
				}
				else
				{
					respPrepare(PCMASTER_STC_RECRUN,1);	/* recorder running */
				}
			}
			else								/* recorder is not initialized */
			{
				respPrepare(PCMASTER_STC_NOTINIT,1);	/* recorder not initialized */
			}
		}break;
		case PCMASTER_CMD_STARTREC:					/* start recorder */
		{
			if (recorder.varCnt)			/* recorder initialized */
			{
				if (!(status & ST_REC_RUNNING))		/* recorder not running */
				{
					periphBitClear(ST_REC_RUNNING,&status);		/* stop recorder if it 
																	is running */
					/* initialize time div */
					(recorder.recTime) = (unsigned)(recorder.timeDiv);	
					/* address of triggering variable */
					varaddr=(int *)(recorder.trgVarAddr);	
					switch (status & 0x000f)			
					/* initialize last rcorder value 
						with current value of triggering variable */
					{
						case REC_WORD_UNSIGNED: recorder.recLastVal.uw=(unsigned int)*varaddr; break;
						case REC_WORD_SIGNED: recorder.recLastVal.sw=(int)*varaddr; break;
						case REC_DWORD_UNSIGNED: recorder.recLastVal.ud=(unsigned long)*varaddr; break;
						case REC_DWORD_SIGNED: recorder.recLastVal.sd=(long)*varaddr; break;
					}
					/* activate recorder to wait for trigger */
					periphBitSet(ST_REC_ACTIVATED, &status);
					respPrepare(PCMASTER_STC_OK,1);	/* OK */
				}
				else
				{
					respPrepare(PCMASTER_STC_RECRUN,1);	/* recorder running */
				}
			}
			else								/* recorder is not initialized */
			{
				respPrepare(PCMASTER_STC_NOTINIT,1);	/* recorder not initialized */
			}
		}break;
		case PCMASTER_CMD_STOPREC:					/* stop recorder */
		{
			if (recorder.varCnt)			/* recorder initialized */
			{
				if (status & ST_REC_ACTIVATED)		/* recorder activated */
				{	
					if (status & ST_REC_RUNNING)		/* recorder running */
					{
						respPrepare(PCMASTER_STC_RECDONE,1);	/* recorder finished */
					}
					else								/* recorder not running */
					{	
						/* initialize posttrigger value */
						(recorder.recToEnd) = (recorder.postTrigger);	
						periphBitSet(ST_REC_RUNNING,&status);	/* manually trigger 
																the recorder */
						respPrepare(PCMASTER_STC_OK,1);	/* OK */
					}
				}
				else								/* recorder not activated */
				{
					respPrepare(PCMASTER_STC_RECDONE,1);	/* recorder finished */
				}
			}						
			else									/* recorder not initialized */
			{
				respPrepare(PCMASTER_STC_NOTINIT,1);
			}
		}break;				
		case PCMASTER_CMD_GETRECBUFF:					/* get recorder buffer */
		{
			if ((recorder.varCnt) != 0)		/* recorder initialized */
			{
				if (!(status & ST_REC_RUNNING))		/* recorder not running */
				{
					/* recorder buffer address */
					dataBuff[1]=(unsigned int)(recorderBuff) & 0x00FF;	
					
					/* recorder buffer address */
					dataBuff[2]=(unsigned int)(recorderBuff) >> 8;		
					i=(recorder.recPos)\
						/(recorder.recSetLen);
					
					/* position of the last sample in recorder buffer */
					dataBuff[3]=(i) & 0x00FF;		
					
					/* position of the last sample in recorder buffer */
					dataBuff[4]=(i) >> 8;			
					
					respPrepare(PCMASTER_STC_OK,5);	/* OK */
				}
				else								/* recorder running */
				{
					respPrepare(PCMASTER_STC_SERVBUSY,1);	/* service busy */
				}
			} else									/* recorder initialized */
			{
				respPrepare(PCMASTER_STC_NOTINIT,1);	/* recorder not initialized */
			}
		}break;
		#endif
		#ifdef PCMASTER_INCLUDE_CMD_APPCMD
		case PCMASTER_CMD_GETAPPCMDSTS:	/* get user application	command call status */
		{
			dataBuff[1]=appCmdSts;		/* copy status byte in the output buffer */
			respPrepare(PCMASTER_STC_OK,2);			/* OK */
		}break;
		#endif
		#ifndef PCMASTER_INCLUDE_CMD_GETINFO_BRIEF
		case PCMASTER_CMD_GETINFO:					/* get info about hardware */
		{
			cmdGetInfo();							/* execute the command */
			
			respPrepare(PCMASTER_STC_OK,(1 + 10 + PCMASTER_IDT_STRING_LEN)); /* OK */
		}break;
		#endif
		#ifdef PCMASTER_INCLUDE_CMD_GETINFO_BRIEF
		case PCMASTER_CMD_GETINFOBRIEF:		/* get brief info about hardware */
		{
			cmdGetInfoBrief();						/* execute the command */
			
			respPrepare(PCMASTER_STC_OK,(1 + 6)); /* OK */
		}break;
		#endif		

		/* --------------------------
		    standard format commands
		   -------------------------- */
		#ifdef PCMASTER_INCLUDE_CMD_SCOPE
		case PCMASTER_CMD_SETUPSCOPE:					/* setup scope */
		{
			if ( (dataBuff[2]==0) || (dataBuff[2] > 8) )	
			/* varCnt is zero or greater than 8 */
			{
				respPrepare(PCMASTER_STC_INVBUFF,1);	/* invalid buffer size */
				return;
			}
		
			/* init scope */
			cmdScopeInit(&dataBuff[2], &scope.varCnt);
			
			/* check size of variable */
			for(i=0 ; i<(scope.varCnt) ; i++)	
			{
			/* varSize is 0 or 1,3,5,7,... */
				if ( ((scope.varDef[i].varSize)==0) || (((scope.varDef[i].varSize)%2)!=0) )
				{
					/* invalid size of variable */
					respPrepare(PCMASTER_STC_INVSIZE,1);	
					/* reset scope */
					scope.varCnt = 0;					
					return;
				}
			}
			respPrepare(PCMASTER_STC_OK,1);
		}break;
		#endif
		#ifdef PCMASTER_INCLUDE_CMD_RECORDER
		case PCMASTER_CMD_SETUPREC:				/* setup recorder */
		{
			periphBitClear(ST_REC_RUNNING | ST_REC_ACTIVATED, &status);   /* reset the recorder */

			if ( (dataBuff[17]==0) || (dataBuff[17] > 8) )	
			/* varCnt is zero or greater than 8 */
			{
				respPrepare(PCMASTER_STC_INVBUFF,1);	/* invalid buffer size */
				return;
			}

			/* init recorder */
			cmdRecInit(&dataBuff[2],\
				&(recorder.trgMode));	

			if (recorder.trgMode)
			{
				if (!( ((recorder.trgVarSize)==2) || ((recorder.trgVarSize)==4) ) )		
				/* trigger variable size is not 2 or 4 */
				{
					respPrepare(PCMASTER_STC_INVSIZE,1);	/* invalid buffer size */
					/* reset scope */
					(recorder.varCnt) = 0;			
					return;
				}
				status = (status & 0xfff0) | (((recorder.trgVarSize) << 1) + (recorder.trgVarSigned));
			}
			
			/* init scope */
			cmdScopeInit(&dataBuff[17],&(recorder.varCnt));	
			
			/* check variable sizes */
			for(i=0 ; i<(recorder.\
				varCnt) ; i++)	
			{
				if ( ((recorder.\
					varDef[i].varSize)==0) || (((recorder.varDef[i].varSize)%2)!=0) )	
				/* varSize is 0 or 1,3,5,7,... */
				{
					respPrepare(PCMASTER_STC_INVSIZE,1); /* invalid buffer size */
					/* reset scope */
					recorder.varCnt=0;					
					return;
				}
			}
		
			recorder.recTime = recorder.timeDiv;
			recorder.recToEnd=1;
			recorder.recPos=0;
						
			recorder.recSetLen=0;							
			for (i=0 ; i < recorder.varCnt ; i++)	
			/* check if recorder buffer size is valid */
			{
				/* length of one set of samples (in bytes) */
				recorder.recSetLen += recorder.varDef[i].varSize;			
			}
			/* length (in words) */
			recorder.recSetLen = (recorder.recSetLen) >> 1;	
			/* buffer length required (in words) */
			recorder.totalSmps = recorder.totalSmps * recorder.recSetLen;	
			
			if ((recorder.totalSmps) <= PCMASTER_REC_BUFF_LEN)		
			/* buffer length required is smaller than recorder buffer */
			{
				varaddr=(int *)recorder.trgVarAddr;
				switch (status & 0x000f)
				{
					/* unsigned word */
					case REC_WORD_UNSIGNED: recorder.recLastVal.uw=(unsigned int)*varaddr; break;	
					/* signed word */
					case REC_WORD_SIGNED: recorder.recLastVal.sw=(int)*varaddr; break;			
					/* unsigned double */
					case REC_DWORD_UNSIGNED: recorder.recLastVal.ud=(unsigned long)*varaddr; break;	
					/* unsigned double */
					case REC_DWORD_SIGNED: recorder.recLastVal.sd=(long)*varaddr; break;			
				}
				respPrepare(PCMASTER_STC_OK,1);
				periphBitSet(ST_REC_ACTIVATED, &status);		/* recorder activated */
			} 
			else
			/* invalid buffer size specified */
			{
				recorder.varCnt=0;
				respPrepare(PCMASTER_STC_INVBUFF,1);	/* invalid buffer size */
				break;
			}
		}break;
		#endif
		case PCMASTER_CMD_READMEM:					/* read block of memory */
		{
			length=dataBuff[2];
			
			if ((length+2) <= (PCMASTER_BUFFER_SIZE))
			/* memory block is too long */	
			{
				/* read memory */
				cmdReadMem(&dataBuff[3],&dataBuff[1]);	
				respPrepare(PCMASTER_STC_OK,(length+1));	/* OK */
			}
			else						
			/* response greater than buffer */
			{
				/* response buffer overflow */
				respPrepare(PCMASTER_STC_RSPBUFFOVF,1);	
			}
		}break;
		case PCMASTER_CMD_WRITEMEM:				/* write block of memory */
		{
			/* read length of memory block from the message */
			length=dataBuff[2];	
			
			asm{move sr,i};		/* store SR register (store IPL setting) */
			archDisableInt();	/* disable interrupts for this operation */
			/* write to memory */
			cmdWriteMem(&dataBuff[3],&dataBuff[5]);	
			asm{move i,sr};	/* restore SR register (restore IPL setting ) */
			
			respPrepare(PCMASTER_STC_OK,1);
		}break;
		case PCMASTER_CMD_WRITEMEMMASK:			/* write to memory with mask */
		{
			/* read length of memory block from the message */
			length=dataBuff[2];	
			
			asm{move sr,i};		/* store SR register (store IPL setting) */
			archDisableInt();	/* disable interrupts for this operation */
			/* write to memory */
			cmdWriteMemMask(&dataBuff[3],&dataBuff[length+3+2]); 
			asm{move i,sr};	/* restore SR register (restore IPL setting ) */
			
			respPrepare(PCMASTER_STC_OK,1);			/* OK */
		}break;	
		#ifdef PCMASTER_INCLUDE_CMD_APPCMD
		case PCMASTER_CMD_CALLAPPCMD:			/* call user application command */
		{
			if (dataBuff[1] > PCMASTER_APPCMD_BUFF_LEN)	
			/* check Application Command length */
			{
				respPrepare(PCMASTER_STC_INVBUFF,1);	/* invalid buffer size */
			}
			else
			{
				if (appCmdSts==PCMASTER_APPCMD_RUNNING)	
				/* Application Command already called */
				{
					respPrepare(PCMASTER_STC_SERVBUSY,1);	/* service is busy	 */
				}
				else					/* no Application Command was called */
				{
				/* copy Application Command data to Application Command buffer */
					cmdCallAppCmd(&dataBuff[1],appCmdBuff);	
					appCmdSts = PCMASTER_APPCMD_RUNNING;
					respPrepare(PCMASTER_STC_OK,1);	/* OK */
				}
			}
		}break;
		#endif
		default:									/* invalid command */
		{
			respPrepare(PCMASTER_STC_INVCMD,1);		/* invalid command */
		}break; 
	}
}

/*------------------------------------------------------------------------------*/

#ifdef PCMASTER_INCLUDE_CMD_RECORDER
void pcmasterRecorder(void)
{
int *addr;
union{
	unsigned int	uw;
	unsigned long	ud;
	signed int 		sw;
	signed long		sd;
} actual;

	if (status & ST_REC_ACTIVATED)	/* recorder activated */
	{
		if (recorder.recTime == recorder.timeDiv)
		/* now is the right time to make samples */
		{

            if (!(ST_REC_READ_PRETRIG & status))
            {
                if (recPretrigCount >= (recorder.totalSmps - 
                            (recorder.recSetLen * recorder.postTrigger))) 
                    periphBitSet(ST_REC_READ_PRETRIG, &status);
                    else recPretrigCount += recorder.recSetLen;
            }
		    
			switch (status & 0x000f)
			{
			case REC_WORD_UNSIGNED:
			/* size=2, unsigned */
			{
				addr=(int *)recorder.trgVarAddr;
				actual.uw=(unsigned int)*addr;			/* read actual value */
				
				/* read new samples */
				readSample(&recorder.varCnt, &recorderBuff[recorder.recPos]);	
				recorder.recPos += recorder.recSetLen;
				
				/* wrap around */
				if (recorder.recPos >= recorder.totalSmps) recorder.recPos=0;	

				if ( (!(status & ST_REC_RUNNING)) && (status & ST_REC_READ_PRETRIG) )
				{
					if ((recorder.trgMode	== PCMASTER_REC_TRIGRIS) && \
						(actual.uw >= recorder.trgTreshold.uw) && \
						(recorder.recLastVal.uw <	recorder.trgTreshold.uw))
					/* recorder is configured to rising edge
					   actual value greater than treshold
					   last value smaller than treshold */
					{
 						periphBitSet(ST_REC_RUNNING, &status);				
   						recorder.recToEnd = recorder.postTrigger-1;
					}
					else if ((recorder.trgMode == PCMASTER_REC_TRIGFAL) && \
							(actual.uw <= recorder.trgTreshold.uw) && \
							(recorder.recLastVal.uw > recorder.trgTreshold.uw))
					/* recorder is configured to falling edge
					   actual value smaller than treshold
					   last value greater than treshold */
					{
   						periphBitSet(ST_REC_RUNNING, &status);				
   						recorder.recToEnd = recorder.postTrigger-1;
					}
				}
				else  if(status & ST_REC_READ_PRETRIG) recorder.recToEnd--;
			
				if (recorder.recToEnd==0)
				/* buffer is full */
				{
					periphBitClear(ST_REC_RUNNING | ST_REC_ACTIVATED | \
					                ST_REC_READ_PRETRIG, &status);		
					recorder.recToEnd=1;
				}
				/* remember last value */
				recorder.recLastVal.uw = actual.uw;	
				/* reset recorder time */
				recorder.recTime = 0;		
			}break; 
			
			case REC_WORD_SIGNED:
			/* size=2, signed */			
			{
				addr=(int *)recorder.trgVarAddr;
				actual.sw=(int)*addr;					/* read actual value */
				
				/* read new samples */
				readSample(&recorder.varCnt, &recorderBuff[recorder.recPos]);	
				recorder.recPos += recorder.recSetLen;
				/* wrap around */
				if (recorder.recPos >= recorder.totalSmps) recorder.recPos=0;	
									
				if ( (!(status & ST_REC_RUNNING)) && (status & ST_REC_READ_PRETRIG) )
				{
					if ((recorder.trgMode == PCMASTER_REC_TRIGRIS) && \
						(actual.sw >= recorder.trgTreshold.sw) && \
						(recorder.recLastVal.sw < recorder.trgTreshold.sw))
					/* recorder is configured to rising edge
					   actual value greater than treshold
					   last value smaller than treshold */
					{
   						periphBitSet(ST_REC_RUNNING, &status);				
   						recorder.recToEnd = recorder.postTrigger-1;
					}
					else if ((recorder.trgMode == PCMASTER_REC_TRIGFAL) && \
							(actual.sw <= recorder.trgTreshold.sw) && \
							(recorder.recLastVal.sw > recorder.trgTreshold.sw))
					/* recorder is configured to falling edge
					   actual value smaller than treshold
					   last value greater than treshold */
					{
   						periphBitSet(ST_REC_RUNNING, &status);				
   						recorder.recToEnd = recorder.postTrigger-1;
					}
				}
				else if(status & ST_REC_READ_PRETRIG) recorder.recToEnd--;
			
				if (recorder.recToEnd==0)
				/* buffer is full */
				{
					periphBitClear(ST_REC_RUNNING | ST_REC_ACTIVATED | \
					                ST_REC_READ_PRETRIG, &status);			
					recorder.recToEnd=1;
				}
				/* remember last value */
				recorder.recLastVal.sw=actual.sw;	
				/* reset recorder time */
				recorder.recTime=0;		
			}break;
			
			case REC_DWORD_UNSIGNED:
			/* size=4, unsigned	*/
			{
				addr=(int *)recorder.trgVarAddr;
				actual.ud=(unsigned long)*addr;			/* read actual value */
				
				/* read new samples */
				readSample(&recorder.varCnt, &recorderBuff[recorder.recPos]);		

				recorder.recPos += recorder.recSetLen;
				/* wrap around */
				if (recorder.recPos >= recorder.totalSmps) recorder.recPos=0;		
									
				if ( (!(status & ST_REC_RUNNING)) && (status & ST_REC_READ_PRETRIG) )
				{
					if ((recorder.trgMode == PCMASTER_REC_TRIGRIS) && \
						(actual.ud >= recorder.trgTreshold.ud) && \
						 (recorder.recLastVal.ud < recorder.trgTreshold.ud))
					/* recorder is configured to rising edge
					   actual value greater than treshold
					   last value smaller than treshold */
					{
   						periphBitSet(ST_REC_RUNNING, &status);				
   						recorder.recToEnd = recorder.postTrigger-1;
					}
					else if ((recorder.trgMode == PCMASTER_REC_TRIGFAL) && \
							(actual.ud <= recorder.trgTreshold.ud) && \
							(recorder.recLastVal.ud > recorder.trgTreshold.ud))
					/* recorder is configured to falling edge
					   actual value smaller than treshold
					   last value greater than treshold */
					{
   						periphBitSet(ST_REC_RUNNING, &status);				
   						recorder.recToEnd = recorder.postTrigger-1;
					}
				}
				else recorder.recToEnd--;
			
				if (recorder.recToEnd==0)
				/* buffer is full */
				{
					periphBitClear(ST_REC_RUNNING | ST_REC_ACTIVATED |\
					                ST_REC_READ_PRETRIG, &status);			
					recorder.recToEnd=1;
				}
				/* remember last value */
				recorder.recLastVal.ud=actual.ud;	
				/* reset recorder time */
				recorder.recTime=0;					
			}break;
			
			case REC_DWORD_SIGNED:
			/* size=4, signed */
			{
				addr=(int *)recorder.trgVarAddr;
				actual.sd=(long)*addr;					/* read actual value */
				
				/* read new samples */
				readSample(&recorder.varCnt, &recorderBuff[recorder.recPos]);	

				recorder.recPos += recorder.recSetLen;
				/* wrap around */
				if (recorder.recPos >= recorder.totalSmps) recorder.recPos=0;		
									
				if ( (!(status & ST_REC_RUNNING)) && (status & ST_REC_READ_PRETRIG) )
				{
					if ((recorder.trgMode == PCMASTER_REC_TRIGRIS) && \
						(actual.sd >= recorder.trgTreshold.sd) && \
						(recorder.recLastVal.sd < recorder.trgTreshold.sd))
					/* recorder is configured to rising edge
					   actual value greater than treshold
					   last value smaller than treshold */
					{
   						periphBitSet(ST_REC_RUNNING, &status);				
   						recorder.recToEnd = recorder.postTrigger-1;
					}
					else if ((recorder.trgMode == PCMASTER_REC_TRIGFAL) && \
							(actual.sd <= recorder.trgTreshold.sd) && \
							(recorder.recLastVal.sd > recorder.trgTreshold.sd))
					/* recorder is configured to falling edge
					   actual value smaller than treshold
					   last value greater than treshold */
					{
   						periphBitSet(ST_REC_RUNNING, &status);				
   						recorder.recToEnd = recorder.postTrigger-1;
					}
				}
				else recorder.recToEnd--;
			
				if (recorder.recToEnd==0)
				/* buffer is full */
				{
					periphBitClear(ST_REC_RUNNING | ST_REC_ACTIVATED | \
					                ST_REC_READ_PRETRIG, &status);			
					recorder.recToEnd=1;
				}
				/* remember last value */
				recorder.recLastVal.sd=actual.sd;	
				/* reset recorder time */
				recorder.recTime=0;		
			}break;
			case 0: /* no trigger variable specified */
			{
				/* read new samples */
				readSample(&recorder.varCnt, &recorderBuff[recorder.recPos]);	

				recorder.recPos += recorder.recSetLen;
				/* wrap around */
				if (recorder.recPos >= recorder.totalSmps) recorder.recPos=0;		
									
				if (status & ST_REC_RUNNING)
				{
				    recorder.recToEnd--;
				}
			
				if (recorder.recToEnd==0)   /* buffer is full */
				{
					periphBitClear(ST_REC_RUNNING | ST_REC_ACTIVATED | \
					                ST_REC_READ_PRETRIG, &status);			
					recorder.recToEnd = 1;
				}
				/* reset recorder time */
				recorder.recTime=0;		
			}break;
			}
		}
		/* wait for the right time */
		else recorder.recTime++;			
    }
    else /* recorder not activated */
    {
        recPretrigCount = 0;
        periphBitClear(ST_REC_READ_PRETRIG, &status);
    }
}
#endif

/*------------------------------------------------------------------------------*/

static void messageData(UWord16 startOfMessage)
{
	if (startOfMessage == 0)	/* no start of message */
	{
		if (status & ST_STARTED)	/* start of message already detected */
		{
			if (pos != length)		
			{
				/* read byte, accumulate checksum */
				dataBuff[pos] = inChar;		
				checkSum += inChar;			/* checksum accumulation */
				pos++;						/* next position in buffer */

				if (status & ST_STD_CMD)	
				/* inChar contains length of standard format message */
				{
					length = inChar + 2;	/* read length of message */
					periphBitClear(ST_STD_CMD,&status);	/* clear flag */
			
					if (length > (PCMASTER_BUFFER_SIZE))
					/* command is greater than input buffer */
					{
						/* clear flag */
						periphBitClear(ST_STARTED | ST_ST_CHAR_REC,&status);	
						/* input buffer overflow */
						respPrepare(PCMASTER_STC_CMDBUFFOVF,1);	
						sendResponse(&response);	/* send response to PC */
						/* disable Rx Full Interrupt during transmitting*/
						ioctl(SCI_0, SCI_RX_FULL_INT, SCI_DISABLE);  
					}
				}			
			}
			else	/* end of message */
			{
				checkSum += inChar;			/* accumulate checksum */
				if ((checkSum & 0x00FF) == 0) /* correct checksum */
				{
					messageDecode();
				}
				else						/* checksum error */
				{
					/* checksum error response */
					respPrepare(PCMASTER_STC_CMDSERR,1);	
				}
				
				/* clear flag */
				periphBitClear(ST_STARTED | ST_ST_CHAR_REC,&status);	
				sendResponse(&response);	/* send response to PC */
				/* disable Rx Full Interrupt during transmitting*/
				ioctl(SCI_0, SCI_RX_FULL_INT, SCI_DISABLE);  

			}
		}
	}
	else			/* start of message */
	{
		/* reset receiver, read first byte of message */
		periphBitSet(ST_STARTED,&status);			/* message receiving */
		/* read byte, start of checksum accumulating */
		checkSum = dataBuff[0] = inChar;	
		/* next position in buffer */
		pos = 1;							
		/* value sufficient for standard format commands */
		length = 2;							
		
		if (inChar >= 0xC0)		/* special format command */
		{
			length=((inChar & 0x30) >> 3) + 1;	/* length decoding */
		}
		else	
		/* standard format command (next byte will be length of the message) */
		{
			periphBitSet(ST_STD_CMD,&status);			/* wait for next character */
		}
	}
}

/*------------------------------------------------------------------------------*/

#pragma interrupt
void pcmasterRxTxIsr(void)
{
	archPushAllRegisters();
	
	/* message is transmitted */
	if (status & ST_SENDING) sendBuffer(); 	/* send data */			
	else
	{
		ioctl(SCI_0, SCI_RX_FULL_INT, SCI_DISABLE);  /* disable Rx Full Interrupt */
		archEnableInt();	/* enable interrupts to make this routine interruptible */
	
		inChar = ioctl(SCI_0, SCI_GET_STATUS_REG, NULL);
		inChar = ioctl(SCI_0, SCI_READ_DATA, NULL);		/* read received character */

		if ((status & ST_ST_CHAR_REC) == 0)	/* last byte was not '+' */
		{
			if (inChar == '+') 			/* '+' received */
			{
				periphBitSet(ST_ST_CHAR_REC,&status);
			}
			else 						/* any byte received */
			{
				messageData(0);			/* byte received */
			}
		}
		else				/* the last byte was '+' */
		{
			if (inChar == '+') 			/* doubled '+' (this is the second one) */
			{
				messageData(0);			/* byte received */
			}
			else					/* start of message */
			{
				messageData(1);		/* byte received */
			}	

				periphBitClear(ST_ST_CHAR_REC,&status);	/* clear flag */
		}
	
		ioctl(SCI_0, SCI_RX_FULL_INT, SCI_ENABLE); /* enable Rx Full Interrupt */
	}
	
 	archPopAllRegisters();
}

/*------------------------------------------------------------------------------*/

#pragma interrupt
void pcmasterRxErrorIsr(void)
{
	UWord16 pcm_tmp;

	pcm_tmp = ioctl(SCI_0, SCI_GET_STATUS_REG, NULL);
	ioctl(SCI_0, SCI_CLEAR_STATUS_REG, NULL);
}		

/*-----------------------------------------------------------------------------
	SPECIAL FEATURES FUNCTIONS
  ---------------------------------------------------------------------------*/

void pcmasterRxTxPoll(void)
{
	/* message is transmitted */
	if ((status & ST_SENDING)&&(ioctl(SCI_0, SCI_GET_TX_EMPTY, NULL))) sendBuffer(); 	/* send data */			
	else if (ioctl(SCI_0, SCI_GET_RX_FULL, NULL))
	{

		inChar = ioctl(SCI_0, SCI_GET_STATUS_REG, NULL);
		inChar = ioctl(SCI_0, SCI_READ_DATA, NULL);		/* read received character */

		if ((status & ST_ST_CHAR_REC) == 0)	/* last byte was not '+' */
		{
			if (inChar == '+') 			/* '+' received */
			{
				periphBitSet(ST_ST_CHAR_REC,&status);
			}
			else 						/* any byte received */
			{
				messageData(0);			/* byte received */
			}
		}
		else				/* the last byte was '+' */
		{
			if (inChar == '+') 			/* doubled '+' (this is the second one) */
			{
				messageData(0);			/* byte received */
			}
			else					/* start of message */
			{
				messageData(1);		/* byte received */
			}	

				periphBitClear(ST_ST_CHAR_REC,&status);	/* clear flag */
		}
	}
}

/*------------------------------------------------------------------------------*/			

#ifdef PCMASTER_INCLUDE_CMD_RECORDER
UWord16 *pcmasterGetRecBuffPtr(void) 
{
	return (recorderBuff);
}
#endif

/*------------------------------------------------------------------------------*/

void pcmasterRecBuffFull(void) 
{
	status &= ~ST_REC_ACTIVATED;
}

/*------------------------------------------------------------------------------*/

UWord16 pcmasterGetRecBuffEmpty(void) 
{
	return (status & ST_REC_ACTIVATED);
}				

/*------------------------------------------------------------------------------*/

